C    Odeidon, a program for structural identification of ODEs
C    Copyright (C) 2011 Peter Gennemark, Dag Wedelin
C
C    This program is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with this program.  If not, see <http://www.gnu.org/licenses/>.

C    Contact: Peter Gennemark, Dag Wedelin
C             www.odeidentification.org
C
C
C

C     Parameter estimation as described in Gennemark and Wedelin 2007
      subroutine alg_paraest(noiselevel,maxiter,error,debug)

      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)


      integer nexp,nvare,npts(maxexp),npts2,nvar
      double precision edata(maxexp,maxvar+1,2*maxpts) 
      double precision edata2(maxexp,maxvar+1,2*maxpts)   
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      integer spline2(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
                 
      logical useip(maxvar)     ! use ip data or sim data in simulate single


C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      

      double precision R(maxexp*maxpts),RES(maxexp*maxpts) ! residuals
      double precision R2(maxexp*maxvar*maxpts)
      double precision RES2(maxexp*maxvar*maxpts) ! residuals
      double precision X(39)
      integer N

      integer noiselevel,debug,maxiter
      double precision error

      integer ivar,iter
      double precision errcur(maxvar)
      logical var(maxvar)
      integer idummy
      logical initconc          ! T if init conc consider as parameter, ow F

      common/myinitconc/initconc
      common/simulg/var,idummy
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata2/edata2,npts2,useip
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
  
      do 10 ivar=1,nvar       
         useip(ivar)=.true.  
 10   continue

      iter=1
 1000 continue                  !LOOP
      
      if (debug.gt.0) write(*,*) 'alg_paraest, iter=',iter
      
C     Estimate one equation at a time
      do 20, ivar=1,nvar
         if (dependent(ivar)) then 
            if (iter.eq.1) then
               call estvar(ivar,errcur(ivar),1,.false.) 
               if (debug.gt.0) call printvar(maxvar,maxr,maxv,maxk
     $              ,nvar,modtype,modk,modn,modv,dependent,ivar)
            else
               call estvar(ivar,errcur(ivar),1,.true.) 
               if (debug.gt.0) call printvar(maxvar,maxr,maxv,maxk
     $              ,nvar,modtype,modk,modn,modv,dependent,ivar)
            endif
            call estvar(ivar,errcur(ivar),2,.true.) 
            if (debug.gt.0) call printvar(maxvar,maxr,maxv,maxk,nvar,
     $           modtype,modk,modn,modv,dependent,ivar)
            
            if (debug.gt.0) write(*,*) 'alg_paraest, ivar=',ivar
     $           ,' errcur(',ivar,')=',errcur(ivar)!,' useip=',useip
            
         endif
 20   continue

C     Estimate all     
      initconc=.false.
      if (noiselevel.gt.0) initconc=.true.
      if (debug.gt.0) write(*,*) 'estgeneral, initconc=',initconc
      
      call estgeneral(error,.true.) ! TRUE -> do local estimation of submodels first
      
      
      if (debug.gt.0) then
         write(*,*) 'alg_paraest, global error=',error
      endif
      
   
      if (iter.ge.maxiter) then
         return
      endif

      initconc=.false.
      if (debug.gt.0) write(*,*) 'alg_paraest, reset initconc=',initconc
      

      call readdata2(edata,edata2,spline2,
     *     nexp,nvare,npts,npts2,
     *     nvar,
     *     debug-1)             ! give printing opportunity 
      
      if (iter.eq.1) then
         do 1220 ivar=1,nvar       
            useip(ivar)=.false.  
 1220    continue
      endif
      

      iter=iter+1
      goto 1000    
      
      end
      


