C    Odeidon, a program for structural identification of ODEs
C    Copyright (C) 2011 Peter Gennemark, Dag Wedelin
C
C    This program is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with this program.  If not, see <http://www.gnu.org/licenses/>.

C    Contact: Peter Gennemark, Dag Wedelin
C             www.odeidentification.org
C
C
C
C     Calculate true error (OUT) of model by using simulate
C     res (OUT) residual (part of the error) 
      subroutine errtru(error,R,RES,N)
     
      implicit none
      integer maxexp,maxpts,maxvar,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision sdata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      
      integer nvar

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      integer ivar,istate,N
      double precision error,lambda
      
      double precision R(N),RES(N) ! residuals

      integer kn,rcount,getKsOfType,ireact,iexp,kii,i
      double precision stdev(maxexp,maxvar,maxpts) ! standard deviation
      logical debug
      double precision L,K,NS ! loglikelihood, number of parameters, number of samples

      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda

      debug=.false.
      error=0.d0
      rcount=1
      do 30 iexp=1,nexp
         if (debug) write(*,*) 'errtru, ==== Experiment ',iexp
         call simulate(nvar,
     *        modtype,modk,modkinit,modn,modv,dependent,
     *        nexp,nvare,npts(iexp),
     *        edata,sdata,spline,
     *        iexp,
     *        istate)
         if (debug) write(*,*) 'errtru, ==== sim done istate ',istate
         if (istate.lt.0.d0) then
            error=1.d300 
            do 5 kii=rcount,N
               R(kii)=1.d10
               RES(kii)=1.d10
 5          continue
            return
         endif
         do 15 ivar=1,nvar
            if (dependent(ivar)) then
               do 10 i=1,npts(iexp)   
                  R(rcount)=sdata(iexp,(ivar+1),i)/stdev(iexp,(ivar
     $                 ),i)
                  RES(rcount)=(-sdata(iexp,(ivar+1),i)+edata(iexp
     $                 ,(ivar+1),i))/stdev(iexp,ivar,i)
c$$$                  write(*,*) iexp,' ',ivar,' ',i,' ',sdata(iexp,(ivar+1)
c$$$     $                 ,i),' ',edata(iexp,(ivar+1),i),' ERR=',(error
c$$$     $                 +RES(rcount)**2)
c$$$                  if (i.ge.2) stop
                  error=error+RES(rcount)**2
                  rcount=rcount+1
 10            continue 
            endif
 15      continue
 30   continue   
      kn=0
      do 50, ivar=1,nvar
         do 40, ireact=1,modn(ivar)
            kn=kn+getKsOfType(modtype(ivar,ireact))
 40      continue
 50   continue
      L=-0.5*error
      K=kn
      NS=rcount-1
      !error=0.5*error+lambda*kn 
      error=-L+lambda*K 
      end                       ! End of errtru




C     Calculate true error (OUT) of submodel (defined in common var)
C     using simulate
C     res (OUT) residual (part of the error) 
      subroutine errvars(error,R,RES,N)
     
      implicit none
      integer maxexp,maxpts,maxvar,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision sdata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      
      integer nvar

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      integer ivar,istate,N
      double precision error,lambda
      
      double precision R(N),RES(N) ! residuals

      integer kn,rcount,getKsOfType,ireact,iexp,kii,i
      double precision stdev(maxexp,maxvar,maxpts) ! standard deviation 
      logical debug,var(maxvar)
      integer iexpcommon

      double precision edata2(maxexp,maxvar+1,2*maxpts) 
      integer npts2
      logical useip(maxvar)     ! use ip data or sim data in simulate single

      double precision L,K,NS ! loglikelihood, number of parameters, number of samples

      common/simulg/var,iexpcommon 
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata2/edata2,npts2,useip
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda

      debug=.false.

      error=0.d0
      rcount=1
      do 30 iexp=1,nexp
         iexpcommon=iexp ! fix exp to simulate
         if (debug) write(*,*) 'errvars, ==== Experiment ',iexp
         if (debug) write(*,*) 'errvars, error=',error
         if (debug) write(*,*) 'errvars, var=',var
         call simulatevars(nvar,
     *        modtype,modk,modkinit,modn,modv,dependent,
     *        nexp,nvare,npts,
     *        edata,sdata,
     *        istate)
         if (debug) write(*,*) 'errvars, ==== sim done istate ',istate
         if (istate.lt.0.d0) then
            error=1.d300 
            do 5 kii=rcount,N
               R(kii)=1.d10
               RES(kii)=1.d10
 5          continue
            return
         endif
         do 15 ivar=1,nvar
            if (var(ivar).and.dependent(ivar)) then
               do 10 i=1,npts(iexp)   
                  R(rcount)=sdata(iexp,(ivar+1),i)/stdev(iexp,(ivar
     $                 ),i)
                  RES(rcount)=(-sdata(iexp,(ivar+1),i)+edata(iexp
     $                 ,(ivar+1),i))/stdev(iexp,ivar,i)
                  error=error+RES(rcount)**2
                  rcount=rcount+1
 10            continue 
            endif
 15      continue
 30   continue   
      kn=0
      do 50, ivar=1,nvar
         if (var(ivar)) then
            do 40, ireact=1,modn(ivar)
               kn=kn+getKsOfType(modtype(ivar,ireact))
 40         continue
         endif
 50   continue
      L=-0.5*error
      K=kn
      NS=rcount-1
      !error=0.5*error+lambda*kn 
      error=-L+lambda*K 
      end                       ! End of errvars



C     Calculate true error (OUT) of model by using simulate
C     Consider ONLY experiment iexp
C     Update R and RES in correct positions, N corresponds to length of full R and RES
C     RES (OUT) residual (part of the error) 
      subroutine errtruforexp(error,R,RES,N,theexp)
     
      implicit none
      integer maxexp,maxpts,maxvar,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision sdata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      
      integer nvar

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      integer ivar,istate,N
      double precision error,lambda
      
      double precision R(N),RES(N) ! residuals

      integer kn,rcount,getKsOfType,ireact,iexp,kii,i,theexp
      double precision stdev(maxexp,maxvar,maxpts) ! standard deviation

      logical debug
      double precision L,K,NS ! loglikelihood, number of parameters, number of samples

      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda

      debug=.false.

      error=0.d0
      rcount=1
      do 30 iexp=1,nexp
         if (iexp.eq.theexp) then
            if (debug) write(*,*) 'errtru, ==== Experiment ',iexp
            call simulate(nvar,
     *           modtype,modk,modkinit,modn,modv,dependent,
     *           nexp,nvare,npts(iexp),
     *           edata,sdata,spline,
     *           iexp,
     *           istate)
            if (debug) write(*,*) 'errtru, ==== sim done istate ',istate
            if (istate.lt.0.d0) then
               error=1.d300 
               do 5 kii=rcount,N
                  R(kii)=1.d10
                  RES(kii)=1.d10
 5             continue
               return
            endif
         endif
         do 15 ivar=1,nvar
            if (dependent(ivar)) then
               do 10 i=1,npts(iexp)   
                  if (theexp.eq.iexp) then
                     R(rcount)=sdata(iexp,(ivar+1),i)/stdev(iexp,(ivar
     $                    ),i)
                     RES(rcount)=(-sdata(iexp,(ivar+1),i)+edata(iexp
     $                    ,(ivar+1),i))/stdev(iexp,ivar,i)
                     error=error+RES(rcount)**2
                  endif
                  rcount=rcount+1
 10            continue 
            endif
 15      continue
         if (theexp.eq.iexp) goto 1000 ! break
 30   continue   
 1000 continue

      kn=0
      do 50, ivar=1,nvar
         do 40, ireact=1,modn(ivar)
            kn=kn+getKsOfType(modtype(ivar,ireact))
 40      continue
 50   continue 
      L=-0.5*error
      K=kn
      NS=rcount-1
      !error=0.5*error+lambda*kn 
      error=-L+lambda*K 
      end                       ! End of errtruforexp






C     Calculate true error (OUT) of model by using simulate
C     Consider ONLY experiment iexp
C     Update R and RES in correct positions, N corresponds to length of full R and RES
C     RES (OUT) residual (part of the error) 
      subroutine errvarsforexp(error,R,RES,N,theexp)
     
      implicit none
      integer maxexp,maxpts,maxvar,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision sdata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      
      integer nvar

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      integer ivar,istate,N
      double precision error,lambda
      
      double precision R(N),RES(N) ! residuals

      integer kn,rcount,getKsOfType,ireact,iexp,kii,i,theexp
      double precision stdev(maxexp,maxvar,maxpts) ! standard deviation 
      

      logical debug


      logical var(maxvar)
      integer iexpcommon

      double precision edata2(maxexp,maxvar+1,2*maxpts) 
      integer npts2
      logical useip(maxvar)     ! use ip data or sim data in simulate single
      double precision L,K,NS ! loglikelihood, number of parameters, number of samples

      common/simulg/var,iexpcommon 
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata2/edata2,npts2,useip
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda

      debug=.false.

      error=0.d0
      rcount=1
      do 30 iexp=1,nexp
         if (iexp.eq.theexp) then
            if (debug) write(*,*) 'errtru, ==== Experiment ',iexp
            iexpcommon=theexp
            call simulatevars(nvar,
     *           modtype,modk,modkinit,modn,modv,dependent,
     *           nexp,nvare,npts,
     *           edata,sdata,
     *           istate)
            if (debug) write(*,*) 'errtru, ==== sim done istate ',istate
            if (istate.lt.0.d0) then
               error=1.d300 
               do 5 kii=rcount,N
                  R(kii)=1.d10
                  RES(kii)=1.d10
 5             continue
               return
            endif
         endif
         do 15 ivar=1,nvar
            if (var(ivar).and.dependent(ivar)) then
               do 10 i=1,npts(iexp)   
                  if (theexp.eq.iexp) then
                     R(rcount)=sdata(iexp,(ivar+1),i)/stdev(iexp,(ivar
     $                    ),i)
                     RES(rcount)=(-sdata(iexp,(ivar+1),i)+edata(iexp
     $                    ,(ivar+1),i))/stdev(iexp,ivar,i)
                     error=error+RES(rcount)**2
                  endif
                  rcount=rcount+1
 10            continue 
            endif
 15      continue
         if (theexp.eq.iexp) goto 1000 ! break
 30   continue   
 1000 continue
      
      kn=0
      do 50, ivar=1,nvar
         if (var(ivar)) then
            do 40, ireact=1,modn(ivar)
               kn=kn+getKsOfType(modtype(ivar,ireact))
 40         continue
         endif
 50   continue
      L=-0.5*error
      K=kn
      NS=rcount-1
      !error=0.5*error+lambda*kn 
      error=-L+lambda*K 
      end                       ! End of errvarsforexp





C     Calculate error (OUT) of ONE equation/variable ivar (IN) using
C     simulatesingle 
C     R,RES (OUT) residual (part of the error) 
C     all exps have been considered the method returns anyway
C     edata2, simulated data of the currently best model
C     npts2 - npts in edata2
C     useip(ivar) tells if we should use smoothed data or simulated 
C     data (edata2) in simulatesingle
      subroutine errvar(error,R,RES,N,ivar)
      implicit none
      integer maxexp,maxpts,maxvar,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp),npts2
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision edata2(maxexp,maxvar+1,2*maxpts)
      double precision sdata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      
      integer nvar,N

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      integer ivar,istate,i,ireact,iexp,kii
      integer getKsOfType
      double precision error,lambda

      double precision R(maxexp*maxpts),RES(maxexp*maxpts) ! residuals
      
      integer kn,rcount
      double precision stdev(maxexp,maxvar,maxpts) ! standard deviation
      
      logical useip(maxvar)       ! use interpolated data in simuletesingle

      logical debug
      double precision L,K,NS ! loglikelihood, number of parameters, number of samples

      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata2/edata2,npts2,useip
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda
 
      debug=.false.
      if (debug) write(*,*) 'errrvar, useip(.)',(useip(i),i=1,nvar)

      error=0.d0
      rcount=1

C     calculate kn=no of parameters
      kn=0
      do 10, ireact=1,modn(ivar)
         kn=kn+getKsOfType(modtype(ivar,ireact))
 10   continue
 
      do 40 iexp=1,nexp 
         if (debug) write(*,*)'errvar, simulate exp=',iexp
         call simulatesingle(nvar,
     *        modtype,modk,modkinit,modn,modv,dependent,
     *        nexp,nvare,npts(iexp),
     *        edata,sdata,spline,
     *        iexp,
     *        istate,
     *        ivar,
     *        edata2,npts2,useip)
         if (istate.lt.0.d0) then
            error=1.d300
            !write(*,*) 'errvar, istate',istate  
            do 2 kii=rcount,N
               R(kii)=1.d10
               RES(kii)=1.d10
 2          continue
            return
         endif
         do 30 i=1,npts(iexp)  
            R(rcount)=sdata(iexp,(ivar+1),i)/stdev(iexp,(ivar
     $           ),i)
            RES(rcount)=(-sdata(iexp,(ivar+1),i)+edata(iexp
     $           ,(ivar+1),i))/stdev(iexp,ivar,i)
            error=error+RES(rcount)**2
            rcount=rcount+1
 30      continue 
 40   continue  
      L=-0.5*error
      K=kn
      NS=rcount-1
      !error=0.5*error+lambda*kn 
      error=-L+lambda*K  
      end                       ! End of errvar
    




C     Calculate error (OUT) of ONE equation/variable ivar (IN) 
C     using edata2, simulated data of the currently best model
C     res (OUT) residual (part of the error) 
C     if quitbo==true and error>erropt before
C     all exps have been considered the method returns anyway
      subroutine errvar2(error,res,ivar)

      implicit none
      integer maxexp,maxpts,maxvar,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp),npts2
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision edata2(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      
      integer nvar,getKsOfType

      
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent


      logical useip(maxvar)     ! use ip data or sim data in simulate single
      integer ivar,ireact,iexp,i
      double precision tempe,error,y2,gy,res,lambda
      
      integer kn,rcount
      double precision stdev(maxexp,maxvar,maxpts) ! standard deviation 
      
      double precision L,K,NS ! loglikelihood, number of parameters, number of samples

      !logical debug
      !debug=.true.

      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata2/edata2,npts2,useip
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda
 
      error=0.d0

      kn=0
      do 10, ireact=1,modn(ivar)
         kn=kn+getKsOfType(modtype(ivar,ireact))
 10   continue
      rcount=1
      do 40 iexp=1,nexp
         do 30 i=1,npts(iexp)     
            y2=gy(nexp,nvare,npts2,
     *           edata2,
     *           iexp,ivar,edata(iexp,1,i))
            tempe=((y2-
     *           edata(iexp,(ivar+1),i))/
     *           stdev(iexp,ivar,i))**2
            error=error+tempe
            rcount=rcount+1
 30      continue 
 40   continue  
      res=0.5*error
      L=-0.5*error
      K=kn
      NS=rcount-1
      !error=0.5*error+lambda*kn 
      error=-L+lambda*K 
      end                       ! End of errvar2
      

