C    Odeidon, a program for structural identification of ODEs
C    Copyright (C) 2011 Peter Gennemark, Dag Wedelin
C
C    This program is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with this program.  If not, see <http://www.gnu.org/licenses/>.

C    Contact: Peter Gennemark, Dag Wedelin
C             www.odeidentification.org
C
C
C
C     estimate model based on graph of shared parameeters and estvars
C     Logical general, T if general estimation, merge parameters, F if not
      subroutine estbyestvars(general)

      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
           
      integer nvar
      logical graph(maxvar,maxvar)
      logical general
      
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      double precision error
      integer debug,ivar,ivar2,idummy 
      logical skip,var(maxvar)

      logical initconc          ! T if init conc consider as parameter, ow F
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/myinitconc/initconc
      common/simulg/var,idummy  
      
      debug=1

      call getGraph(graph,nexp,nvar)

      if (debug.gt.0) then 
         write(*,*) 'estbyestvars, graph='
         do 20, ivar=1,nvar
            write(*,*) (graph(ivar,ivar2),ivar2=1,nvar)
 20      continue
      endif
      
      if (debug.gt.0) write(*,*) 'estbyestvars, initconc=',initconc

      do 100 ivar=1,nvar
 
         skip=.true.
         do 50 ivar2=1,nvar       
            var(ivar2)=.false. 
 50      enddo

         var(ivar)=.true.
         do 60 ivar2=1,nvar      
            if (graph(ivar,ivar2)) then 
               var(ivar2)=.true.
               skip=.false.
            endif
 60      enddo
         
         if (skip) goto 99

         if (debug.gt.0) then
            write(*,*) 'estbyestvars, var = ',(var(ivar2),ivar2=1,nvar)
         endif

         if (general) then
            call estvarsgeneral(edata,spline,
     *           nexp,nvare,npts,
     *           nvar,
     *           modtype,modk,modkinit,modn,modv,dependent,
     *           error)
         else
            call estvars(edata,spline,
     *           nexp,nvare,npts,
     *           nvar,
     *           modtype,modk,modkinit,modn,modv,dependent,
     *           error)
         endif
         
         if (debug.gt.0) then
            write(*,*) 'estbyestvars, global error=',error
         endif
         do 70 ivar2=1,nvar  
            if (debug.gt.0.and.var(ivar2)) call printvar(maxvar,maxr
     $           ,maxv,maxk,nvar,modtype,modk,modn,modv,dependent,ivar2)
 70      enddo
 99      continue
 100  enddo
      end




C     estimate all parameters simulatanously using dn2gb
C     logical doestvars, T if we should do est of subprat before global est
      subroutine est(error,doestvars)

      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
           
      integer nvar
      logical doestvars
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      double precision error
      integer debug,ipara,ivar,i,iLIV,iLV,ireact,iexp
      integer getKsOfType

C     DN2GB
      INTEGER P, LIV, LV, UI(1),NF,N,LR
      DOUBLE PRECISION B(2,400), TY(1), X(400) 
      EXTERNAL DUMMY, JACOB1, FUN1
      parameter(LIV=1200,LV=600000,LR=12000)
      double precision R(LR),RES(LR),V(LV)
      INTEGER IV(LIV)

      logical initconc          ! T if init conc consider as parameter, ow F
      common/myinitconc/initconc
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/myBest/B ! communicates est-fun2

!     LIV = 82 + 4*P 
!     LV = 105 + P*(N + 2*P + 21) + 2*N

      if (doestvars) call estbyestvars(.false.) 
      debug=0
      P=0
      N=0
      do 10, ivar=1,nvar
         do 8, ireact=1,modn(ivar)
            P=P+getKsOfType(modtype(ivar,ireact))
 8       continue
         if (dependent(ivar)) then
            do 9, iexp=1,nexp
               N=N+npts(iexp)
 9          continue
            if (initconc) P=P+nexp ! initial conc
         endif
 10   continue
      
      if (P.gt.400) then
         write(*,*) 'est, ivar=',ivar,' P ',P
     $        ,' > 400. Increase sizes of datastructures',
     *        'NOTE: also in fun1 and fun1general!'
         stop
      endif

      if (debug.gt.1) write(*,*) 'est, P ',P,' N='
     $     ,N
      iLIV = 82 + 4*P
      iLV = 105 + P*(N + 2*P + 21) + 2*N
      if (iLIV.gt.LIV) then
         write(*,*) 'est, LIV should be ', iLIV, ' is ',LIV
         stop
      endif
      if (iLV.gt.LV) then
         write(*,*) 'est, LV should be ', iLV, ' is ',LV
         stop
      endif
      if (N.gt.LR) then
         write(*,*) 'est, LR should be ', N, ' is ',LR
         write(*,*) 'est, also check R and RES.'
         stop
      endif
      
      
      call modeltox (maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,X,B,P)
      
      if (debug.gt.0) then
         do 80 i=1,P
            write(*,*) 'est, X(',i,')=',X(i),' [',B(1,i),', ',B(2,i),']'
 80      continue   
      endif
      
      DO 90 I=1,LIV 
         IV(I)=0
 90   CONTINUE 
      DO 100 I=1,LV 
         V(I)=0.E0
 100  CONTINUE 
      
c$$$      if (debug.gt.1) then
c$$$         write(*,*) 'est, printmodel'
c$$$         call printmodel(maxvar,maxr,maxv,maxk,nvar,
c$$$     *        modtype,modk,modn,modv,dependent)      
c$$$         write(*,*) 'est, call fun1'
c$$$         call FUN1(N, P, X, NF, R, UI, TY,DUMMY) 
c$$$         call printmodel(maxvar,maxr,maxv,maxk,nvar,
c$$$     *        modtype,modk,modn,modv,dependent)
c$$$         write(*,*) 'est, call dn2gb'
c$$$      endif


      call DN2GB(N,P,X, B, FUN1, JACOB1, IV, LIV, LV, V, UI, TY,DUMMY) 
      
      call xtomodel (maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,X,P) 
     
      call errtru(error,R,RES,N)

      if (debug.gt.0) then
         write(*,*) 'IV1= ',IV(1)
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         write(*,*) 'Error=',error
      endif
      end

      
C     estimate all parameters simulatanously using dn2gb
C     logical doestvars, T if we should do est of subprat before global est
      subroutine estgeneral(error,doestvars)

      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
           
      integer nvar
      logical doestvars
      
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      
      double precision error
      integer debug,ipara,P2
      integer ivar,i,iLIV,iLV,ireact,iexp


      INTEGER P, LIV, LV, UI(1),NF,N,LR
      DOUBLE PRECISION B(2,400), TY(1), X(400) 
      EXTERNAL DUMMY, JACOB1GENERAL, FUN1GENERAL
      parameter(LIV=3000,LV=1500000,LR=30000)
      double precision R(LR),RES(LR),V(LV)
      INTEGER IV(LIV)


      logical initconc          ! T if init conc consider as parameter, ow F
      common/myinitconc/initconc
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/myBest/B ! communicates estgeneral-fun2

!     P=(maxvar-2)*(maxr+maxr+maxr*2+maxr*3) 
!     N=(maxvar-2)*maxexp*realistic maxpts 
!     LIV = 82 + 4*P 
!     LV = 105 + P*(N + 2*P + 21) + 2*N 
 

      if (doestvars) call estbyestvars(.true.)


      debug=0
C     get no of parameters P, X and B
      call modeltoxgeneral(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,
     *     X,B,200,P,initconc)

      N=0
      do 10, ivar=1,nvar
         if (dependent(ivar)) then
            do 9, iexp=1,nexp
               N=N+npts(iexp)
 9          continue
         endif
 10   continue
      if (P.gt.400) then
         write(*,*) 'estgeneral, ivar=',ivar,' P ',P
     $        ,' > 400. Increase sizes of datastructures',
     *        'NOTE: also in fun1 and fun1general!'
         stop
      endif
      if (debug.gt.1) write(*,*) 'estgeneral, P ',P,' N='
     $     ,N
      iLIV = 82 + 4*P
      iLV = 105 + P*(N + 2*P + 21) + 2*N
      if (iLIV.gt.LIV) then
         write(*,*) 'estgeneral, LIV should be ', iLIV, ' is ',LIV
         stop
      endif
      if (iLV.gt.LV) then
         write(*,*) 'estgeneral, LV should be ', iLV, ' is ',LV
         stop
      endif
      if (N.gt.LR) then
         write(*,*) 'estgeneral, LR should be ', N, ' is ',LR
         write(*,*) 'estgeneral, also check R and RES.'
         stop
      endif
      
      call modeltoxgeneral(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,
     *     X,B,P,P2,initconc)
      if (debug.gt.0) then
         do 80 i=1,P
            write(*,*) 'estgeneral, X(',i,')=',X(i),' [',B(1,i),', ',B(2
     $           ,i),']'
 80      continue   
      endif
      DO 90 I=1,LIV 
         IV(I)=0
 90   CONTINUE 
      DO 100 I=1,LV 
         V(I)=0.E0
 100  CONTINUE 
      if (debug.gt.0) write(*,*) 'estgeneral, call dn2gb'
      CALL DN2GB(N,P,X, B, FUN1GENERAL, JACOB1GENERAL, IV, LIV, LV, V,
     $     UI, TY,DUMMY) 
      if (debug.gt.0) then
         do 81 i=1,P
            write(*,*) 'estgeneral, X(',i,')=',X(i),' [',B(1,i),', ',B(2
     $           ,i),']'
 81      continue   
      endif

      call xtomodelgeneral(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,
     *     X,P,initconc)
      call errtru(error,R,RES,N)
      if (debug.gt.0) then
         write(*,*) 'IV1= ',IV(1)
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         write(*,*) 'Error=',error

         write(*,*) 'estgeneral, error=',error
      endif
      end



C     Model not modified
C     errcur not affected
C     error, OUT, resulting error
      subroutine estmakeedata2(error,spline2)
      implicit none
      
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer nexp,nvare,npts(maxexp),npts2
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision edata2(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      integer spline2(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 

      
      integer nvar
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

C     ------ TEMP MODEL ------
      integer tmodtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision tmodk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision tmodkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer tmodn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer tmodv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]


      double precision errtem
      double precision error,err0,res,restem,c2 
      logical useip(maxvar)
      integer debug,iseed,ivar,istate,iexp,i,iv

      DOUBLE PRECISION X(35)
      integer ipara,ireact,j,getKsOfType

      logical initconc          ! T if init conc consider as parameter, ow F
      common/myinitconc/initconc
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata2/edata2,npts2,useip
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
 

      debug=1 !0-3
      iseed=10
      c2=0.d0
      if (debug.gt.2) then
         write(*,*) 'estmakeedata2, initial model' 
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
      endif
      
C     1. === Error using interpolated data 
      do 10 iv=1,nvar       
         useip(iv)=.true.  
 10   continue 
        
      call icopy(maxvar*maxr,modtype,1,tmodtype,1)      
      call dcopy(maxvar*maxr*maxk*3,modk,1,tmodk,1)    
      call dcopy(maxvar*maxexp*3,modkinit,1,tmodkinit,1)
      call icopy(maxvar,modn,1,tmodn,1)
      call icopy(maxvar*maxr*maxv,modv,1,tmodv,1)
 
      
      err0=0d0
      do 15, ivar=1,nvar
         if (dependent(ivar)) then
            call estvar12(ivar,errtem)
            err0=err0+errtem 
         endif
 15   continue 


C     Reset 
      call icopy(maxvar*maxr,tmodtype,1,modtype,1)      
      call dcopy(maxvar*maxr*maxk*3,tmodk,1,modk,1)    
      call dcopy(maxvar*maxexp*3,tmodkinit,1,modkinit,1)
      call icopy(maxvar,tmodn,1,modn,1)
      call icopy(maxvar*maxr*maxv,tmodv,1,modv,1)
      if (debug.gt.0) write(*,5000) 'estmakeedata2, error=',err0,
     *     ' (IP edata)'
 5000 format(A,D10.3,A)
     


C     2 === Error using simulated data

      if (debug.gt.1) write(*,*) 'estmakeedata2 call est'
c$$$  call estgeneral(edata,spline,maxexp,maxpts,
c$$$  *     nexp,nvare,npts,
c$$$  *     maxvar,maxr,maxv,maxk,nvar,
c$$$  *     modtype,modk,modn,modv,dependent,
c$$$  *     errtem)
           
      call est(errtem,.false.)
      !call est(errtem,.true.)

      do 300 iexp=1,nexp
         if (debug.gt.1) write(*,*) 'estmakeedata2, simulate exp ',iexp
         call simulate(nvar,
     *        modtype,modk,modkinit,modn,modv,dependent,
     *        nexp,nvare,npts2,
     *        edata2,edata2,spline2,
     *        iexp,
     *        istate)
         if (debug.gt.1) write(*,*) 'estmakeedata2, sim done istate '
     $        ,istate
         if (istate.lt.0.d0) then
            write(*,*) 'estmakeedata2, sim done istate '
     $           ,istate
            !stop
            goto 9999
         endif
 300  continue
      
      do 30 iv=1,nvar       
         useip(iv)=.false.  
 30   continue 


      
      error=0d0
      do 35, ivar=1,nvar
         if (dependent(ivar)) then
            call estvar12(ivar,errtem)
            error=error+errtem
         endif
 35   continue 
      
      if (debug.gt.0) write(*,5001) 'estmakeedata2, error=',error,
     *     ' (SIM edata) (e=',err0,' IP edata)'
 5001 format(A,D10.3,A,D10.3,A)
      call icopy(maxvar*maxr,tmodtype,1,modtype,1)      
      call dcopy(maxvar*maxr*maxk*3,tmodk,1,modk,1)    
      call dcopy(maxvar*maxexp*3,tmodkinit,1,modkinit,1)
      call icopy(maxvar,tmodn,1,modn,1)
      call icopy(maxvar*maxr*maxv,tmodv,1,modv,1)


      if (err0.le.error*1.001) then ! IP data better as first approximation
      !if (err0.le.error*1.01) then   ! IP data better as first approximation
         do 50 iv=1,nvar       
            useip(iv)=.true.  
 50      continue 
      endif

      if (debug.gt.0) write(*,*) 'estmakeedata2, useip()=',(useip(i),i=1
     $     ,nvar)

      return


 9999 continue
      call icopy(maxvar*maxr,tmodtype,1,modtype,1)      
      call dcopy(maxvar*maxr*maxk*3,tmodk,1,modk,1)    
      call dcopy(maxvar*maxexp*3,tmodkinit,1,modkinit,1)
      call icopy(maxvar,tmodn,1,modn,1)
      call icopy(maxvar*maxr*maxv,tmodv,1,modv,1)
      do 501 iv=1,nvar       
         useip(iv)=.true.  
 501  continue 
 
      end ! END of subroutine estmakeedata2

