C    Odeidon, a program for structural identification of ODEs
C    Copyright (C) 2011 Peter Gennemark, Dag Wedelin
C
C    This program is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with this program.  If not, see <http://www.gnu.org/licenses/>.

C    Contact: Peter Gennemark, Dag Wedelin
C             www.odeidentification.org
C
C
C
C     Smooth data
C     data is stored in a 3-d array with experiment number, 
C     variable (index 1 is time, indices 2.. are the variables) 
C     and pts (points)
C     edata(maxexp,maxvar,2*maxpts)
C     edata(iexp,1,1..npts) contains npts time values for experiment iexp
C     edata(iexp,1+ivar,1..npts) contains npts concentration values for ivar in iexp
C     edata(iexp,1+ivar,maxpts+(1..npts)) contains npts derivatives for ivar in iexp
C     s=smoothing parameter
C     debug=debug level, 0=none, 1=little, 2=more
      subroutine smoothdata(s,debug)
      implicit none

      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer maxtypes
      parameter (maxtypes=10)
      integer nexp,nvare,npts(maxexp),npts2
      double precision edata(maxexp,maxvar+1,2*maxpts)  
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 


C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      integer nvar,k,j,ii,icho,i,iexp
      double precision s,tem
      double precision time,smoo(maxvar),gysmth
      integer debug
      logical useip(maxvar)     ! use ip data or sim data in simulate single

      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      
C     Smooth dependent variables
      do 80 i= 1,nexp          
         do 70 j= 1,nvare  
            if (dependent(j)) then  
               if (debug.gt.1) write(*,*) 'code_rk_exp, smooth iexp=',i
     $              ,' var=',j,' spline=', spline(i,j) 
               tem=gysmth(i,j,0.d0,0,.true.,s)
               if (spline(i,j).eq.1) then 
                  do 60 k= 1,npts(i)   
                     edata(i,j,maxpts+k)=gysmth(i,j,edata(i,1,k),
     *                    1,.false.,s)
                     if (edata(i,j,maxpts+k).gt.-1.d100.and.
     *                    edata(i,j,maxpts+k).lt.1.d100) then
                                !ok
                     else
                        write(*,*) 'code_rk_exp, smooth'
                        write(*,*) 'edata(',i,',',j,',',maxpts+k,')=',
     *                       edata(i,j,maxpts+k)
                        stop
                     endif
 60               continue
               endif
            endif               ! end if dependent
 70      continue
 80   continue
      return
 9999 stop
      end






C     Simulate edata2, similar to edata but with more data-points
C     data is stored in a 3-d array with experiment number, 
C     variable (index 1 is time, indices 2.. are the variables) 
C     and pts (points)
C     edata(maxexp,maxvar,2*maxpts)
C     edata(iexp,1,1..npts) contains npts time values for experiment iexp
C     edata(iexp,1+ivar,1..npts) contains npts concentration values for ivar in iexp
C     edata(iexp,1+ivar,maxpts+(1..npts)) contains npts derivatives for ivar in iexp
C     spline=monitor spline type for each variable: 
C            (0=linear interpolation, 1=smoothing spline)
C     debug=debug level, 0=none, 1=little, 2=more
      subroutine readdata2(edata,edata2,spline,
     *     nexp,nvare,npts,npts2,
     *     nvar,
     *     debug)
      
      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)

      integer nexp,nvare,npts(maxexp),npts2
      double precision edata(maxexp,maxvar+1,2*maxpts)
      double precision edata2(maxexp,maxvar+1,2*maxpts)

      integer nvar
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 

      
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      double precision deltat
      integer debug,i,j,k,iexp,istate

      common/mydata3/modk,modkinit,modn,modv,modtype,dependent  
      
      npts2=maxpts              ! Number of points is the maximum allowed
      if (debug.gt.1) then 
         write(*,*) 'readdata2, npts = ',(npts(i),i=1,nexp)
      endif
      if (nexp.GT.maxexp) then
         write(*,*) 'readdata2, nexp = ', nexp, '> maxexp =', maxexp
         goto 9999
      endif
      if (nvare.GT.maxvar) then
         write(*,*) 'readdata2, nvare = ', nvare, '> maxvar =', maxvar
         goto 9999
      endif
      if (npts2.GT.maxpts) then
         write(*,*) 'readdata2, npts2 = ', npts2, '> maxpts =', maxpts
         goto 9999
      endif
      
      
      
      do 20 i= 1, nexp 
         if (npts(i).GT.maxpts) then
            write(*,*) 'readdata, npts(',i,') = ', npts(i), 
     *           '> maxpts =', maxpts
            goto 9999
         endif      
         do 10 j= 1,nvare
            spline(i,j)=0
 10      enddo
 20   enddo
      


C     Set time values for all experiments, uniformly distributed
      do 26 i= 1, nexp          ! Loop over the exps
         edata2(i,1,1)=edata(i,1,1)
         edata2(i,1,npts2)=edata(i,1,npts(i))
         deltat=(edata(i,1,npts(i))-edata(i,1,1))/(npts2-1)
         do 22 k=2,npts2-1
            edata2(i,1,k)=edata2(i,1,1)+deltat*(k-1)
 22      enddo
         if (debug.gt.1) then 
            write(*,*) (edata2(1,1,k),k=1,npts2)
         endif
 26   continue
      
C     Init conc for all experiments
      do 40 i= 1, nexp          ! Loop over the exps
         do 30 j= 2, (nvare+1)   ! init values
            edata2(i,j,1)=edata(i,j,1)
            edata2(i,j,2)=edata(i,j,2)
            if (debug.gt.1) then 
               write(*,*) 'Read(',i,',',j,',',1,')=',edata2(i,j,1)
            endif
 30      enddo
 40   enddo

C     Simulate data  
      do 50 iexp= 1, nexp       ! Loop over the exps  
         call simulate(nvar,
     *        modtype,modk,modkinit,modn,modv,dependent,
     *        nexp,nvare,npts2,
     *        edata2,edata2,spline,
     *        iexp,
     *        istate)
         if (istate.lt.0.d0)then
            write(*,*) 'exp, warning. dlsode istate=',istate
         endif
 50   enddo
      

      return
 9999 stop
      end
  

  
C     Print edata
C     data is stored in a 3-d array with experiment, 
C     variable (index 1 is time, indices 2.. are the variables) 
C     and pts (points)
      subroutine printdata(edata,spline,
     *     maxexp,maxvar,maxpts,nexp,nvare,npts)      
      implicit none
      integer maxexp,maxvar,maxpts
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      integer i,j,k

      write(*,*) 'nexp',nexp
      write(*,*) 'nvare',nvare
      write(*,*) 'npts',(npts(i),i=1,nexp)
      do 30 i= 1, nexp          ! Loop over the data points
         write(*,*) '-- exp=',i
         do 20 j= 1, (nvare+1)   ! also read time values
            if (j.eq.1) then 
               write(*,*) '---- time'
               do 10 k= 1, npts(i)
                  write(*,*) '------ point=',edata(i,j,k)
 10            enddo
            else  
               write(*,*) '---- var=',(j-1)
               do 15 k= 1, npts(i)
                  write(*,*) '------ point=',edata(i,j,k)
 15            enddo
            endif
 20      enddo
 30   enddo

      do 60 i= 1, nexp          ! Init conc for each exp
         write(*,*) '-- exp=',i
         do 50 j= 1,nvare   
            write(*,*) '---- y0(',j,')=',edata(i,(j+1),1)
 50      enddo
         write(*,*) '-----spline=',(spline(i,j),j=1,nvare)
 60   enddo  

      end
     
C     Print summary of edata 
C     data is stored in a 3-d array with experiment, 
C     variable (index 1 is time, indices 2.. are the variables) 
C     and pts (points)
      subroutine printdata2(edata,spline,
     *     maxexp,maxvar,maxpts,nexp,nvare,npts)  
      implicit none
      integer maxexp,maxvar,maxpts
      integer nexp,nvare,npts(maxexp)
      double precision edata(maxexp,maxvar+1,2*maxpts)
      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 
      integer i,j,imax
      write(*,100) 'nexp=',nexp
      write(*,100) 'nvare=',nvare
      write(*,*) 'npts=',(npts(i),i=1,nexp)
 100  format(A,I3)
      imax=nexp
      if (nexp.gt.30) imax=30
      do 600 i= 1,imax          ! Loop over the data points
         write(*,200) ' exp=',i,' y0=',(edata(i,j,1), j= 2, nvare+1)
 200     format(A,I2,A,30D10.2)
         write(*,300) '       spline=',(spline(i,j),j=1,nvare)
 300     format(A,20I3)
 600  enddo
      end
      







C     Get jderiv of experiment exp, variable var at time t. If first is
C     true, it is the first call and smoothing parameters are calculated
C     with upper bound s on the discrete weighted mean square distance
C     of the approximation f from the data.
      double precision function gysmth(exp,var,t,jderiv,first,s)
      
      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)

      integer nexp,nvare,npts(maxexp),nvar
      double precision edata(maxexp,maxvar+1,2*maxpts)

      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline 

      integer exp,var,jderiv
      double precision t,delta,slope
      
      integer lsm

      double precision s,sfinal
      double precision ppvalumain,sf
 

      logical first,debug

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      integer low,mid,high,i,j
      double precision slope1,slope2
      
                                
C     errsettings
      double precision stdev(maxexp,maxvar,maxpts), lambda
    
      common/mydata/edata,spline,npts,nexp,nvar,nvare
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/errsettings/stdev,lambda
 
      debug=.false.

C     Independent variables
      if (.not.dependent(var)) then
         if (t.le.0) then
            gysmth=edata(exp,(var+1),1)
         else
            gysmth=edata(exp,(var+1),2)
         endif
         return
      endif


C     Cubic smoothing spline or linear interpolation


      if (spline(exp,var).eq.1) then ! Cubic smoothing spline
         gysmth=ppvalumain(var,exp,t,jderiv,npts(exp))
         return

      else if (jderiv.eq.0) then ! LINEAR IP, ZEROTH DERIVATIVE 
         low=1                  !  binary search to find element
         high=npts(exp)
 90      if (low.le.high) then  ! Loop starts
            if (debug) write(*,*) 'loop starts'
            mid=(low+high)/2 
            if (edata(exp,1,mid).lt.t) then 
               low=mid+1
               goto 90
            else if (edata(exp,1,mid).gt.t) then 
               high=mid-1
               goto 90
            else 
               gysmth=edata(exp,(var+1),mid) ! perfect match
               return
            endif
         endif
         if ((high-low).eq.-1) then ! everything alright, low/high interchanged
            if (high.eq.0) then
               gysmth=edata(exp,(var+1),1)
               return
            endif
            delta=t-edata(exp,1,high)
            slope=(edata(exp,(var+1),low)-
     *           edata(exp,(var+1),high))/
     *           (edata(exp,1,low)-edata(exp,1,high))
            gysmth=edata(exp,(var+1),high)+slope*delta
            return
         else
            write(*,*) 'gysmth, error high=',high,' low=',low 
            gysmth=0.d0
            return
         endif
      elseif (jderiv.eq.1) then ! LINEAR IP, FIRST DERIVATIVE 
         if (t.lt.edata(exp,1,1).or.t.gt.edata(exp,1,npts(exp)))
     $        then 
            write(*,*) 'gysmth, t=',t,' outside range'
            gysmth=0.d0
            return
         endif
         low=1                  !  binary search to find element
         high=npts(exp)
 100      if (low.le.high) then  ! Loop starts
            mid=(low+high)/2 
            if (edata(exp,1,mid).lt.t) then 
               low=mid+1
               goto 100
            else if (edata(exp,1,mid).gt.t) then 
               high=mid-1
               goto 100
            else                ! perfect match 
               if (mid.eq.1) then 
                  gysmth=(edata(exp,(var+1),2)-
     *                 edata(exp,(var+1),1))/
     *                 (edata(exp,1,2)-edata(exp,1,1))
               else if (mid.eq.npts(exp)) then 
                  gysmth=(edata(exp,(var+1),npts(exp))- edata(exp
     $                 ,(var+1),(npts(exp)-1)))/ (edata(exp,1
     $                 ,(npts(exp))) -edata(exp,1,(npts(exp)-1)))
               else
                  slope1=(edata(exp,(var+1),(mid+1))-
     *                 edata(exp,(var+1),mid))/
     *                 (edata(exp,1,(mid+1))-edata(exp,1,mid))
                  slope2=(edata(exp,(var+1),mid)-
     *                 edata(exp,(var+1),(mid-1)))/
     *                 (edata(exp,1,mid)-edata(exp,1,(mid-1)))
                  gysmth=(slope1+slope2)/2 
               endif
               return
            endif
         endif
         if ((high-low).eq.-1) then ! everything alright, low/high interchanged
            gysmth=(edata(exp,(var+1),low)-edata(exp,(var+1)
     $           ,high))/(edata(exp,1,low)-edata(exp,1,high))
            return
         else
            write(*,*) 'gysmth, error high=',high,' low=',low 
            gysmth=0.d0
            return
         endif
      endif
      return
      end


C     Get value of experiment exp, variable var at time t
      double precision function gy(nexp,nvare,npts,
     *     edata,
     *     exp,var,t)
      implicit none

      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)



      integer nexp,nvare,npts
      double precision edata(maxexp,maxvar+1,2*maxpts)

      integer exp,var
      
      integer low,mid,high
      double precision t,slope,delta

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent


      common/mydata3/modk,modkinit,modn,modv,modtype,dependent


      !logical debug
      !debug=.false.
      !if (debug) write(*,*) 'exp=',exp,' var=',var,' t=',t


C     Independent variables
      if (.not.dependent(var)) then
         if (t.le.0) then
            gy=edata(exp,(var+1),1)
         else
            gy=edata(exp,(var+1),2)
         endif
         return
      endif


      low=1                     !  binary search to find element
      high=npts
 10   if (low.le.high) then     ! Loop starts
         mid=(low+high)/2 
         !if (debug) write(*,*) 'mid=',mid,' expt(mid)=',edata(exp,1,mid)
         if (edata(exp,1,mid).lt.t) then 
            low=mid+1
            goto 10
         else if (edata(exp,1,mid).gt.t) then 
            high=mid-1
            goto 10
         else 
            !if (debug) write(*,*) 'perfect match, mid=',mid
            gy=edata(exp,(var+1),mid) ! perfect match
            return
         endif
      endif
      if ((high-low).eq.-1) then ! everything alright, low/high interchanged
         !if (debug) write(*,*) 'gy, high=',high,' low=',low 
         delta=t-edata(exp,1,high)
         slope=(edata(exp,(var+1),low)-edata(exp,(var+1),high))/
     *        (edata(exp,1,low)-edata(exp,1,high))
         gy=edata(exp,(var+1),high)+slope*delta
         return
      else
         write(*,*) 'gy, error high=',high,' low=',low 
         gy=0.d0
         return
      endif
      end

C     Get delta value of experiment exp, variable var at time t
      double precision function gydot(nexp,nvare,npts,
     *     edata,
     *     exp,var,t)
      
      implicit none

      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)



      integer nexp,nvare,npts
      double precision edata(maxexp,maxvar+1,2*maxpts)

      integer exp,var
      
      integer low,mid,high
      double precision t,slope1,slope2

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      common/mydata3/modk,modkinit,modn,modv,modtype,dependent


      !logical debug
      !debug=.false.
      !if (debug) write(*,*) 'exp=',exp,' var=',var,' t=',t


C     Independent variables
      if (.not.dependent(var)) then
         gydot=0.d0
         return
      endif

      if (t.lt.edata(exp,1,1).or.t.gt.edata(exp,1,npts)) then 
         write(*,*) 'gydot, t=',t,' outside range'
         gydot=0.d0
         return
      endif
      low=1                     !  binary search to find element
      high=npts
 10   if (low.le.high) then     ! Loop starts
         mid=(low+high)/2 
         !if (debug) write(*,*) 'mid=',mid,' expt(mid)=',edata(exp,1,mid)
         if (edata(exp,1,mid).lt.t) then 
            low=mid+1
            goto 10
         else if (edata(exp,1,mid).gt.t) then 
            high=mid-1
            goto 10
         else ! perfect match 
            !if (debug) write(*,*) 'perfect match, mid=',mid
            if (mid.eq.1) then 
               gydot=(edata(exp,(var+1),2)-
     *              edata(exp,(var+1),1))/
     *              (edata(exp,1,2)-edata(exp,1,1))
            else if (mid.eq.npts) then 
               gydot=(edata(exp,(var+1),npts)-
     *              edata(exp,(var+1),(npts-1)))/
     *              (edata(exp,1,(npts))-edata(exp,1,(npts-1)))
            else
               slope1=(edata(exp,(var+1),(mid+1))-
     *              edata(exp,(var+1),mid))/
     *              (edata(exp,1,(mid+1))-edata(exp,1,mid))
               slope2=(edata(exp,(var+1),mid)-
     *              edata(exp,(var+1),(mid-1)))/
     *              (edata(exp,1,mid)-edata(exp,1,(mid-1)))
               gydot=(slope1+slope2)/2 
            endif
            return
         endif
      endif
      if ((high-low).eq.-1) then ! everything alright, low/high interchanged
         !if (debug) write(*,*) 'gydot, high=',high,' low=',low 
         gydot=(edata(exp,(var+1),low)-edata(exp,(var+1),high))/
     *        (edata(exp,1,low)-edata(exp,1,high))
         return
      else
         write(*,*) 'gydot, error high=',high,' low=',low 
         gydot=0.d0
         return
      endif
      end





