C    Odeidon, a program for structural identification of ODEs
C    Copyright (C) 2011 Peter Gennemark, Dag Wedelin
C
C    This program is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    This program is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.

C    You should have received a copy of the GNU General Public License
C    along with this program.  If not, see <http://www.gnu.org/licenses/>.

C    Contact: Peter Gennemark, Dag Wedelin
C             www.odeidentification.org
C
C
C
C     Print model equations
      subroutine printmodel(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modn,modv,dependent)

      implicit none
      integer maxvar,maxr,maxv,maxk,nvar,ivar

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      do 10 ivar=1,nvar
         if (dependent(ivar)) then
            call printvar(maxvar,maxr,maxv,maxk,nvar,
     *           modtype,modk,modn,modv,dependent,ivar)
         else
            write(*,100) 'dX(',ivar,')/dt=0 (defined independent)' 
         endif
 10   continue
 100  format(A,I2,A)
      end


C     Save solution to problem pname in file fname (output)
      subroutine savemodel(nvar,nexp,pname,fname,error,res)

      implicit none

      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)

      character*30 fname,pname
      integer nvar,ivar,nexp,i
      double precision error,res
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent

      integer u
      parameter (u=30)

      character*30 getNameOfReactionType
      character*9 buff
      character*8 buff2,buff3,buff4,buff5,buff6 
      integer p1,p2,p11,p22,iv,getVsOfType,getKsOfType,jj,ireact,k2

C     gfortran
      character(8)  :: date
      character(10) :: time
      character(5)  :: zone
      integer,dimension(8) :: values
      
      
      double precision s        ! smoothiong parameter
      integer RAND(2)           ! Random starts in estvar
      integer SEED
      
      double precision modkinit(maxexp,maxvar,3) 
      logical initconc          ! T if init conc consider as parameter, ow F
      
      common/myrandomstarts/RAND
      common/mysmoothingpara/s
      common/myrandomseed/SEED
      common/mydata3/modk,modkinit,modn,modv,modtype,dependent
      common/myinitconc/initconc



      fname(1:4)='OUT_'
      fname(5:)=pname
      k2 = index(fname(1:), ' ') ! find next blank 
      fname(k2:k2+1)='_' 
      k2=k2+1
      call int2string(SEED,buff2,p1,p2)
      fname(k2:k2+p2-p1+1)=buff2(p1:p2)
      !write(*,*) fname


      open (u,FILE=fname,STATUS='UNKNOWN') ! Open the data file
  
      write(u,*) 'format version = 1.0'
      write(u,*) ''
      write(u,*) ''
      write(u,*) 'begin solution of problem ',pname
      write(u,*) ''
C     g77
      !call date(buff)
      !call time(buff2)
      !write(u,*) 'date = ',buff,' ',buff2  
C     gfortran
      call date_and_time(date,time,zone,values)
      write(u,*) 'date = ',date,' ',time(1:4)
      write(u,*) 'url = http://www.odeidentification.org'
      write(u,*) ''   
      write(u,*)
     $     '// Settings for the Gennemark & Wedelin (2007) algorithm'
      write(u,*) 'randomStarts_ = ',RAND(1),' ',RAND(2)
      write(u,*) 'smoothingParameter = ',s
      write(u,*) 'randomSeed = ',SEED
      write(u,*) ''

      do 10 ivar=1,nvar
         if (dependent(ivar)) then
            write(u,*) ''
            write(u,*) ''
            write(u,*) '// VARIABLE ',ivar
            write(u,*) ' '
            do 1300 ireact=1,modn(ivar)
               call int2string(ireact,buff2,p1,p2)
               call int2string(ivar,buff3,p11,p22)
               write(u,1350) 'reaction_',buff2(p1:p2),
     *                    ' of variable_',buff3(p11:p22)
 1350          format(A,A,A,A) 
               write(u,1360)'has type = '
     $              ,getNameOfReactionType(modtype(ivar,ireact))
 1360          format(A,A)
               do 200 iv=1,getVsOfType(modtype(ivar,ireact))
                  call int2string(modv(ivar,ireact,iv),buff3,p11,p22)
                  call int2string(iv,buff2,p1,p2)
                  write(u,1370)'has variable X',buff2(p1:p2)
     $                 ,' = variable_',buff3(p11:p22)
 1370             format(A,A,A,A)
 200           enddo
               do 300 iv=1,getKsOfType(modtype(ivar,ireact))
                  call int2string(iv,buff2,p1,p2)
                  write(u,1380)'has parameter k',buff2(p1:p2),' = '
     $                 ,modk(ivar,ireact,iv,1)
 1380             format(A,A,A,D20.12)
 300           enddo
               write(u,*) ' '
         
      
 1300       enddo
         endif
 10   continue


      write(u,*) ''
      write(u,*) ''
      write(u,*) '// INITIAL VALUES'
      if (initconc) then
         write(u,*) ''
         do 1650 i= 1, nexp  
            do 647 ivar= 1, nvar
               if (dependent(ivar)) then
                  call int2string(i,buff2,p1,p2) 
                  call int2string(ivar,buff3,p11,p22) 
                  write(u,654) 'variable_',buff3(p11:p22),
     %              ' of experiment_',buff2(p1:p2)
                  write(u,655)'has initialValue = ',modkinit(i,ivar
     $                 ,1)
                  write(u,*) ''
 654              format(A,A,A,A)
 655              format(A,100D14.4)
               endif
 647        enddo
 1650     enddo
      else ! perfect data, not needed
         write(u,*) '// Perfect data, not required'
      endif



      write(u,*) ''
      write(u,*) ''
      if (error.gt.res*1.000000001) then
         write(u,*) '// ERROR '
         write(u,*) '// error = -L + lambda * K'
         write(u,*) '// residual = -L'
         write(u,*) 'error = ',error 
         write(u,*) 'residual = ',res
      else
         write(u,*) '// ERROR '
         write(u,*) '// residual = -L'
         write(u,*) 'residual = ',res
      endif
      write(u,*) ' '
      write(u,*) 'end of solution of problem ',pname
      close(u)
      end
      


C     Print inital conc parameters
      subroutine printmodkinit(maxvar,maxexp,nvar,nexp,
     *     modkinit,dependent,type)

      implicit none
      integer maxvar,maxexp,nvar,nexp,type
      integer iexp,ivar
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]

      logical dependent(maxvar) 
C     Which variables are dependent
      write(*,*) 'modkinit'
      do 10 ivar=1,nvar
         if (dependent(ivar)) then
            write(*,100) 'ivar ',ivar,' ',(modkinit(iexp,ivar,type),iexp
     $           =1,min(6,nexp)) 
            if (nexp.gt.6) write(*,110) '        ',(modkinit(iexp,ivar
     $           ,type),iexp =7,min(12,nexp))
         endif
 10   continue

 100  format(A,I2,A,12D10.3)
 110  format(A,12D10.3)
      end








      
      
       

C     Get number of model parameters
C     NOT initial values
      SUBROUTINE getKn(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modn,modv,dependent,n)
      
      implicit none
      integer maxvar,maxr,maxv,maxk,nvar
      integer ivar,n,ireact,j,iexp
      integer getKsOfType

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      n=0
      do 100 ivar=1,nvar
         if (dependent(ivar)) then
            do 60 ireact=1,modn(ivar)
               n=n+getKsOfType(modtype(ivar,ireact))
 60         continue
         endif
 100  continue
      end
      
C     Get number of model parameters
C     NOT initial values
      SUBROUTINE getKnvars(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modn,modv,dependent,n,var)
      
      implicit none
      integer maxvar,maxr,maxv,maxk,nvar
      integer ivar,n,ireact,j,iexp
      integer getKsOfType

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent,n)

      logical var(maxvar)
    
      n=0
      do 100 ivar=1,nvar
         if (var(ivar)) then
            if (dependent(ivar)) then
               do 60 ireact=1,modn(ivar)
                  n=n+getKsOfType(modtype(ivar,ireact))
 60            continue
            endif
         endif
 100  continue
      end
      

C     Get graph (nvar*nvar). Indicate if two variables share parameters
      subroutine getGraph(graph,nexp,nvar)
      implicit none
      integer maxexp,maxvar,maxpts,maxr,maxv,maxk
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)

      integer nvar,ipara,ivar,nexp
      logical graph(maxvar,maxvar)
     
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType
      integer ivar2mem,iantal,ik,iv

      common/mydata3/modk,modkinit,modn,modv,modtype,dependent

      debug=0
      if (debug.gt.0) write(*,*) 'getGraph'
      do 2, ivar=1,nvar
         do 1, ivar2=1,nvar
            graph(ivar,ivar2)=.false.
 1       continue
 2    continue
      do 50, ivar=1,nvar
         do 40 ireact=1,modn(ivar) ! for all reactions
            iantal=1
            do 30, ivar2=1,nvar
               do 20 ireact2=1,modn(ivar2) ! for all reactions
                  if (ivar.ne.ivar2.and.
     *                 modtype(ivar,ireact).eq.modtype(ivar2,ireact2))
     *                 then
                     same=.true. ! same variables?
                     do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                        if (modv(ivar,ireact,iv).ne.
     *                       modv(ivar2,ireact2,iv)) then
                           same=.false.
                        endif
 10                  enddo
                     if (same) then
                        if (modk(ivar,ireact,1,2).ne.
     *                       modk(ivar2,ireact2,1,2)) then ! different bounds 
                           iantal=iantal+1
                           ivar2mem=ivar2 ! remember this reaction
                        endif
                     endif
                  endif
 20            continue
 30         continue
            if (iantal.eq.2.and.ivar.lt.ivar2mem) then
               graph(ivar,ivar2mem)=.true.
            elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
               if (debug.gt.0) write(*,*)
     $              'getGraph, nothing'
            endif
 40      continue
 50   continue
      end



C     nout - the number of parameters to consider
      subroutine getKngeneral(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,
     *     nout)
      implicit none
      integer maxvar,nvar,maxr,maxv,maxk,nin,nout,ipara,ivar,maxexp,nexp
     
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType
      integer ivar2mem,ireact2mem,iantal,ik,iv

      debug=0
      nout=0
      if (debug.gt.0) write(*,*) 'getKngeneral'
      do 50, ivar=1,nvar
         do 40 ireact=1,modn(ivar) ! for all reactions
            iantal=1
            do 30, ivar2=1,nvar
               do 20 ireact2=1,modn(ivar2) ! for all reactions
                  if (ivar.ne.ivar2.and.
     *                 modtype(ivar,ireact).eq.modtype(ivar2,ireact2))
     *                 then
                     same=.true. ! same variables?
                     do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                        if (modv(ivar,ireact,iv).ne.
     *                       modv(ivar2,ireact2,iv)) then
                           same=.false.
                        endif
 10                  enddo
                     if (same) then
                        if (modk(ivar,ireact,1,2).ne.
     *                       modk(ivar2,ireact2,1,2)) then ! different bounds 
                           iantal=iantal+1
                           ivar2mem=ivar2 ! remember this reaction
                           ireact2mem=ireact2
                        endif
                     endif
                  endif
 20            continue
 30         continue
            
            if (iantal.eq.2.and.ivar.lt.ivar2mem) then
               do 35 ik=1,getKsOfType(modtype(ivar,ireact))
                  nout=nout+1
 35            enddo
            elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
               if (debug.gt.0) write(*,*)
     $              'getKngeneral nothing'
            else
               do 36 ik=1,getKsOfType(modtype(ivar,ireact))
                  nout=nout+1
 36            enddo
               if (debug.gt.0) write(*,*)
     $              'getKngeneral simple add, new nout=',nout
            endif
 40      continue
 50   continue
      end






C     nout - the number of parameters to consider
      subroutine getKnvarsgeneral(nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,nexp,
     *     nout)
      implicit none
      integer maxvar,nvar,maxr,maxv,maxk,nin,nout,ivar,maxexp
      integer nexp,maxpts
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical doinit,same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType,iexp
      integer ivar2mem,ireact2mem,iantal,ik,iv

      logical var(maxvar)
      integer idummy
      common/simulg/var,idummy 

      debug=0
      nout=0

      if (debug.gt.0) write(*,*) 'varstoxgeneral'
      do 50, ivar=1,nvar
         if (var(ivar)) then
            do 40 ireact=1,modn(ivar) ! for all reactions
               iantal=1
               do 30, ivar2=1,nvar
                  if (var(ivar2)) then
                     do 20 ireact2=1,modn(ivar2) ! for all reactions
                        if (ivar.ne.ivar2.and.
     *                       modtype(ivar,ireact).eq.
     *                       modtype(ivar2,ireact2))
     *                       then
                           same=.true. ! same variables?
                           do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                              if (modv(ivar,ireact,iv).ne.
     *                             modv(ivar2,ireact2,iv)) then
                                 same=.false.
                              endif
 10                        enddo
                           if (same) then
                              if (modk(ivar,ireact,1,2).ne.
     *                             modk(ivar2,ireact2,1,2)) then ! different bounds 
                                 iantal=iantal+1
                                 ivar2mem=ivar2 ! remember this reaction
                                 ireact2mem=ireact2
                              endif
                           endif
                        endif
 20                  continue
                  endif
 30            continue
            
               if (iantal.eq.2.and.ivar.lt.ivar2mem) then
                  do 35 ik=1,getKsOfType(modtype(ivar,ireact))
                     nout=nout+1
 35               enddo
               elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
                  if (debug.gt.0) write(*,*)
     $                 'varstoxgeneral nothing'
               else
                  do 36 ik=1,getKsOfType(modtype(ivar,ireact))
                     nout=nout+1
 36               enddo
                  if (debug.gt.0) write(*,*)
     $                 'varstoxgeneral simple add, new nout=',nout
               endif
 40         continue
         endif
 50   continue
      end
      






C     X is a vector of all parameters
C     set X values from the model
C     B = bounds
c     n = number of parameters
      SUBROUTINE modeltox(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,X,B,n)
      
      implicit none
      integer maxvar,maxr,maxv,maxk,nvar,nexp,maxexp
      integer ivar,n,ipara,ireact,j,iexp
      integer getKsOfType

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      double precision X(n),B(2,n)
      ipara=1
      do 100 ivar=1,nvar
         if (dependent(ivar)) then
            do 60 ireact=1,modn(ivar)
               do 50 j=1,getKsOfType(modtype(ivar,ireact))
                  X(ipara)=modk(ivar,ireact,j,1) 
                  B(1,ipara)=modk(ivar,ireact,j,2)
                  B(2,ipara)=modk(ivar,ireact,j,3)
                  ipara=ipara+1
 50            continue
 60         continue
         endif
 100  continue
      if (ipara.eq.(n+1)) then  ! ok stop
         
      else                      ! consider initial values as parameters
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (dependent(ivar)) then
                  X(ipara)=modkinit(iexp,ivar,1) 
                  B(1,ipara)=modkinit(iexp,ivar,2)
                  B(2,ipara)=modkinit(iexp,ivar,3)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      if (ipara.ne.(n+1)) then
         write(*,*) 'modeltox, error, n=',n,' ipara=',ipara
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         stop
      endif
      end



C     X is a vector of all parameters
C     set X values from the submodel defined by common var(maxvar)
C     B = bounds
c     n = number of parameters
      SUBROUTINE varstox(nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,nexp,X,B,n)
      

      implicit none
      integer maxvar,maxr,maxv,maxk,nvar,nexp,maxexp,maxpts
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer ivar,n,ipara,ireact,j,iexp,idummy
      integer getKsOfType

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      double precision X(n),B(2,n)

      logical var(maxvar)
      common/simulg/var,idummy  

      ipara=1
      do 100 ivar=1,nvar
         if (var(ivar).and.dependent(ivar)) then
            do 60 ireact=1,modn(ivar)
               do 50 j=1,getKsOfType(modtype(ivar,ireact))
                  X(ipara)=modk(ivar,ireact,j,1) 
                  B(1,ipara)=modk(ivar,ireact,j,2)
                  B(2,ipara)=modk(ivar,ireact,j,3)
                  ipara=ipara+1
 50            continue
 60         continue
         endif
 100  continue
      if (ipara.eq.(n+1)) then  ! ok stop
         
      else                      ! consider initial values as parameters
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (var(ivar).and.dependent(ivar)) then
                  X(ipara)=modkinit(iexp,ivar,1) 
                  B(1,ipara)=modkinit(iexp,ivar,2)
                  B(2,ipara)=modkinit(iexp,ivar,3)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      if (ipara.ne.(n+1)) then
         write(*,*) 'varstox, error, n=',n,' ipara=',ipara
         write(*,*) 'varstox, var=',var
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         stop
      endif
      end



C     X is a vector of all parameters
C     set X values to the model
      SUBROUTINE xtomodel (maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,X,n)

      implicit none
      integer maxvar,maxr,maxv,maxk,nvar,n,ipara,ivar,maxexp,nexp
      integer getKsOfType,j,ireact,iexp
      double precision X(n)
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
    
      ipara=1
      do 100, ivar=1,nvar
         if (dependent(ivar)) then
            do 60 ireact=1,modn(ivar)
               do 50 j=1,getKsOfType(modtype(ivar,ireact))
                  modk(ivar,ireact,j,1)=X(ipara)
                  ipara=ipara+1
 50            continue
 60         continue
         endif
 100  continue
      if (ipara.eq.(n+1)) then  ! ok stop
         
      else                      ! consider initial values as parameters
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (dependent(ivar)) then
                  modkinit(iexp,ivar,1)=X(ipara)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      if (ipara.ne.(n+1)) then
         write(*,*) 'xtomodel, error, n=',n,' ipara=',ipara
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         stop
      endif
      end



C     X is a vector of all parameters
C     set X values to the submodel defined by common var(maxvar)
      SUBROUTINE xtovars (nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,nexp,X,n)

      implicit none
      integer maxvar,maxr,maxv,maxk,nvar,n,ipara,ivar,maxexp,maxpts,nexp
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      integer getKsOfType,j,ireact,iexp
      double precision X(n)
C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
      logical var(maxvar)
      integer idummy
      common/simulg/var,idummy 
    
      ipara=1
      do 100, ivar=1,nvar
         if (var(ivar).and.dependent(ivar)) then
            do 60 ireact=1,modn(ivar)
               do 50 j=1,getKsOfType(modtype(ivar,ireact))
                  modk(ivar,ireact,j,1)=X(ipara)
                  ipara=ipara+1
 50            continue
 60         continue
         endif
 100  continue
      if (ipara.eq.(n+1)) then  ! ok stop
         
      else                      ! consider initial values as parameters
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (var(ivar).and.dependent(ivar)) then
                  modkinit(iexp,ivar,1)=X(ipara)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      if (ipara.ne.(n+1)) then
         write(*,*) 'xtomodel, error, n=',n,' ipara=',ipara
         write(*,*) 'xtovars, var=',var
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         stop
      endif
      end

C     nin - 1st call: a high number, ow the number of parameters to consider
C     nout - the number of parameters to consider
C     doinit - True if initial conc are parameters, ow false
      subroutine modeltoxgeneral(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,
     *     X,B,nin,nout,
     *     doinit)
      implicit none
      integer maxvar,nvar,maxr,maxv,maxk,nin,nout,ipara,ivar,maxexp,nexp
      double precision X(nin),B(2,nin)

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical doinit,same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType,iexp
      integer ivar2mem,ireact2mem,iantal,ik,iv

      debug=0
      ipara=1

      if (debug.gt.0) write(*,*) 'modeltoxgeneral'
      do 50, ivar=1,nvar
         do 40 ireact=1,modn(ivar) ! for all reactions
            iantal=1
            do 30, ivar2=1,nvar
               do 20 ireact2=1,modn(ivar2) ! for all reactions
                  if (ivar.ne.ivar2.and.
     *                 modtype(ivar,ireact).eq.modtype(ivar2,ireact2))
     *                 then
                     same=.true. ! same variables?
                     do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                        if (modv(ivar,ireact,iv).ne.
     *                       modv(ivar2,ireact2,iv)) then
                           same=.false.
                        endif
 10                  enddo
                     if (same) then
                        if (modk(ivar,ireact,1,2).ne.
     *                       modk(ivar2,ireact2,1,2)) then ! different bounds 
                           iantal=iantal+1
                           ivar2mem=ivar2 ! remember this reaction
                           ireact2mem=ireact2
                        endif
                     endif
                  endif
 20            continue
 30         continue
            
            if (iantal.eq.2.and.ivar.lt.ivar2mem) then
               do 35 ik=1,getKsOfType(modtype(ivar,ireact))
                  X(ipara)=(abs(modk(ivar,ireact,ik,1))
     $                 +abs(modk(ivar2mem,ireact2mem,ik,1)))/2
                  if (debug.gt.0) write(*,*) 'modeltoxgeneral X(',ipara,
     $                 ')=',X(ipara)
                  B(1,ipara)=max(modk(ivar,ireact,ik,2),modk(ivar2mem
     $                 ,ireact2mem,ik,2))
                  B(2,ipara)=max(modk(ivar,ireact,ik,3),modk(ivar2mem
     $                 ,ireact2mem,ik,3))
                  ipara=ipara+1
 35            enddo
            elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
               if (debug.gt.0) write(*,*)
     $              'modeltoxgeneral nothing'
            else
               do 36 ik=1,getKsOfType(modtype(ivar,ireact))
                  X(ipara)=modk(ivar,ireact,ik,1)
                  if (debug.gt.0) write(*,*) 'modeltoxgeneral X(',ipara,
     $                 ')=',X(ipara)
                  B(1,ipara)=modk(ivar,ireact,ik,2)
                  B(2,ipara)=modk(ivar,ireact,ik,3)
                  ipara=ipara+1
 36            enddo
               if (debug.gt.0) write(*,*)
     $              'modeltoxgeneral simple add, new ipara=',ipara
            endif
 40      continue
 50   continue

      if (doinit) then
        do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (dependent(ivar)) then
                  X(ipara)=modkinit(iexp,ivar,1) 
                  B(1,ipara)=modkinit(iexp,ivar,2)
                  B(2,ipara)=modkinit(iexp,ivar,3)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      !if (debug.gt.0) write(*,*) 'modeltoxgeneral',ipara,nin
      nout=ipara-1
      !if (debug.gt.0) write(*,*) 'modeltoxgeneral nout=',nout
      end





C     nin - 1st call: a high number, ow the number of parameters to consider
C     nout - the number of parameters to consider
C     doinit - True if initial conc are parameters, ow false
      subroutine varstoxgeneral(nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,nexp,
     *     X,B,nin,nout,
     *     doinit)
      implicit none
      integer maxvar,nvar,maxr,maxv,maxk,nin,nout,ipara,ivar,maxexp
      integer nexp,maxpts
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      double precision X(nin),B(2,nin)

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical doinit,same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType,iexp
      integer ivar2mem,ireact2mem,iantal,ik,iv

      logical var(maxvar)
      integer idummy
      common/simulg/var,idummy 

      debug=0
      ipara=1

      if (debug.gt.0) write(*,*) 'varstoxgeneral'
      do 50, ivar=1,nvar
         if (var(ivar)) then
            do 40 ireact=1,modn(ivar) ! for all reactions
               iantal=1
               do 30, ivar2=1,nvar
                  if (var(ivar2)) then
                     do 20 ireact2=1,modn(ivar2) ! for all reactions
                        if (ivar.ne.ivar2.and.
     *                       modtype(ivar,ireact).eq.
     *                       modtype(ivar2,ireact2))
     *                       then
                           same=.true. ! same variables?
                           do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                              if (modv(ivar,ireact,iv).ne.
     *                             modv(ivar2,ireact2,iv)) then
                                 same=.false.
                              endif
 10                        enddo
                           if (same) then
                              if (modk(ivar,ireact,1,2).ne.
     *                             modk(ivar2,ireact2,1,2)) then ! different bounds 
                                 iantal=iantal+1
                                 ivar2mem=ivar2 ! remember this reaction
                                 ireact2mem=ireact2
                              endif
                           endif
                        endif
 20                  continue
                  endif
 30            continue
            
               if (iantal.eq.2.and.ivar.lt.ivar2mem) then
                  do 35 ik=1,getKsOfType(modtype(ivar,ireact))
                     X(ipara)=(abs(modk(ivar,ireact,ik,1))
     $                    +abs(modk(ivar2mem,ireact2mem,ik,1)))/2
                     if (debug.gt.0) write(*,*) 'varstoxgeneral X(',
     *                    ipara,
     $                    ')=',X(ipara)
                     B(1,ipara)=max(modk(ivar,ireact,ik,2),modk(ivar2mem
     $                    ,ireact2mem,ik,2))
                     B(2,ipara)=max(modk(ivar,ireact,ik,3),modk(ivar2mem
     $                    ,ireact2mem,ik,3))
                     ipara=ipara+1
 35               enddo
               elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
                  if (debug.gt.0) write(*,*)
     $                 'varstoxgeneral nothing'
               else
                  do 36 ik=1,getKsOfType(modtype(ivar,ireact))
                     X(ipara)=modk(ivar,ireact,ik,1)
                     if (debug.gt.0) write(*,*) 'varstoxgeneral X(',
     *                    ipara,
     $                    ')=',X(ipara)
                     B(1,ipara)=modk(ivar,ireact,ik,2)
                     B(2,ipara)=modk(ivar,ireact,ik,3)
                     ipara=ipara+1
 36               enddo
                  if (debug.gt.0) write(*,*)
     $                 'varstoxgeneral simple add, new ipara=',ipara
               endif
 40         continue
         endif
 50   continue

      if (doinit) then
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (var(ivar).and.dependent(ivar)) then
                  X(ipara)=modkinit(iexp,ivar,1) 
                  B(1,ipara)=modkinit(iexp,ivar,2)
                  B(2,ipara)=modkinit(iexp,ivar,3)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      !if (debug.gt.0) write(*,*) 'varstoxgeneral',ipara,nin
      nout=ipara-1
      !if (debug.gt.0) write(*,*) 'varstoxgeneral nout=',nout
      end
      


C     n - the number of parameters to consider
C     doinit - True if initial conc are parameters, ow false
      SUBROUTINE xtomodelgeneral(maxvar,maxr,maxv,maxk,nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,maxexp,nexp,
     *     X,n,
     *     doinit)   
      implicit none
      integer maxvar,nvar,maxr,maxv,maxk,n,ipara,ivar,maxexp,nexp
      double precision X(n)

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical doinit,same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType,iexp
      integer ivar2mem,ireact2mem,iantal,ik,iv



      debug=0
      ipara=1
      if (debug.gt.0) write(*,*) 'xtomodelgeneral'
      do 50, ivar=1,nvar
         do 40 ireact=1,modn(ivar) ! for all reactions
            iantal=1
            do 30, ivar2=1,nvar
               do 20 ireact2=1,modn(ivar2) ! for all reactions
                  if (ivar.ne.ivar2.and.
     *                 modtype(ivar,ireact).eq.modtype(ivar2,ireact2))
     *                 then
                     same=.true. ! same variables?
                     do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                        if (modv(ivar,ireact,iv).ne.
     *                       modv(ivar2,ireact2,iv)) then
                           same=.false.
                        endif
 10                  enddo
                     if (same) then
                        if (modk(ivar,ireact,1,2).ne.
     *                       modk(ivar2,ireact2,1,2)) then ! different bounds 
                           iantal=iantal+1
                           ivar2mem=ivar2 ! remember this reaction
                           ireact2mem=ireact2
                        endif
                     endif
                  endif
 20            continue
 30         continue
            
            if (iantal.eq.2.and.ivar.lt.ivar2mem) then
               do 35 ik=1,getKsOfType(modtype(ivar,ireact))
                  if (modk(ivar,ireact,ik,2).ge.0.d0) then
                     modk(ivar,ireact,ik,1)=X(ipara)
                  else
                     modk(ivar,ireact,ik,1)=-X(ipara)
                  endif
                  if (modk(ivar2mem,ireact2mem,ik,2).ge.0.d0) then
                     modk(ivar2mem,ireact2mem,ik,1)=X(ipara)
                  else
                     modk(ivar2mem,ireact2mem,ik,1)=-X(ipara)
                  endif
                  ipara=ipara+1
 35            enddo
            elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
               if (debug.gt.0) write(*,*)
     $              'xtomodelgeneral nothing'
            else
               do 36 ik=1,getKsOfType(modtype(ivar,ireact))
                  modk(ivar,ireact,ik,1)=X(ipara)
                  ipara=ipara+1
 36            enddo
               if (debug.gt.0) write(*,*)
     $              'xtomodelgeneral simple add, new ipara=',ipara
            endif
 40      continue
 50   continue
      if (doinit) then
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (dependent(ivar)) then
                  modkinit(iexp,ivar,1)=X(ipara)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      if (ipara.ne.(n+1)) then
         write(*,*) 'xtomodelgeneral, error, n=',n,' ipara=',ipara
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         stop
      endif
      end




C     n - the number of parameters to consider
C     doinit - True if initial conc are parameters, ow false
      SUBROUTINE xtovarsgeneral(nvar,
     *     modtype,modk,modkinit,modn,modv,dependent,nexp,
     *     X,n,
     *     doinit)   
      implicit none
      integer maxvar,nvar,maxr,maxv,maxk,n,ipara,ivar,maxexp
      integer maxpts, nexp
      parameter (maxexp=20,maxvar=19,maxpts=201,maxr=10,maxv=3,maxk=5)
      double precision X(n)

C     ------ MODEL ------
      integer modtype(maxvar,maxr) 
C     Type of reaction [1...maxr] for variable [1...maxvar]
      double precision modk(maxvar,maxr,maxk,3) 
C     Last index=1: Parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=2: Lower bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
C     Last index=3: Upper bound for parameter [1...maxk] for reaction [1...maxr] in variable [1...maxvar]
      double precision modkinit(maxexp,maxvar,3) 
C     Last index=1: Init conc for experiment [1...maxexp], variable [1...maxvar]
C     Last index=2: Lower bound for experiment [1...maxexp], variable [1...maxvar]
C     Last index=3: Upper bound for experiment [1...maxexp], variable [1...maxvar]
      integer modn(maxvar) 
C     Number of reactions for variable [1...maxvar]
      integer modv(maxvar,maxr,maxv)
C     Variable [1...maxv] for reaction [1...maxr] in variable [1...maxvar]
      logical dependent(maxvar) 
C     Which variables are dependent
 
      integer debug
      logical doinit,same
      integer ivar2,ireact,ireact2,getVsOfType,getKsOfType,iexp
      integer ivar2mem,ireact2mem,iantal,ik,iv

      logical var(maxvar)
      integer idummy
      common/simulg/var,idummy 


      debug=0
      ipara=1
      if (debug.gt.0) write(*,*) 'xtovarsgeneral, var=',var
      do 50, ivar=1,nvar
         if (var(ivar)) then
            do 40 ireact=1,modn(ivar) ! for all reactions
               iantal=1
               do 30, ivar2=1,nvar
                  if (var(ivar2)) then
                     do 20 ireact2=1,modn(ivar2) ! for all reactions
                        if (ivar.ne.ivar2.and.
     *                       modtype(ivar,ireact).eq.
     *                       modtype(ivar2,ireact2))
     *                       then
                           same=.true. ! same variables?
                           do 10, iv=1,getVsOfType(modtype(ivar,ireact))
                              if (modv(ivar,ireact,iv).ne.
     *                             modv(ivar2,ireact2,iv)) then
                                 same=.false.
                              endif
 10                        enddo
                           if (same) then
                              if (modk(ivar,ireact,1,2).ne.
     *                             modk(ivar2,ireact2,1,2)) then ! different bounds 
                                 iantal=iantal+1
                                 ivar2mem=ivar2 ! remember this reaction
                                 ireact2mem=ireact2
                              endif
                           endif
                        endif
 20                  continue
                  endif
 30            continue
               
               if (iantal.eq.2.and.ivar.lt.ivar2mem) then
                  do 35 ik=1,getKsOfType(modtype(ivar,ireact))
                     if (modk(ivar,ireact,ik,2).ge.0.d0) then
                        modk(ivar,ireact,ik,1)=X(ipara)
                     else
                        modk(ivar,ireact,ik,1)=-X(ipara)
                     endif
                     if (modk(ivar2mem,ireact2mem,ik,2).ge.0.d0) then
                        modk(ivar2mem,ireact2mem,ik,1)=X(ipara)
                     else
                        modk(ivar2mem,ireact2mem,ik,1)=-X(ipara)
                     endif
                     ipara=ipara+1
 35               enddo
               elseif (iantal.eq.2.and.ivar.ge.ivar2mem) then ! nothing, already taken
                  if (debug.gt.0) write(*,*)
     $                 'xtovarsgeneral nothing'
               else
                  do 36 ik=1,getKsOfType(modtype(ivar,ireact))
                     modk(ivar,ireact,ik,1)=X(ipara)
                     ipara=ipara+1
 36               enddo
                  if (debug.gt.0) write(*,*)
     $                 'xtovarsgeneral simple add, new ipara=',ipara
               endif
 40      continue
      endif
 50   continue
      if (doinit) then
         do 300 iexp=1,nexp
            do 200 ivar=1,nvar
               if (var(ivar).and.dependent(ivar)) then
                  modkinit(iexp,ivar,1)=X(ipara)
                  ipara=ipara+1
               endif
 200        continue
 300     continue
      endif
      if (ipara.ne.(n+1)) then
         write(*,*) 'xtovarsgeneral, error, n=',n,' ipara=',ipara
         call printmodel(maxvar,maxr,maxv,maxk,nvar,
     *        modtype,modk,modn,modv,dependent)
         stop
      endif
      end
      

      




