  
      SUBROUTINE DUMMY 
      RETURN 
      END 

*DECK ICOPY
      SUBROUTINE ICOPY (N, IX, INCX, IY, INCY)
C***BEGIN PROLOGUE  ICOPY
C***PURPOSE  Copy a vector.
C***LIBRARY   SLATEC (BLAS)
C***CATEGORY  D1A5
C***TYPE      INTEGER (ICOPY-S, DCOPY-D, CCOPY-C, ICOPY-I)
C***KEYWORDS  BLAS, COPY, LINEAR ALGEBRA, VECTOR
C***AUTHOR  Boland, W. Robert, (LANL)
C           Clemens, Reginald, (PLK)
C***DESCRIPTION
C
C                B L A S  Subprogram
C    Description of Parameters
C
C     --Input--
C        N  number of elements in input vector(s)
C       IX  integer vector with N elements
C     INCX  storage spacing between elements of IX
C       IY  integer vector with N elements
C     INCY  storage spacing between elements of IY
C
C     --Output--
C       IY  copy of vector IX (unchanged if N .LE. 0)
C
C     Copy integer IX to integer IY.
C     For I = 0 to N-1, copy  IX(LX+I*INCX) to IY(LY+I*INCY),
C     where LX = 1 if INCX .GE. 0, else LX = 1+(1-N)*INCX, and LY is
C     defined in a similar way using INCY.
C
C***REFERENCES  C. L. Lawson, R. J. Hanson, D. R. Kincaid and F. T.
C                 Krogh, Basic linear algebra subprograms for Fortran
C                 usage, Algorithm No. 539, Transactions on Mathematical
C                 Software 5, 3 (September 1979), pp. 308-323.
C***ROUTINES CALLED  (NONE)
C***REVISION HISTORY  (YYMMDD)
C   930201  DATE WRITTEN
C***END PROLOGUE  ICOPY
      INTEGER IX(*), IY(*)
C***FIRST EXECUTABLE STATEMENT  ICOPY
      IF (N .LE. 0) RETURN
      IF (INCX .EQ. INCY) IF (INCX-1) 5,20,60
C
C     Code for unequal or nonpositive increments.
C
    5 IIX = 1
      IIY = 1
      IF (INCX .LT. 0) IIX = (-N+1)*INCX + 1
      IF (INCY .LT. 0) IIY = (-N+1)*INCY + 1
      DO 10 I = 1,N
        IY(IIY) = IX(IIX)
        IIX = IIX + INCX
        IIY = IIY + INCY
   10 CONTINUE
      RETURN
C
C     Code for both increments equal to 1.
C
C     Clean-up loop so remaining vector length is a multiple of 7.
C
   20 M = MOD(N,7)
      IF (M .EQ. 0) GO TO 40
      DO 30 I = 1,M
        IY(I) = IX(I)
   30 CONTINUE
      IF (N .LT. 7) RETURN
   40 MP1 = M + 1
      DO 50 I = MP1,N,7
        IY(I) = IX(I)
        IY(I+1) = IX(I+1)
        IY(I+2) = IX(I+2)
        IY(I+3) = IX(I+3)
        IY(I+4) = IX(I+4)
        IY(I+5) = IX(I+5)
        IY(I+6) = IX(I+6)
   50 CONTINUE
      RETURN
C
C     Code for equal, positive, non-unit increments.
C
   60 NS = N*INCX
      DO 70 I = 1,NS,INCX
        IY(I) = IX(I)
   70 CONTINUE
      RETURN
      END
      subroutine  dcopy(n,dx,incx,dy,incy)
c
c     copies a vector, x, to a vector, y.
c     uses unrolled loops for increments equal to one.
c     jack dongarra, linpack, 3/11/78.
c     modified 12/3/93, array(1) declarations changed to array(*)
c
      double precision dx(*),dy(*)
      integer i,incx,incy,ix,iy,m,mp1,n
c
      if(n.le.0)return
      if(incx.eq.1.and.incy.eq.1)go to 20
c
c        code for unequal increments or equal increments
c          not equal to 1
c
      ix = 1
      iy = 1
      if(incx.lt.0)ix = (-n+1)*incx + 1
      if(incy.lt.0)iy = (-n+1)*incy + 1
      do 10 i = 1,n
        dy(iy) = dx(ix)
        ix = ix + incx
        iy = iy + incy
   10 continue
      return
c
c        code for both increments equal to 1
c
c
c        clean-up loop
c
   20 m = mod(n,7)
      if( m .eq. 0 ) go to 40
      do 30 i = 1,m
        dy(i) = dx(i)
   30 continue
      if( n .lt. 7 ) return
   40 mp1 = m + 1
      do 50 i = mp1,n,7
        dy(i) = dx(i)
        dy(i + 1) = dx(i + 1)
        dy(i + 2) = dx(i + 2)
        dy(i + 3) = dx(i + 3)
        dy(i + 4) = dx(i + 4)
        dy(i + 5) = dx(i + 5)
        dy(i + 6) = dx(i + 6)
   50 continue
      return
      end


C     Read integer from character array cline
C     Start reading at position pos
C     Return integer value val and the start position posnext for
C     reading the next item (separated by spaces)
      subroutine nextInt(cline,pos,posnext,val)
      implicit none
      character cline*1000 
      integer pos,posnext,val,k2
      integer kode,pos2,index
C     skip initial blanks
 10   k2 = index(cline(pos:), ' ')  
      if (k2.eq.1) then ! current position is space
         pos=pos+1
         goto 10
      endif
      pos2 = index(cline(pos+1:),' ') 
      !write(*,*) 'pos2=',pos2
      posnext=pos+pos2+1
      read(cline(pos:posnext-1), '(BN,I20.0)', iostat=kode) val
      if(kode .ne. 0) then 
         write(*,*)'nextInt, error, cline=', cline,' pos=',pos
         stop
      endif 
      end

C     Read boolean from character array cline
C     Start reading at position pos
C     Return boolean value val and the start position posnext for
C     reading the next item (separated by spaces)
      subroutine nextBool(cline,pos,posnext,val)
      implicit none
      character cline*1000,temp*10 
      integer pos,posnext,k2
      logical val
      integer kode,pos2,index
C     skip initial blanks
 10   k2 = index(cline(pos:), ' ')  
      if (k2.eq.1) then ! current position is space
         pos=pos+1
         goto 10
      endif
      pos2 = index(cline(pos+1:),' ') 
      !write(*,*) 'pos2=',pos2
      posnext=pos+pos2+1
      read(cline(pos:posnext-1), '(BN,A)', iostat=kode) temp
      if(kode .ne. 0) then 
         write(*,*)'nextBool, error, cline=',cline,' pos=',pos
         stop
      endif 
      !write(*,*) 'temp=',temp 
      if (temp.eq.'T'.or.temp.eq.'TRUE'.or.
     *     temp.eq.'t'.or.temp.eq.'true') then
         val=.true.
      else if (temp.eq.'F'.or.temp.eq.'FALSE'.or.
     *        temp.eq.'f'.or.temp.eq.'false') then
         val=.false.
      else
         write(*,*)'nextBool, error, cline=',cline,' pos=',pos
         write(*,*)'nextBool, must be TRUE, T, FALSE, F'
         write(*,*)'nextBool,      or true, t, false, f'
         stop
      endif
      end

C     Read string (character*50) from character array cline
C     Start reading at position pos
C     Return val and the start position posnext for
C     reading the next item (separated by spaces)
      subroutine nextString(cline,pos,posnext,val)
      implicit none
      character cline*1000,val*50 
      integer pos,posnext,k2
      integer kode,pos2,index
C     skip initial blanks
 10   k2 = index(cline(pos:), ' ')  
      !write(*,*) 'k2=',k2,' pos=',pos,' ',cline(1:20)
      if (k2.eq.1) then ! current position is space
         pos=pos+1
         goto 10
      endif
      pos2 = index(cline(pos+1:),' ') 
      !write(*,*) 'pos2=',pos2
      posnext=pos+pos2+1
      !write(*,*) 'pos=',pos
      !write(*,*) 'posnext=',posnext
      if (posnext-1.le.1000) then
         val=cline(pos:posnext-1)
      else
         val=' '
      endif
      !write(*,*) 'val=',val
      end




      
C     Read double precision value from character array cline
C     Start reading at position pos
C     Return double precision value val and the start position posnext for
C     reading the next item (separated by spaces)
      subroutine nextDouble(cline,pos,posnext,val)
      implicit none
      character cline*1000 
      integer pos,posnext,k2
      double precision val
      integer kode,pos2,index
C     skip initial blanks
 10   k2 = index(cline(pos:), ' ')  
      if (k2.eq.1) then ! current position is space
         pos=pos+1
         goto 10
      endif
      pos2 = index(cline(pos+1:),' ') 
      !write(*,*) 'pos2=',pos2
      posnext=pos+pos2+1
      read(cline(pos:posnext-1), '(BN,F24.0)', iostat=kode) val
      if(kode .ne. 0) then 
         write(*,*)'nextDouble, error, cline=', cline,' pos=',pos
         stop
      endif 
      end


C     a the integer
C     b char array size 10
C     p1 postion to start in
C     p2 postion to stop in
      subroutine int2string(a,buff,p1,p2)

      integer a,p1,p2
      character*24 bufftemp
      character*8 buff

      integer pos,k2,pos2,posnext
      
      write(bufftemp,*) a
      !write(*,*) 'buff=',buff
      pos=1                     !skip initial blanks 
 10   k2 = index(bufftemp(pos:), ' ') 
      !write(*,*) 'k2 ',k2 
      if (k2.eq.1) then ! current position is space
         pos=pos+1
         goto 10
      endif
      !write(*,*) 'pos ',pos
      pos2 = index(bufftemp(pos+1:),' ') 
      !write(*,*) 'pos2=',pos2
      posnext=pos+pos2
      p1=pos
      p2=posnext-1

      buff(1:p2-p1+1)=bufftemp(p1:p2)
      p2=p2-p1+1
      p1=1
      !write(*,*) 'int2str_',buff(pos:posnext-1),'_',buff(pos:posnext-1)
     
      end



c$$$C     a the integer
c$$$C     b char array size 10
c$$$C     p1 postion to start in
c$$$C     p2 postion to stop in
c$$$      subroutine int2string(a,buff,p1,p2)
c$$$
c$$$      integer a,p1,p2
c$$$      character*8 buff
c$$$
c$$$      integer pos,k2,pos2,posnext
c$$$      
c$$$      write(*,*) 'a is ',a
c$$$      write(buff,*) a
c$$$      !write(*,*) 'buff=',buff
c$$$      pos=1                     !skip initial blanks 
c$$$ 10   k2 = index(buff(pos:), ' ') 
c$$$      !write(*,*) 'k2 ',k2 
c$$$      if (k2.eq.1) then ! current position is space
c$$$         pos=pos+1
c$$$         goto 10
c$$$      endif
c$$$      !write(*,*) 'pos ',pos
c$$$      pos2 = index(buff(pos+1:),' ') 
c$$$      !write(*,*) 'pos2=',pos2
c$$$      posnext=pos+pos2
c$$$      p1=pos
c$$$      p2=posnext-1
c$$$      !write(*,*) 'int2str_',buff(pos:posnext-1),'_',buff(pos:posnext-1)
c$$$     
c$$$      end
