/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/



package Java;

import java.io.*;
import Java.debug.*;
import Java.solution.*;
import Java.problem.*;


/**
 * Make SBML file of a solution model
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class MakeSBML {
    

/**
 * Make SBML file of a solution model given as argv[0]
 * @param argv array of Strings, first item contains path to solution file
 */
    public static void main(String[] argv){  
        if (argv.length<=2){
	   Debug.error("MakeSBML","main","Three inputs required, see User's guide.");
        }
	Debug.ON = false;
	Debug.ERROR = false;
	Debug.WARNING = false;
	String filetree = System.getProperty("user.dir") + 
	    System.getProperty("file.separator");
	File file=new File(filetree+argv[0]);
	if (file==null){
	    Debug.error("MakeSBML","main","Input problem file not found.");
	}
	Problem p=new Problem();
	p.read(file,0);
	
	// Solution file
	File file2=new File(filetree+argv[1]);
	if (file2==null){
	    Debug.error("MakeSBML","main","Input solution file not found.");
	}
	Solution s=new Solution(p);
	s.read(file2,1);    
	
	
	File file3=new File(filetree+argv[2]);
	if (file3==null){
	    Debug.error("MakeSBML","main","Output solution file not found.");
	}
	s.saveSBML(file3,file2.getName(),1);
    }
    
    
}


