/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
    www.odeidentification.org
*/


package Java;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import javax.swing.border.*;
import javax.swing.JOptionPane.*;
import Java.debug.*;
import Java.misc.*;
import Java.problem.*;
import Java.gui.*;

/**
 * Plot an experiment from a problem file
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class PlotExperiment extends JFrame implements ActionListener, ItemListener{

    Problem p;
    Experiment experiment;//the active experiment
    
    GraphPanel graphPanel; // Graph of exp data
    JButton scaleUpButton;//scale up sSlider
    JButton scaleDownButton;//scale up sSlider	
    JLabel valueLabel;//show s

    JLabel valLabel;
    JTextField valTF;   
    JLabel sliderMaxLabel;
    JTextField sliderMaxTF;
    JLabel sliderMinLabel;
    JTextField sliderMinTF;

    MyJSlider sSlider; // max mean distance in smoothing spline
    double min=0; // slider min value
    double max=1; // slider max value
    double val; // slider value
    
    JCheckBox[] checkedVariables;

    Color[] colors={Color.BLUE,Color.RED,Color.GREEN,Color.BLACK, 
		    Color.MAGENTA, Color.PINK, Color.ORANGE,Color.CYAN,
		    Color.DARK_GRAY,Color.YELLOW};

/**
 *  Create panels and action handling
 * @param p a problem object
 * @param exp an experiment object
 */
    public PlotExperiment(Problem p,Experiment exp) {

	this.experiment=exp;
	

        val=p.getSmoothingParameter();
	max=val*2;

	setLayout(new BorderLayout());
	
	//Titled borders
	Border innerBorder =
      
	    BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.blue,1),
					       BorderFactory.createEmptyBorder(2,2,2,2));
	Font borderFont=new Font("Dialog",Font.PLAIN,14);

	Border innerBorderSmall = BorderFactory.createLineBorder(Color.black);
	Font borderFontSmall=new Font("Dialog",Font.PLAIN,10);
	
	// Ordinary font
	Font font=new Font("Dialog",Font.PLAIN,12);
	
	
	





	/*
	 * BOXES, EAST
	 */

	JPanel checkPanel = new JPanel(new GridLayout(p.getNumberOfVariables(),1));
	checkedVariables = new JCheckBox[p.getNumberOfVariables()];
	for(int j=0; j<p.getNumberOfVariables(); j++){ 
	    checkedVariables[j]=new JCheckBox("Var "+(j+1));
	    //System.out.println(colors[j%colors.length]);
	    checkedVariables[j].setForeground (colors[j%colors.length]);
	    
	    checkedVariables[j].setSelected(p.getVariable(j).isDependent());
	    checkedVariables[j].addItemListener(this);
	    checkedVariables[j].setEnabled(p.getVariable(j).isDependent());
	    checkPanel.add(checkedVariables[j]);
	}
	getContentPane().add(checkPanel,BorderLayout.EAST);
	
	
	
	/*
	 * GRAPHPANEL, CENTER
	 */
	graphPanel=new GraphPanel(p,exp,checkedVariables,colors);
	getContentPane().add(graphPanel,BorderLayout.CENTER);

	
	/*
	 * INTERPOLATIONPANEL, SOUTH
	 */
	JPanel interpolationPanel = new JPanel(new BorderLayout());
	
	interpolationPanel.setBorder(BorderFactory.createTitledBorder(innerBorder,
								      "Smoothing spline parameter",
								      TitledBorder.CENTER,
								      TitledBorder.DEFAULT_POSITION,
								      borderFont));
	
	JPanel sliderPanel = new JPanel(new BorderLayout()); //slider + control buttons
	

	JPanel buttonPanel = new JPanel(new GridLayout(3,2));
	double s=p.getSmoothingParameter();
	valLabel = new JLabel("Current value");
	valTF=new JTextField(new Double(Misc.Round(val,2)).toString());   
	sliderMinLabel= new JLabel("Slider min");
	sliderMinTF=new JTextField(new Double(Misc.Round(min,2)).toString()); 
	sliderMaxLabel= new JLabel("Slider max");
	sliderMaxTF=new JTextField(new Double(Misc.Round(max,2)).toString());   
	
	buttonPanel.add(valLabel);
	buttonPanel.add(valTF);
	buttonPanel.add(sliderMinLabel);
	buttonPanel.add(sliderMinTF);
	buttonPanel.add(sliderMaxLabel);
	buttonPanel.add(sliderMaxTF);
	sliderPanel.add(buttonPanel,BorderLayout.WEST);
	
	valTF.setFont(font);
	valTF.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Debug.print(this,"actionPerformed","valTF");
		    if (experiment!=null){
			String s=valTF.getText();
			double tempval=Double.parseDouble(s);
			if (tempval>max) {
			    val=tempval;
			    max=tempval;
			    String se="Slider max value is adjusted to contain new value.";
			    JOptionPane.showMessageDialog(null,se,
							  "Information",
							  JOptionPane.INFORMATION_MESSAGE);
			    sliderMaxTF.setText(new Double(Misc.Round(max,2)).toString());
			    sSlider.set(min, max, val);
			}
			else if (tempval<0.0) {	
			    String se="Smoothing parameter must be greater than or equal to zero.";
			    JOptionPane.showMessageDialog(null,se,
							  "Information",
							  JOptionPane.INFORMATION_MESSAGE);
			    valTF.setText(new Double(Misc.Round(val,2)).toString());
			}
			else if (tempval<min) {
			    val=tempval;
			    min=tempval;
			    String se="Slider min value is adjusted to contain new value.";
			    JOptionPane.showMessageDialog(null,se,
							  "Information",
							  JOptionPane.INFORMATION_MESSAGE);
			    sliderMinTF.setText(new Double(Misc.Round(min,2)).toString());
			    sSlider.set(min, max, val);
			}
			else {
			    val=tempval;
			    sSlider.set(min, max, val);
			}
		    }
		}
	    });


	sliderMinTF.setFont(font);
	sliderMinTF.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Debug.print(this,"actionPerformed","sliderMinTF");
		    if (experiment!=null){
			String s=sliderMinTF.getText();
			double tempmin=-1;
			try { 
			    tempmin = Double.valueOf(s).doubleValue(); 
			    if (tempmin<=val && tempmin>=0.0) {
				min=tempmin;
				sSlider.set(min, max, val);
			    }
			    else {
				String se="Slider min value should be greater or equal to zero and less or equal to the current value";
				JOptionPane.showMessageDialog(null,se,
							      "Error",
							      JOptionPane.ERROR_MESSAGE);
				sliderMinTF.setText(new Double(Misc.Round(min,2)).toString());
			    }
			} 
			catch(NumberFormatException nfe){ 
			    JOptionPane.showMessageDialog(null,s + " is not a real number",
							  "Not a real number",
							  JOptionPane.ERROR_MESSAGE);
			    sliderMinTF.setText(new Double(Misc.Round(min,2)).toString());
			}
		    }
		}
	    });


	sliderMaxTF.setFont(font);
	sliderMaxTF.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Debug.print(this,"actionPerformed","sliderMaxTF");
		    if (experiment!=null){
			String s=sliderMaxTF.getText();
			double tempmax=-1;
			try { 
			    tempmax = Double.valueOf(s).doubleValue(); 
			    if (tempmax>=val) {
				max=tempmax;
				sSlider.set(min, max, val);
			    }
			    else {
				String se="Slider max value should be greater or equal to the current value";
				JOptionPane.showMessageDialog(null,se,
							      "Error",
							      JOptionPane.ERROR_MESSAGE);
				sliderMaxTF.setText(new Double(Misc.Round(max,2)).toString());
			    }
			} 
			catch(NumberFormatException nfe){ 
			    JOptionPane.showMessageDialog(null,s + " is not a real number",
							  "Not a real number",
							  JOptionPane.ERROR_MESSAGE);
			    sliderMaxTF.setText(new Double(Misc.Round(max,2)).toString());
			}
		    }
		}
	    });

		

	
	// Slider
	sSlider = new MyJSlider(min, max, val);
	sSlider.setFont(font);
	sSlider.setMajorTickSpacing(10);
	sSlider.setMinorTickSpacing(1);
	sSlider.setPaintTicks(true);
	sSlider.setPaintLabels(true);
	sSlider.setBorder(BorderFactory.createEmptyBorder(2,0,2,0));
	sSlider.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		    if (!sSlider.getValueIsAdjusting()) {
			val = sSlider.getDoubleValue(min, max);
			graphPanel.setLocalSmoothingParameter(val);
			graphPanel.repaint();
			valTF.setText(new Double(Misc.Round(val,2)).toString());
		    }
		}
	    });
    
       

	sliderPanel.add(sSlider,BorderLayout.CENTER);
	interpolationPanel.add(sliderPanel,BorderLayout.CENTER);
	getContentPane().add(interpolationPanel,BorderLayout.SOUTH);


    }



    /** Update the graph
     */
    public void actionPerformed(ActionEvent e) {
	String command = e.getActionCommand();
	if (command.equals("update") && experiment!=null){
	    double sd=sSlider.getDoubleValue(min, max);
	    sSlider.setEnabled(false);
	    graphPanel.setLocalSmoothingParameter(sd);
	    graphPanel.repaint();
	}
    }

    /** Update checked variables in plot
     */
    public void itemStateChanged(ItemEvent e) {
	graphPanel.setCheckedVariables(checkedVariables);
	graphPanel.repaint();
    }


    /** Reads a real number from a String
     * @param st a String
     */
    private double getDouble(String s) { 
	try { 
	    double d = Double.valueOf(s).doubleValue(); 
	    return d; 
	} 
	catch(NumberFormatException e){ 
	    String errorMsg=s + " is not a real number."; 
	    Debug.print("Problem","getDouble",errorMsg); 
	    JOptionPane.showMessageDialog(null,errorMsg,
					  "Not a real number",
					  JOptionPane.ERROR_MESSAGE);
		
	    return 0; 
	} 
    } 
  


/**
 * Plot experiment of a problem given as argv[0]
 * @param  argv array of Strings, first item contains path to problem file
 */
    public static void main(String[] argv) {
   	
        System.out.println(java.util.Locale.getDefault());
        if (argv.length==0){
            Debug.error("PlotExperiment","main","Input problem file required.");
        }

	// File system variables
	String filetree = System.getProperty("user.dir") + 
	    System.getProperty("file.separator");
	
	// Problem file
	File file=new File(filetree+argv[0]);
	if (file==null){
 	    Debug.error("PlotExperimen","main","Input problem file not found.");
        }
	int iexp=Misc.getInteger(argv[1],0);
    
	Problem p=new Problem();
	p.read(file,0);
	

	PlotExperiment f=new PlotExperiment(p,p.getExperiment(iexp-1));// rescale to 0...Nexp-1
	
	f.setSize(800,640);
	f.setLocation(300,200);
	//f.getContentPane().add(BorderLayout.CENTER, new JTextArea(10, 40));
	f.setVisible(true);
	f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	
	f.setTitle("Problem "+p.getName()+", experiment "+iexp);

    }





}


