/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
    www.odeidentification.org
*/


package Java.debug;

/**
 * Debug printouts and error messages
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class Debug {
    
    /** Debug printouts on or off*/
    public static boolean ON = true;
    /** Error messages on or off*/
    public static boolean ERROR = true;
    /** Warning messages on or off*/
    public static boolean WARNING = true;
    /** Max number of printed out warnings, default 5*/
    public static int MAX_WARNINGS=5;


    private static int nWarnings=0;

    /**
     * Debug printout
     * @param o object from which print out is called
     * @param method method from which print out is called
     */
    public static void print( Object o, String method) {
	if (ON) System.out.println( o.getClass().getName() + "." + method);
    }

    /**
     * Debug printout
     * @param o object from which print out is called
     * @param method method from which print out is called
     * @param msg debug message
     */
    public static void print( Object o, String method, String msg) {
	if (ON) System.out.println( o.getClass().getName() + "." + method + " - " + msg);
    }
    
    /**
     * Debug printout from constructor
     * @param o object from which print out is called
     */
    public static void constructor( Object o) {
	if (ON) System.out.println( o.getClass().getName() + ".constructor");
    }
    
    /**
     * Debug printout from constructor
     * @param o object from which print out is called
     * @param msg debug message
     */
    public static void constructor( Object o, String msg) {
	if (ON) System.out.println( o.getClass().getName() + ".constructor - " + msg);
    }
  
      
    /**
     * Debug printout from static environment
     * @param cl name of class from which print out is called
     * @param method method from which print out is called
     */
    public static void stat( String cl, String method) {
	if (ON) System.out.println(cl + "." + method);
    }

    /**
     * Debug printout from static environment
     * @param cl name of class from which print out is called
     * @param method method from which print out is called
     * @param msg debug message
     */
    public static void stat( String cl, String method, String msg) {
	if (ON) System.out.println(cl + "." + method + " - " + msg);
    }

    /**
     * Error message and program termination
     * @param o object from which error is called
     * @param method method from which error is called
     */
    public static void error( Object o, String method) {
	if (ERROR) { 
	    System.out.println( "Error: " + o.getClass().getName() + "." + method);
	    System.exit(1);
	}
    }  

    /**
     * Error message and program termination
     * @param o object from which error is called
     * @param method method from which error is called
     * @param msg errormessage
     */
    public static void error( Object o, String method, String msg) {
	if (ERROR) {
	    System.out.println( "Error: " + o.getClass().getName() + "." + method+" - "+msg);
	    System.exit(1);
	}
    }

    /**
     * Warning message and program termination
     * @param o object from which warning is called
     * @param method method from which warning is called
     */
    public static void warning( Object o, String method) {
	if (WARNING) {
	    nWarnings++;
	    if (nWarnings<MAX_WARNINGS) {
		System.out.println( "Warning: " + o.getClass().getName() + "." + method);
	    }
	    else if (nWarnings==MAX_WARNINGS) {
		System.out.println( "Warning: " + o.getClass().getName() + "." + method);
		System.out.println( "Further warnings suppressed.");
	    }
	}
    }  

    /**
     * Warning message and program termination
     * @param o object from which warning is called
     * @param method method from which warning is called
     * @param msg warningmessage
     */
    public static void warning( Object o, String method, String msg) {
	if (WARNING) {
	    nWarnings++;
	    if (nWarnings<MAX_WARNINGS) {
		System.out.println( "Warning: " + o.getClass().getName() + "." + method+" - "+msg);
	    }
	    else if (nWarnings==MAX_WARNINGS) {
		System.out.println( "Warning: " + o.getClass().getName() + "." + method+" - "+msg);
		System.out.println( "Further warnings suppressed.");
	    }
	}
    }
  
    /**
     * Error message and program terminationca
     * @param cl name of class from which error is called
     * @param method method from which error is called
     */
    public static void staterror( String cl, String method) {
	if (ERROR) {
	    System.out.println( "Error: " + cl + "." + method);
	    System.exit(1);
	}
    }
    /**
     * Error message and program termination
     * @param cl name of class from which error is called
     * @param method method from which error is called
     * @param msg errormessage
     */
    public static void staterror( String cl, String method, String msg) {
	if (ERROR) {
	    System.out.println( "Error: " + cl + "." + method+" - "+msg);
	    System.exit(1);
	}
    }
    



    /**
     * Warning message and program termination
     * @param cl name of class from which warning is called
     * @param method method from which warning is called
     */
    public static void statwarning( String cl, String method) {
	if (WARNING) {
	    nWarnings++;
	    if (nWarnings<MAX_WARNINGS) {
		System.out.println( "Warning: " + cl + "." + method); 
	    }
	    else if (nWarnings==MAX_WARNINGS) {
		System.out.println( "Warning: " + cl + "." + method); 
		System.out.println( "Further warnings suppressed.");
	    }
	}
    }
    /**
     * Warning message and program termination
     * @param cl name of class from which warning is called
     * @param method method from which warning is called
     * @param msg warningmessage
     */
    public static void statwarning( String cl, String method, String msg) {
	if (WARNING) {
	    nWarnings++;
	    if (nWarnings<MAX_WARNINGS) {
		System.out.println( "Warning: " + cl + "." + method+" - "+msg); 
	    }
	    else if (nWarnings==MAX_WARNINGS) {
		System.out.println( "Warning: " + cl + "." + method+" - "+msg); 
		System.out.println( "Further warnings suppressed.");
	    }
	}
    }    
    
    

}
