/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/


package Java.generateFortran;

import java.io.*;
import Java.debug.*;
import Java.problem.*;
/**
 * Generate reactions.f
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class GenerateFortran_myReactions{


    /**
     * Make reactions.f in directory for a certain problem
     * @param directory the directory
     * @param p the problem
     */
    public static void generate(String directory,Problem p) {
	File file = new File(directory + 
			     "reactions.f");
	try{
	   // Create file 
	    FileWriter fstream = new FileWriter(file);
	    BufferedWriter out = new BufferedWriter(fstream);

	    
	    out.write("C     This file was automatically generated by GenerateFortran_myReactions.generate()\n");
	    out.write("C     on "+(new java.util.Date())+"\n");
	    out.write("C\n");

	    // getNumberOfReactionTypes
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getNumberOfReactionTypes");
	    out.write("\n"); 
	    out.write("C     Get number of defined reaction types\n");
	    out.write("      integer function getNumberOfReactionTypes()\n");
	    out.write("      implicit none\n");
	    out.write("      getNumberOfReactionTypes="+p.getNumberOfReactionTypes()+"\n");
	    out.write("      return\n");
	    out.write("      end\n");
	    out.write("      \n");

	    // getNameOfReactionType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getNameOfReactionType");
	    out.write("\n"); 
	    out.write("C     Get name of reaction type itype\n");
	    out.write("      character*30 function getNameOfReactionType(itype)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		String n=p.getReactionType(i).getName();
		if (n.length()>30) {
		    GenerateFortran.prettyPrint(out,"getNameOfReactionType='"+n.substring(0,30)+"'",1);
		    Debug.stat("GenerateFortran_myReactions","generate",n+" shortened to "+n.substring(0,30));
		}
		else {
		    GenerateFortran.prettyPrint(out,"getNameOfReactionType='"+n+"'",1);
		}
	    }
	    out.write("      else\n");
	    out.write("         write(*,*) 'code_rk_myreactions.getNameOfReactionType',\n");
	    out.write("     *     ', error itype=',itype\n");
	    out.write("         stop\n");
	    out.write("         getNameOfReactionType= 'Error in getNameOfReactionType'\n");
	    out.write("      endif\n");
	    out.write("      end\n");
	    out.write("      \n");
	    
	    // getVariableNameOfReactionType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getVariableNameOfReactionTyp");
	    out.write("\n"); 
	    out.write("C     Get name of local variable iname of reaction type itype\n");
	    out.write("      character*30 function getVariableNameOfReactionType(iname,itype)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype,iname\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalVariables();j++){
		    if (j==0) GenerateFortran.prettyPrint(out,"if (iname.eq."+(j+1)+") then",1);
		    else GenerateFortran.prettyPrint(out,"else if (iname.eq."+(j+1)+") then",1);
		    String n=p.getReactionType(i).getLocalVariableName(j);
		    if (n.length()>30) {
			GenerateFortran.prettyPrint(out,"getVariableNameOfReactionType='"+n.substring(0,30)+"'",2);
			Debug.stat("GenerateFortran_myReactions","generate",n+" shortened to "+n.substring(0,30));
		    }
		    else {
			GenerateFortran.prettyPrint(out,"getVariableNameOfReactionType='"+n+"'",2);
		    }
		    GenerateFortran.prettyPrint(out,"return",2);
		    if (j==p.getReactionType(i).getNumberOfLocalVariables()-1) GenerateFortran.prettyPrint(out,"endif",1);
		}
	    }
	    out.write("      endif\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getVariableNameOfReactionType',\n");
	    out.write("     *  'error iname=',iname\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getVariableNameOfReactionType',\n");
	    out.write("     *  'error itype=',itype\n");
	    out.write("      stop\n");
	    out.write("      getVariableNameOfReactionType='Error in getNameOfReactionType'\n");
	    out.write("      end\n");
	    out.write("      \n");


	    // getParameterNameOfReactionType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getParameterNameOfReactionTyp");
	    out.write("\n"); 
	    out.write("C     Get name of local parameter ipara of reaction type itype\n");
	    out.write("      character*30 function getParameterNameOfReactionType(ipara,itype)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype,ipara\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalParameters();j++){
		    if (j==0) GenerateFortran.prettyPrint(out,"if (ipara.eq."+(j+1)+") then",1);
		    else GenerateFortran.prettyPrint(out,"else if (ipara.eq."+(j+1)+") then",1);
		    String n=p.getReactionType(i).getLocalParameterName(j);
		    if (n.length()>30) {
			GenerateFortran.prettyPrint(out,"getParameterNameOfReactionType='"+n.substring(0,30)+"'",2);
			Debug.stat("GenerateFortran_myReactions","generate",n+" shortened to "+n.substring(0,30));
		    }
		    else {
			GenerateFortran.prettyPrint(out,"getParameterNameOfReactionType='"+n+"'",2);
		    }
		    GenerateFortran.prettyPrint(out,"return",2);
		    if (j==p.getReactionType(i).getNumberOfLocalParameters()-1) GenerateFortran.prettyPrint(out,"endif",1);
		}
	    }
	    out.write("      endif\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getParameterNameOfReactionType',\n");
	    out.write("     *  'error ipara=',ipara\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getParameterNameOfReactionType',\n");
	    out.write("     *  'error itype=',itype\n");
	    out.write("      stop\n");
	    out.write("      getParameterNameOfReactionType='Error in getNameOfReactionType'\n");
	    out.write("      end\n");
	    out.write("      \n");


	    // getEquationOfReactionType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getEquationOfReactionType");
	    out.write("\n"); 
	    out.write("C     Get equation of reaction type itype\n");
	    out.write("      character*200 function getEquationOfReactionType(itype)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		GenerateFortran.prettyPrint(out,"getEquationOfReactionType='"+
			    p.getReactionType(i).getEquation()+"'",1);
	    }
	    out.write("      else\n");
	    out.write("        write(*,*) 'code_rk_myreactions.getEquationOfReactionType',\n");
	    out.write("     *    'error itype=',itype\n");
	    out.write("        stop\n");
	    out.write("        getEquationOfReactionType='Error in getNameOfReactionType'\n");
	    out.write("      endif\n");
	    out.write("      end\n");
	    out.write("      \n");

	    // getIndexOfReactionType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getIndexOfReactionType");
	    out.write("\n"); 
	    out.write("C     Get itype of reaction with name\n");
	    out.write("      integer function getIndexOfReactionType(name)\n");
	    out.write("      implicit none\n");
	    out.write("      character*30 name,getNameOfReactionType\n");
	    out.write("      integer i,getNumberOfReactionTypes\n");
	    out.write("      getIndexOfReactionType=-1\n");
	    out.write("      do 100 i=1,getNumberOfReactionTypes()\n");
	    out.write("         if (name.eq.getNameOfReactionType(i)) then\n");
	    out.write("            getIndexOfReactionType=i\n");
	    out.write("            return\n");
	    out.write("         endif\n");
	    out.write(" 100  enddo\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getIndexOfReactionType(',\n");
	    out.write("     *  name,'), error'\n");
	    out.write("      stop\n");
	    out.write("      end\n");
	    out.write("      \n");



	    // getKsOfType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getKsOfType");
	    out.write("\n"); 
	    out.write("C     Get number of parameters of type itype\n");
	    out.write("      integer function getKsOfType(itype)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		GenerateFortran.prettyPrint(out,"getKsOfType="+
			    p.getReactionType(i).getNumberOfLocalParameters(),1);
	    }
	    out.write("      else\n");
	    out.write("        write(*,*) 'code_rk_myreactions.getKsOfType',\n");
	    out.write("     *    'error itype=',itype\n");
	    out.write("        stop\n");
	    out.write("        getKsOfType=-1\n");
	    out.write("      endif\n");
	    out.write("      end\n");
	    out.write("      \n");

	    // getVsOfType
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getVsOfType");
	    out.write("\n"); 
	    out.write("C     Get number of variables of type itype\n");
	    out.write("      integer function getVsOfType(itype)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		GenerateFortran.prettyPrint(out,"getVsOfType="+
			    p.getReactionType(i).getNumberOfLocalVariables(),1);
	    }
	    out.write("      else\n");
	    out.write("        write(*,*) 'code_rk_myreactions.getVsOfType',\n");
	    out.write("     *    'error itype=',itype\n");
	    out.write("        stop\n");
	    out.write("        getVsOfType=-1\n");
	    out.write("      endif\n");
	    out.write("      end\n");
	    out.write("      \n");



	    // getInitialValue
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, getInitialValue");
	    out.write("\n"); 
	    out.write("C     Set value for parameter ipara of a reaction of type itype.\n");
	    out.write("C     itype - the type of the reaction\n");
	    out.write("C     ipara - parameter number to consider, 1..getKsOfType(itype)\n");
	    out.write("C     lowbound- lower bound\n");
	    out.write("C     uppbound - upper bound\n");  
	    out.write("C     value - (output) the initial value\n");
	    out.write("      subroutine getInitialValue(itype,ipara,\n");
	    out.write("     *     lowbound,uppbound,value)\n");
	    out.write("      implicit none\n");
	    out.write("      integer itype,ipara\n");
	    out.write("      integer ireact\n");
	    out.write("      double precision lowbound,uppbound,value\n");
	    out.write("      double precision closetozero\n");
	    out.write("      closetozero=lowbound\n");
	    out.write("      if (uppbound.le.0.d0) closetozero=uppbound\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (itype.eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (itype.eq."+(i+1)+") then",0);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalParameters();j++){
		    if (j==0) {
			GenerateFortran.prettyPrint(out,"if (ipara.eq."+(j+1)+") then",1);
			GenerateFortran.prettyPrint(out,"value=closetozero",2);
		    }
		    else {
			GenerateFortran.prettyPrint(out,"else if (ipara.eq."+(j+1)+") then",1);
			GenerateFortran.prettyPrint(out,"value=(lowbound+uppbound)/2",2);
		    }
		    GenerateFortran.prettyPrint(out,"return",2);
		    if (j==p.getReactionType(i).getNumberOfLocalParameters()-1) GenerateFortran.prettyPrint(out,"endif",1);
		}
	    }
	    out.write("      endif\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getInitialValue',\n");
	    out.write("     *  'error ipara=',ipara\n");
	    out.write("      write(*,*) 'code_rk_myreactions.getInitalValue',\n");
	    out.write("     *  'error itype=',itype\n");
	    out.write("      stop\n");
	    out.write("      end\n");
	    out.write("      \n");



	    // printvar
	    out.write("\n"); 
	    out.write("C     Print euqation for model variable ivar\n");
	    out.write("      subroutine printvar(maxvar,maxr,maxv,maxk,nvar,\n");
	    out.write("     *     modtype,modk,modn,modv,dependent,ivar)\n");
	    out.write("      implicit none\n"); 
	    out.write("      integer maxvar,maxr,maxv,maxk,nvar\n");
	    boolean include_modkinit=false;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    out.write("      integer ivar,j\n"); 
	    out.write("      write(*,5) 'dX(',ivar,')/dt='\n");
	    out.write("  5   format(A,I2,A)\n");
	    out.write("      do 10 j=1,modn(ivar)\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(ivar,j).eq."+(i+1)+") then",0);
		else GenerateFortran.prettyPrint(out,"else if (modtype(ivar,j).eq."+(i+1)+") then",0);
		String e=p.getReactionType(i).getEquation();
		//System.out.println(e);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalParameters();j++){
		    e=e.replace("k"+(j+1),"',modk(ivar,j,"+(j+1)+",1),'");
		    //System.out.println(e);
		}
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalVariables();j++){
		    e=e.replace("X"+(j+1),"X(',modv(ivar,j,"+(j+1)+"),')");
		    //System.out.println(e);
		}
		if (e.startsWith("',")) e=e.substring(2,e.length());
		e=e+"'";
		if (e.endsWith(",'")) e=e.substring(0,e.length()-2);
		if (e.startsWith("X(") || e.startsWith("modk(")) {
		    // ok
		}
		else {
		    e="'"+e;
		}
		//System.out.println("FINAL: "+e);
		GenerateFortran.prettyPrint(out,"write(*,"+(100+i)+") "+e,1);
		// format 
		e=p.getReactionType(i).getEquation();
		String f="format(";;
		for (int j=0;j<e.length();j++){
		    String et=e.substring(j,j+1);
		    if (j==0 && !et.equals("k")) f+="A,";
		    if (et.equals("k")) f+="D11.4,A,"; 
		    else if (et.equals("X")) f+="I2,A,"; 
		}
		f=f.substring(0,f.length()-1); // remove final comma
		f+=")";
		//System.out.println("FINAL: "+f);
		GenerateFortran.prettyPrint(out,f,1,(100+i));

	    }
	    out.write("      else\n");
            out.write("      write(*,*) 'code_rk_myreactions.printvar, ',\n");
            out.write("     *           'incorrect type, ivar=',ivar,\n");
	    out.write("     *           'modtype(',ivar,\n");
	    out.write("     *           ',',j,')',modtype(ivar,j) \n");
	    out.write("            stop\n");
	    out.write("         endif\n");
	    out.write(" 10   enddo\n");
	    out.write("      end\n");
	    out.write("      \n");
	  


	    // FUN2
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, FUN2");
	    out.write("\n"); 
	    out.write("C     LOCAL SEARCH WITH DERIVATIVE METHOD\n");
	    out.write("C     COMPUTES THE RESIDUAL VECTOR, R = R(X)\n");
	    out.write("C \n");    
	    out.write("C     NOTE: LTY the variable to consider\n");
	    out.write("C\n");
	    out.write("C     N - number of data-points (IN)\n");
	    out.write("C     P - number of parameters (IN)\n");
	    out.write("C     X - parameters, length P (IN)\n");
	    out.write("C     NF - invocation count, see below  (IN/OUT)\n");
	    out.write("C     R - residuals (OUT)\n");
	    out.write("C     LTY - additional integer input, here the variable to consider (IN)\n");
	    out.write("C     TY - additional real input, here doesn't used (IN)\n");
	    out.write("C     UF - additional function, here doesn't used (IN)\n");
	    out.write("C\n");
	    out.write("C     WHEN CALCR IS CALLED, NF IS THE INVOCATION COUNT\n");
	    out.write("C     FOR CALCR.  IT IS INCLUDED FOR POSSIBLE USE WITH\n");
	    out.write("C     CALCJ.  IF X IS OUT OF BOUNDS (E.G. IF IT WOULD\n");
	    out.write("C     CAUSE OVERFLOW IN COMPUTING R(X)), THEN CALCR SHOULD\n");
	    out.write("C     SET NF TO 0.  THIS WILL CAUSE A SHORTER STEP TO BE\n");
	    out.write("C     ATTEMPTED.   \n");
	    out.write("C\n");
	    out.write("C     DO NOT CHANGE N, P, OR X.\n");
	    out.write("C\n");
 	    out.write("      subroutine FUN2(N, P, X, NF, R, LTY, TY, UF) \n");
      
	    out.write("      implicit none\n");
	    out.write("      integer N, P, NF, LTY(1) \n");
	    out.write("      double precision X(P), R(N), TY(1) \n");
	    out.write("      external UF \n");
	    out.write("C\n");  
	    out.write("      integer nvar,nexp\n");
	    GenerateFortran.printMaxDefinition(out,p);
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    out.write("      integer nvare,npts(maxexp),npts2\n");
	    out.write("      double precision edata(maxexp,maxvar+1,2*maxpts)\n");
	    out.write("      double precision edata2(maxexp,maxvar+1,2*maxpts)\n");
	    out.write("      integer spline(maxexp,maxvar) ! 0=lin. interpol., 1=cubic smoothing spline\n");
	    out.write("      logical useip(maxvar) \n");
	    out.write("      integer ivar,ipara,j,ireact,iexp,k,i,ieq,jj \n");
	    out.write("      double precision gydot,RES,vtot  \n");
 	    out.write("      integer debug \n");
	    int P=p.getMAX_PARAMETERS_PER_ODE()*p.getNumberOfDependentVariables();
 	    out.write("      double precision B(2,"+P+"),TINY \n");
	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n");
	    out.write("      common/mydata2/edata2,npts2,useip\n");
	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
	    out.write("      common/myB/B ! communicates with estvar\n");
	    out.write("      TINY=0.001d0\n");
	    out.write("      TINY=1.d-10\n");
	    out.write("      do 20 jj=1,P\n");
	    out.write("         if (X(jj).ge.B(1,jj)-TINY.and.X(jj).le.B(2,jj)+TINY) then\n");
	    out.write("         else\n");
	    out.write("            do 10 ieq=1,N\n");
	    out.write("               R(ieq)=1.d10\n");
	    out.write(" 10        enddo\n");
	    out.write("            NF=0\n");
	    out.write("            return\n");
	    out.write("         endif\n");
	    out.write(" 20    enddo\n");
	    out.write("      debug=0\n");
	    out.write("      ivar=LTY(1)\n");
	    out.write("      ieq=1\n");
 	    out.write("      do 400 iexp=1,nexp\n");
	    out.write("         do 300 k=2,npts(iexp)\n");
	    out.write("            ipara=1\n");
	    out.write("            vtot=0.d0\n");
 	    out.write("            do 200 ireact=1,modn(ivar)\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		out.write("C     ---------------------------- "+p.getReactionType(i).getName()+"\n");
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(ivar,ireact).eq."+(i+1)+") then",3);
		else GenerateFortran.prettyPrint(out,"else if (modtype(ivar,ireact).eq."+(i+1)+") then",3);
		String e=p.getReactionType(i).getEquation();
		e=GenerateFortran.adaptEquationToParaest(e,p.getReactionType(i));
		GenerateFortran.prettyPrint(out,"vtot=vtot+"+e,4);
		GenerateFortran.prettyPrint(out,"ipara=ipara+"+p.getReactionType(i).getNumberOfLocalParameters(),4);
	    }
 	    out.write("               else\n");
  	    out.write("                 write(*,*) 'code_rk_myreactions.FUN2, error: ',\n");
  	    out.write("     *                 'modtype(',ivar,',',ireact,')',\n");
  	    out.write("     *                 modtype(ivar,ireact)\n");
  	    out.write("                 stop\n");
  	    out.write("              endif\n");
  	    out.write("200        continue\n");
  	    out.write("            if (useip(ivar)) then \n");
  	    out.write("               R(ieq) = vtot-edata(iexp,ivar,maxpts+k)\n");
   	    out.write("           else\n");
  	    out.write("               R(ieq) = vtot-gydot(nexp,nvare,npts2,\n");
  	    out.write("     *              edata2,\n");
  	    out.write("     *              iexp,ivar,edata(iexp,1,k))\n");
  	    out.write("            endif\n");
  	    out.write("            ieq=ieq+1\n");
  	    out.write(" 300     enddo \n");
   	    out.write(" 400  enddo\n");
   	    out.write("      if (N.ne.(ieq-1)) then\n");
   	    out.write("         write(*,*) 'code_rk_myreactions.FUN2, incorrect N=',N\n");
   	    out.write("         stop\n");
   	    out.write("      endif\n");
   	    out.write("      END \n");


	    // JACOB2
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, JACOB2");
	    out.write("\n"); 
   	    out.write("C     LOCAL SEARCH WITH DERIVATIVE METHOD\n");
   	    out.write("C     COMPUTES THE JACOBIAN MATRIX, J = J(X)\n");
   	    out.write("C     J(I,K) IS SET TO THE PARTIAL \n");
   	    out.write("C     DERIVATIVE OF COMPONENT I OF R WITH RESPECT TO X(K).\n");
   	    out.write("C  \n");   
   	    out.write("C     NOTE: LTY the variable to consider\n");
   	    out.write("C     N - number of data-points (IN)\n");
   	    out.write("C     P - number of parameters (IN)\n");
   	    out.write("C     X - parameters, length P (IN)\n");
   	    out.write("C     NF - invocation count, see below  (IN/OUT)\n");
   	    out.write("C     J - Jacobian (OUT)\n");
   	    out.write("C     LTY - additional integer input, here the variable to consider (IN)\n");
   	    out.write("C     TY - additional real input, here doesn't used (IN)\n");
   	    out.write("C     UF - additional function, here doesn't used (IN)\n");
   	    out.write("C     A subroutine WHICH, GIVEN X, COMPUTES THE JACOBIAN\n");
   	    out.write("C     MATRIX J OF R AT X, I.E., THE N BY P MATRIX WHOSE\n");
   	    out.write("C     (I,K) ENTRY IS THE PARTIAL DERIVATIVE OF THE I-TH\n");
   	    out.write("C     COMPONENT OF R WITH RESPECT TO X(K).  CALCJ MUST BE\n");
   	    out.write("C     DECLARED external IN THE CALLING PROGRAM.  IT IS\n");
   	    out.write("C     INVOKED BY\n");
   	    out.write("C     CALL CALCJ(N,P,X,NF,J,UIPARM,URPARM,UFPARM)\n");
   	    out.write("C     NF IS THE INVOCATION COUNT FOR CALCR AT THE TIME\n");
   	    out.write("C     R(X) WAS EVALUATED.  THE X PASSED TO CALCJ IS\n");
   	    out.write("C     USUALLY THE ONE PASSED TO CALCR ON EITHER ITS MOST\n");
   	    out.write("C     RECENT INVOCATION OR THE ONE PRIOR TO IT.  IF CALCR\n");
   	    out.write("C     SAVES INTERMEDIATE RESULTS FOR USE BY CALCJ, THEN IT\n");
   	    out.write("C     IS POSSIBLE TO TELL FROM NF WHETHER THEY ARE VALID\n");
   	    out.write("C     FOR THE CURRENT X (OR WHICH COPY IS VALID IF TWO\n");
   	    out.write("C     COPIES ARE KEPT).  IF J CANNOT BE COMPUTED AT X,\n");
   	    out.write("C     THEN CALCJ SHOULD SET NF TO 0.  IN THIS CASE, NL2SOL\n");
   	    out.write("C     WILL RETURN WITH IV(1) = 15.  THE OTHER PARAMETERS\n");
   	    out.write("C     TO CALCJ ARE AS DESCRIBED ABOVE AND BELOW.\n");
   	    out.write("C     DO NOT CHANGE N, P, OR X.\n");
    	    out.write("      subroutine JACOB2(N, P, X, NF, J, LTY, TY, UF) \n");
       	    out.write("      implicit none\n");
     	    out.write("      integer N, P, NF, LTY(1) \n");
     	    out.write("      double precision X(P), J(N,P), TY(1) \n");
    	    out.write("      external UF\n");
    	    out.write("      integer nvar,nexp\n");
	    GenerateFortran.printMaxDefinition(out,p);
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
    	    out.write("      double precision stdev(maxexp,maxvar,maxpts), lambda\n");
    	    out.write("      integer nvare,npts(maxexp)\n");
    	    out.write("      double precision edata(maxexp,maxvar+1,2*maxpts)\n");
    	    out.write("      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline \n");
    	    out.write("      integer ivar,ipara,jj,ireact,ieq,iexp,i,k\n");
    	    out.write("      double precision vtot,DELTA\n");
      	    out.write("      integer debug\n");
     	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n");
     	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
     	    out.write("      common/errsettings/stdev,lambda\n");

      	    out.write("      debug=0\n");
      
     	    out.write("      ivar=LTY(1)\n");
      	    out.write("      ieq=1\n");
      	    out.write("      do 400 iexp=1,nexp\n");
     	    out.write("         do 300 k=2,npts(iexp)\n");
     	    out.write("            ipara=1\n");
     	    out.write("            vtot=0.d0\n");
     	    out.write("             do 200 ireact=1,modn(ivar)\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		out.write("C     ---------------------------- "+p.getReactionType(i).getName()+"\n");
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(ivar,ireact).eq."+(i+1)+") then",3);
		else GenerateFortran.prettyPrint(out,"else if (modtype(ivar,ireact).eq."+(i+1)+") then",3);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalParameters();j++){
		    String e=p.getReactionType(i).get_dfdk(j);
		    if (e!=null) {
			e=GenerateFortran.adaptEquationToParaest(e,p.getReactionType(i));
			GenerateFortran.prettyPrint(out,"J(ieq,ipara+"+j+")="+e,4);
		    }
		    else { // numerical derivative
			String e0=p.getReactionType(i).getEquation();
			e0=GenerateFortran.adaptEquationToParaest(e0,p.getReactionType(i));
			GenerateFortran.prettyPrint(out,"DELTA=ABS(X(ipara+"+j+"))*1.d-6",4);
			GenerateFortran.prettyPrint(out,"! The following is needed to avoid division",4);
			GenerateFortran.prettyPrint(out,"!  by 0 when X(I) is close to zero",4);
			GenerateFortran.prettyPrint(out,"DELTA=MAX(DELTA,1.d-14)",4);
			GenerateFortran.prettyPrint(out,"X(ipara+"+j+")=X(ipara+"+j+")+DELTA ! DELTA CHANGE",4);
			String e1=e0.replace("X(ipara+"+j+")","(X(ipara+"+j+")+DELTA)"); // small delta step
			GenerateFortran.prettyPrint(out,"J(ieq,ipara+"+j+")=(("+e1+")-("+e0+
						    "))/DELTA",4);
		    }
		}
		GenerateFortran.prettyPrint(out,"ipara=ipara+"+p.getReactionType(i).getNumberOfLocalParameters(),4);
	    }
     	    out.write("               else\n");
     	    out.write("                  write(*,*) 'jac2, error modtype(',ivar,',',ireact,')',\n");
     	    out.write("     *                 modtype(ivar,ireact)\n");
     	    out.write("                  stop\n");
     	    out.write("               endif\n");
     	    out.write(" 200        continue\n");
     	    out.write("            ieq=ieq+1\n");
     	    out.write(" 300     enddo \n");
     	    out.write(" 400  enddo \n");
     	    out.write("      if (N.ne.(ieq-1).or.P.ne.(ipara-1)) then\n");
   	    out.write("         write(*,*) 'code_rk_myreactions.JAC2, incorrect N=',N\n");
     	    out.write("         stop\n");
     	    out.write("      endif\n");
      	    out.write("      RETURN \n");
      	    out.write("      END \n");
      	    out.write("\n");


	    // simulate
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, simulate");
      	    out.write("C     Simulate model\n");
      	    out.write("C     nvar - no of variables\n");
      	    out.write("C     nexp - no experiments\n");
      	    out.write("C     nvare- no variables in experiments\n");
      	    out.write("C     npts - no points in experiment\n");
      	    out.write("C     edata - exp data\n");
      	    out.write("C     sdata - simulated data OUTPUT\n");
      	    out.write("C     iexp - exp to simulate\n");
      	    out.write("C     istate - state of simulation (<0 indicates error) OUTPUT\n");
      	    out.write("      subroutine simulate(nvar,\n");
      	    out.write("     *     modtype,modk,modkinit,modn,modv,dependent,\n");
      	    out.write("     *     nexp,nvare,npts,\n");
      	    out.write("     *     edata,sdata,spline,\n");
      	    out.write("     *     iexp,\n");
      	    out.write("     *     istate)\n");
      	    out.write("      implicit none\n");
      	    out.write("      integer nvar,nexp \n"); 
      	    GenerateFortran.printMaxDefinition(out,p);
      	    include_modkinit=true;
      	    GenerateFortran.printModelDefinition(out,include_modkinit);    
	    out.write("      integer nvare,npts\n");
	    boolean include_edata2=false;
	    boolean include_spline=true;
	    boolean include_useip=false;
	    GenerateFortran.printExperimentDefinition(out,include_edata2,include_spline,include_useip);
       	    out.write("      double precision sdata(maxexp,maxvar+1,2*maxpts)\n");
	    out.write("      external model, jac\n");
       	    out.write("      integer iexp,iexp2,ireact,i,j,icycle\n");
	    out.write("      integer  iopt, iout, istate, itask, itol, liw, lrw\n");
	    int n=p.getNumberOfVariables();
	    out.write("      parameter (lrw="+(22+9*n+n*n)+") ! 22+9*maxvar+maxvar**2\n"); 
	    out.write("      parameter (liw="+(20+n)+")        ! 20+maxvar\n"); 
	    out.write("      integer  mf,neq,iwork(liw)\n"); 
	    out.write("      double precision  atol, rtol, rwork(lrw),t,tout, y(nvar),y0(nvar)\n"); 
      	    out.write("      logical debug\n"); 
     	    out.write("      common/simul/iexp2        ! communicates with subroutine jac\n"); 
      	    out.write("      debug=.false.\n"); 
      	    out.write("      if (nvar.ne.nvare) then   ! check parameters nvar and nvare\n");
      	    out.write("         write(*,*) 'simulate, nvar=',nvar,' nvare=',nvare\n");
       	    out.write("         stop\n");
       	    out.write("      endif\n");
      	    out.write("      if (iexp.lt.1.or.iexp.gt.nexp) then ! check parameter iexp\n");
       	    out.write("         write(*,*) 'simulate, error exp=',iexp\n");
      	    out.write("         return\n");
      	    out.write("      endif\n");
	    out.write("      iexp2=iexp                ! make copy\n");
      	    out.write("      do 30 i=1,nvar            ! initial values\n");
      	    out.write("         y(i)=modkinit(iexp,i,1)\n");
      	    out.write("         if (debug) write(*,*) 'simulate, y0(',i,')=',y(i)\n");
      	    out.write(" 30   continue\n");
      	    out.write("      neq = nvar\n");
      	    out.write("      itol = 1\n");
      	    out.write("      rtol = 1.D-4\n");
      	    out.write("      atol = 1.D-10\n");
      	    out.write("      itask = 1\n");
      	    out.write("      iopt = 0\n");
      	    out.write("      mf = 21 ! 21 == Jacobian supplied\n");
      	    out.write("              ! 22 == Jacobian not supplied\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (p.getReactionType(i).useNumericalDerivaties()) {
		    out.write("      mf = 22\n");
		    break;
		}
	    }
	    out.write("C     iopt=1\n");
      	    out.write("C     iwork(5)=0\n");
      	    out.write("C     iwork(6)=0 \n");
      	    out.write("C     iwork(7)=0\n");
      	    out.write("C     iwork(8)=0\n");
      	    out.write("C     iwork(9)=0\n");
      	    out.write("C     iwork(10)=0\n");
      	    out.write("C     rwork(5)=0.0\n");
      	    out.write("C     rwork(6)=(edata(iexp,1,npts)-edata(iexp,1,1))/50 ! hmax\n");
      	    out.write("C     rwork(6)=0.0\n");
      	    out.write("C     rwork(7)=0.0\n");
      	    out.write("C     rwork(8)=0.0\n");
      	    out.write("C     rwork(9)=0.0\n");
      	    out.write("C     rwork(10)=0.0\n");
      	    out.write("      t = edata(iexp,1,1)\n");
      	    out.write("      istate = 1\n");
      	    out.write("      do 140 iout = 1,npts\n");
      	    out.write("         tout = edata(iexp,1,iout)\n");
      	    out.write("         call dlsode (model, neq, y, t, tout, itol, rtol, atol, itask,\n");
      	    out.write("     *        istate, iopt, rwork, lrw, iwork, liw, jac, mf) \n");
      	    out.write("         sdata(iexp,1,iout)=t   ! save time data\n");
      	    out.write("         do 135 i=1,neq         ! save variable data\n");
      	    out.write("            if (dependent(i)) then\n");
      	    out.write("               sdata(iexp,(i+1),iout)=y(i)\n");
      	    out.write("            else\n");
      	    out.write("               sdata(iexp,(i+1),iout)=edata(iexp,i+1,iout) \n");
       	    out.write("           endif\n");
      	    out.write(" 135     continue  \n");
      	    out.write("         if (istate.lt. 0)  go to 180\n");
      	    out.write(" 140  continue \n");
      	    out.write("      if (debug) write(*,160)  iwork(11), iwork(12), iwork(13)\n");
       	    out.write(" 160  format(/'# No. steps =',i4,',No. f-s =',i4,',No. J-s =',i4)\n");
       	    out.write("      return\n");
      	    out.write(" 180  if (debug) write(*,190)  istate\n");
      	    out.write(" 190  format('dlsode, error halt. ISTATE =',I3)\n");
      	    out.write("      end\n");


	    // model
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, model");
	    out.write("\n"); 
      	    out.write("C     ODE model, called from dlsode\n");
      	    out.write("      subroutine model(neq,t,y,ydot)\n");
      	    out.write("      implicit none\n");
      	    out.write("      integer neq\n");
      	    out.write("      double precision t,y(neq),ydot(neq)\n");
      	    out.write("      double precision gysmth \n");     
      	    GenerateFortran.printMaxDefinition(out,p);
      	    include_modkinit=true;
      	    GenerateFortran.printModelDefinition(out,include_modkinit);
      	    out.write("      integer iexp,i,ireact\n");
      	    out.write("      common/simul/iexp        ! communicates with subroutine model \n");
      	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
       	    out.write("      do 5 i=1,neq\n");
      	    out.write("         if (.not.dependent(i)) then\n");
      	    out.write("            y(i)=gysmth(iexp,i,t,0,\n");
      	    out.write("     *           .false.,       ! do not recalculate parameters\n");
      	    out.write("     *           0.d0)          ! dummies\n");
      	    out.write("            ydot(i)=0\n");
      	    out.write("         endif\n");
      	    out.write(" 5    enddo\n");
       	    out.write("      do 100 i=1,neq\n");
	    out.write("         if (dependent(i)) then\n");
       	    out.write("            ydot(i)=0\n");
       	    out.write("            do 10 ireact=1,modn(i)\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		out.write("C     ---------------------------- "+p.getReactionType(i).getName()+"\n");
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(i,ireact).eq."+(i+1)+") then",3);
		else GenerateFortran.prettyPrint(out,"else if (modtype(i,ireact).eq."+(i+1)+") then",3);
		String e=p.getReactionType(i).getEquation();
		e=GenerateFortran.adaptEquationToSim(e,p.getReactionType(i));
		GenerateFortran.prettyPrint(out,"ydot(i)=ydot(i)+"+e,4);
	    }
	    out.write("               else\n");
   	    out.write("                  write(*,*) 'code_rk_myreactions.model, ',\n");
	    out.write("     *                 'error modtype(',i,',',ireact,')',\n");
       	    out.write("     *                 modtype(i,ireact)\n");
	    out.write("                  stop\n");
       	    out.write("               endif\n");
       	    out.write(" 10         continue\n");
       	    out.write("         endif\n");
       	    out.write(" 100  continue \n"); 
	    out.write("      end\n"); 
	    





	    // jac
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, jac");
	    out.write("\n"); 
      	    out.write("C     Jacobian of ODE model, called from dlsode\n");
      	    out.write("      subroutine jac (neq, t, y, ml, mu, pd, nrowpd)\n");
      	    out.write("      implicit none\n");
      	    out.write("      integer neq, ml, mu, nrowpd\n");
      	    out.write("      double precision t, y, pd\n");
      	    out.write("      dimension y(neq), pd(nrowpd,neq)\n");
      	    out.write("      double precision gysmth\n");     
      	    GenerateFortran.printMaxDefinition(out,p);
      	    include_modkinit=true;
      	    GenerateFortran.printModelDefinition(out,include_modkinit);
      	    out.write("      integer iexp,i,j,ireact\n");
      	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
      	    out.write("      common/simul/iexp\n");
       	    out.write("      do 5 i=1,neq\n");
      	    out.write("         if (.not.dependent(i)) then\n");
      	    out.write("            y(i)=gysmth(iexp,i,t,0,\n");
      	    out.write("     *           .false.,       ! do not recalculate parameters\n");
      	    out.write("     *           0.d0)          ! dummies\n");
      	    out.write("         endif\n");
      	    out.write(" 5    enddo\n");
       	    out.write("      do 100 i=1,neq\n");
       	    out.write("         do 6 j=1,neq\n");
       	    out.write("            pd(i,j)=0.d0\n");
       	    out.write(" 6       enddo\n");
	    out.write("         if (dependent(i)) then\n");
       	    out.write("            do 10 ireact=1,modn(i)\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		out.write("C     ---------------------------- "+p.getReactionType(i).getName()+"\n");
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(i,ireact).eq."+(i+1)+") then",3);
		else GenerateFortran.prettyPrint(out,"else if (modtype(i,ireact).eq."+(i+1)+") then",3);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalVariables();j++){
		    String e=p.getReactionType(i).get_dfdX(j);
		    if (e!=null) {
			e=GenerateFortran.adaptEquationToSim(e,p.getReactionType(i));
			GenerateFortran.prettyPrint(out,"pd(i,modv(i,ireact,"+(j+1)+
						    "))=pd(i,modv(i,ireact,"+(j+1)+"))+"+e,4);
		    }
		    // elseif numercial derivatives, jac is not called
		}
	    }
	    out.write("               else\n");
   	    out.write("                  write(*,*) 'code_rk_myreactions.jac, ',\n");
	    out.write("     *                 'error modtype(',i,',',ireact,')',\n");
       	    out.write("     *                 modtype(i,ireact)\n");
	    out.write("                  stop\n");
       	    out.write("               endif\n");
       	    out.write(" 10         continue\n");
       	    out.write("         endif\n");
       	    out.write(" 100  continue \n"); 
	    out.write("      end\n");
 







	    // simulatesingle
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, simulatesingle");
	    out.write("\n"); 
	    out.write("C     ODE model, called from dlsode\n"); 
	    out.write("      subroutine modelsingle(neq,t,y,ydot)\n"); 
	    out.write("      implicit none\n"); 
	    out.write("      integer neq\n"); 
	    out.write("      double precision t,y(neq),ydot(neq)\n"); 
	    out.write("      double precision gysmth\n"); 
	    GenerateFortran.printMaxDefinition(out,p);
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    out.write("      integer i,npts2,nexp,nvar,npts(maxexp),nvare\n"); 
	    out.write("      double precision edata(maxexp,maxvar+1,2*maxpts)\n"); 
	    out.write("      double precision edata2(maxexp,maxvar+1,2*maxpts)\n"); 
	    out.write("      integer spline(maxexp,maxvar) ! 0=linear interpolation, 1=cubic smoothing spline \n"); 
	    out.write("      integer iexp,ireact\n"); 
	    out.write("      double precision ytemp(maxvar),gy\n"); 
	    out.write("      logical debug,useip(maxvar)\n"); 
      	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n"); 
	    out.write("      common/mydata2/edata2,npts2,useip\n"); 
	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n"); 
	    out.write("      common/simuls/i,iexp        ! communicates with subroutine model \n"); 
 	    out.write("      if (.not.dependent(i)) then\n"); 
	    out.write("         y(1)=gysmth(iexp,i,t,0,\n"); 
 	    out.write("     *        .false.,          ! do not recalculate parameters\n"); 
 	    out.write("     *        0.d0)        ! dummies\n"); 
 	    out.write("        ydot(1)=0\n"); 
 	    out.write("        return\n"); 
	    out.write("      endif\n"); 
	    out.write("      ydot(1)=0\n"); 
	    out.write("      do 10 ireact=1,modn(i)\n"); 
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		out.write("C     ---------------------------- "+p.getReactionType(i).getName()+"\n");
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(i,ireact).eq."+(i+1)+") then",1);
		else GenerateFortran.prettyPrint(out,"else if (modtype(i,ireact).eq."+(i+1)+") then",1);
		for (int j=0;j<p.getReactionType(i).getNumberOfLocalVariables();j++){
		    if (j==0) GenerateFortran.prettyPrint(out,"if (i.eq.modv(i,ireact,"+(j+1)+")) then ! i matches var "+(j+1),2);
		    else GenerateFortran.prettyPrint(out,"else if (i.eq.modv(i,ireact,"+(j+1)+")) then ! i matches var "+(j+1),2);
		    for (int jj=0;jj<p.getReactionType(i).getNumberOfLocalVariables();jj++){
			if (j!=jj) { 
			    GenerateFortran.prettyPrint(out,"if (useip(modv(i,ireact,"+(jj+1)+"))) then",3); 
			    GenerateFortran.prettyPrint(out,"ytemp("+(jj+1)+")=gysmth(iexp,modv(i,ireact,"+(jj+1)+"),t,0,.false.,0.d0)",4); 
			    GenerateFortran.prettyPrint(out,"else",3); 
			    GenerateFortran.prettyPrint(out,"ytemp("+(jj+1)+")=gy(nexp,nvare,npts2,edata2,iexp,modv(i,ireact,"+(jj+1)+"),t)",4); 
			    GenerateFortran.prettyPrint(out,"endif",3); 
			}
		    }
		    String e=p.getReactionType(i).getEquation();
		    e=GenerateFortran.adaptEquationToSimSingle(e,p.getReactionType(i),j);
		    GenerateFortran.prettyPrint(out,"ydot(1)=ydot(1)+"+e,3); 
		}
		if (p.getReactionType(i).getNumberOfLocalVariables()>0) {
		    GenerateFortran.prettyPrint(out,"else ! i does not matches any var",2);
		}
		for (int jj=0;jj<p.getReactionType(i).getNumberOfLocalVariables();jj++){
		    GenerateFortran.prettyPrint(out,"if (useip(modv(i,ireact,"+(jj+1)+"))) then",3); 
		    GenerateFortran.prettyPrint(out,"ytemp("+(jj+1)+")=gysmth(iexp,modv(i,ireact,"+(jj+1)+"),t,0,.false.,0.d0)",4); 
		    GenerateFortran.prettyPrint(out,"else",3); 
		    GenerateFortran.prettyPrint(out,"ytemp("+(jj+1)+")=gy(nexp,nvare,npts2,edata2,iexp,modv(i,ireact,"+(jj+1)+"),t)",4); 
		    GenerateFortran.prettyPrint(out,"endif",3); 
		}
		String e=p.getReactionType(i).getEquation();
		e=GenerateFortran.adaptEquationToSimSingle(e,p.getReactionType(i),-1);
		GenerateFortran.prettyPrint(out,"ydot(1)=ydot(1)+"+e,3);
		if (p.getReactionType(i).getNumberOfLocalVariables()>0) {
		    GenerateFortran.prettyPrint(out,"endif",1);
		} 
	    }
	    out.write("         else\n");
	    out.write("            write(*,*) 'code_rk_sim, error modtype(',i,\n");
	    out.write("     *           ',',ireact,')=',\n");
	    out.write("     *           modtype(i,ireact)\n");
	    out.write("            stop\n");
	    out.write("         endif\n");
	    out.write(" 10   continue\n");
	    out.write("      end \n");


	    // simulatevars
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, simulatevars");
      	    out.write("C     Simulate submodel specied by bool var(maxvar)\n");
      	    out.write("C     Simulate and simulatesingle are special cases.\n");
      	    out.write("C     Experiment no to simulate set in common variable simulg\n");
      	    out.write("C     nvar - no of variables\n");
      	    out.write("C     nexp - no experiments\n");
      	    out.write("C     nvare- no variables in experiments\n");
      	    out.write("C     npts - no points in experiment\n");
      	    out.write("C     edata - exp data\n");
      	    out.write("C     sdata - simulated data OUTPUT\n");
      	    out.write("C     istate - state of simulation (<0 indicates error) OUTPUT\n");
      	    out.write("C     var - submodel, set in common block\n");
      	    out.write("C     iexp - exp to simulate, set in common block\n");
      	    out.write("      subroutine simulatevars(nvar,\n");
      	    out.write("     *     modtype,modk,modkinit,modn,modv,dependent,\n");
      	    out.write("     *     nexp,nvare,npts,\n");
      	    out.write("     *     edata,sdata,\n");
      	    out.write("     *     istate)\n");
	    out.write("      implicit none\n");
      	    out.write("      integer nvar,nexp,ivar,ivar2\n"); 
      	    GenerateFortran.printMaxDefinition(out,p);
      	    include_modkinit=true;
      	    GenerateFortran.printModelDefinition(out,include_modkinit);    
	    include_edata2=false;
	    include_spline=false;
	    include_useip=false;
	    GenerateFortran.printExperimentDefinition(out,include_edata2,include_spline,include_useip);  
	    out.write("      integer nvare,npts(maxexp),npts2\n");
       	    out.write("      double precision sdata(maxexp,maxvar+1,2*maxpts)\n");
	    out.write("      external modelgeneral, jac\n");
       	    out.write("      integer iexp,i,j,k,ireact\n");
	    out.write("      integer iopt, iout, istate, itask, itol, liw, lrw\n");
	    n=p.getNumberOfVariables();
	    out.write("      parameter (lrw="+(22+9*n+n*n)+") ! 22+9*maxvar+maxvar**2\n"); 
	    out.write("      parameter (liw="+(20+n)+")        ! 20+maxvar\n"); 
	    out.write("      integer mf,neq,iwork(liw)\n"); 
	    out.write("      double precision  atol, rtol, rwork(lrw),t,tout\n");
	    out.write("      double precision  y(maxvar),y0(maxvar)\n"); 
      	    out.write("      logical debug,var(maxvar) \n"); 
     	    out.write("      common/simulg/var,iexp\n");
     	    out.write("      debug=.false.\n"); 
      	    out.write("      if (nvar.ne.nvare) then   ! check parameters nvar and nvare\n");
      	    out.write("         write(*,*) 'simulate, nvar=',nvar,' nvare=',nvare\n");
       	    out.write("         stop\n");
       	    out.write("      endif\n");
      	    out.write("      if (iexp.lt.1.or.iexp.gt.nexp) then ! check parameter iexp\n");
       	    out.write("         write(*,*) 'simulate, error exp=',iexp\n");
      	    out.write("         return\n");
      	    out.write("      endif\n");
      	    out.write("      do 30 i=1,nvar            ! initial values\n");
      	    out.write("         y(i)=modkinit(iexp,i,1)\n");
      	    out.write(" 30   continue\n");
      	    out.write("      neq = nvar\n");
      	    out.write("      itol = 1\n");
      	    out.write("      rtol = 1.D-4\n");
      	    out.write("      atol = 1.D-10\n");
      	    out.write("      itask = 1\n");
      	    out.write("      iopt = 0\n");
      	    out.write("      mf = 22 ! 21 == Jacobian supplied\n");
      	    out.write("              ! 22 == Jacobian not supplied\n");
	    out.write("C     iopt=1\n");
      	    out.write("C     iwork(5)=0\n");
      	    out.write("C     iwork(6)=0 \n");
      	    out.write("C     iwork(7)=0\n");
      	    out.write("C     iwork(8)=0\n");
      	    out.write("C     iwork(9)=0\n");
      	    out.write("C     iwork(10)=0\n");
      	    out.write("C     rwork(5)=0.0\n");
      	    out.write("C     rwork(6)=1.0 ! hmax\n");
      	    out.write("C     rwork(6)=0.0\n");
      	    out.write("C     rwork(7)=0.0\n");
      	    out.write("C     rwork(8)=0.0\n");
      	    out.write("C     rwork(9)=0.0\n");
      	    out.write("C     rwork(10)=0.0\n");
      	    out.write("      t = edata(iexp,1,1)\n");
      	    out.write("      istate = 1\n");
      	    out.write("      do 140 iout = 1,npts(iexp)\n");
      	    out.write("         tout = edata(iexp,1,iout)\n");
      	    out.write("         call dlsode (modelgeneral, neq, y, t, tout, itol, rtol, atol,\n");
      	    out.write("     *         itask,istate, iopt, rwork, lrw, iwork, liw, jac, mf) \n");
      	    out.write("         sdata(iexp,1,iout)=t   ! save time data\n");
      	    out.write("         do 135 i=1,neq         ! save variable data\n");
      	    out.write("            if (dependent(i).and.var(i)) then\n");
      	    out.write("               sdata(iexp,(i+1),iout)=y(i)\n");
      	    out.write("            else\n");
      	    out.write("               sdata(iexp,(i+1),iout)=edata(iexp,i+1,iout) \n");
       	    out.write("           endif\n");
      	    out.write(" 135     continue  \n");
      	    out.write("         if (istate.lt. 0)  go to 180\n");
      	    out.write(" 140  continue \n");
      	    out.write("      if (debug) write(*,160)  iwork(11), iwork(12), iwork(13)\n");
       	    out.write(" 160  format(/'# No. steps =',i4,',No. f-s =',i4,',No. J-s =',i4)\n");
       	    out.write("      return\n");
      	    out.write(" 180  if (debug) write(*,190)  istate\n");
      	    out.write(" 190  format('dlsode, error halt. ISTATE =',I3)\n");
      	    out.write("      end\n");



	    // modelgeneral
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, modelgeneral");
	    out.write("\n"); 
	    out.write("C     ODE model, called from dlsode\n");


	    out.write("C     simualte any number of variables (defined in bool var(maxvar))\n");
	    out.write("C     Take others from data\n");
	    out.write("      subroutine modelgeneral(neq,t,y,ydot)\n");
	    out.write("      implicit none\n");
 	    out.write("      integer neq\n");
 	    out.write("      double precision t,y(neq),ydot(neq)\n");
	    out.write("      double precision gysmth,gy\n");
      	    GenerateFortran.printMaxDefinition(out,p);
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    out.write("	     integer i,ivar,npts2,nexp,nvar,npts(maxexp),nvare\n");
	    out.write("      double precision edata(maxexp,maxvar+1,2*maxpts)\n");
	    out.write("      double precision edata2(maxexp,maxvar+1,2*maxpts)\n");
	    out.write("      integer spline(maxexp,maxvar) ! 0=lin. interpol.,1=cubic smooth. spline\n");
	    out.write("      integer iexp,ireact\n");
	    out.write("      double precision ytemp,ytemp2,ytemp3\n");
 	    out.write("      logical debug,useip(maxvar),var(maxvar)\n");
       	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n");
 	    out.write("      common/mydata2/edata2,npts2,useip\n");
  	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
 	    out.write("      common/simulg/var,iexp  ! communicates simulategeneral\n");
	    out.write("      do 5 i=1,neq\n");
	    out.write("         if (.not.dependent(i)) then ! Find values for dependent variables\n");
 	    out.write("           y(i)=gysmth(iexp,i,t,0,\n");
	    out.write("     *           .false.,       ! do not recalculate parameters\n");
	    out.write("     *           0.d0)     ! dummies\n");
	    out.write("            ydot(i)=0\n");
	    out.write("         endif\n");
 	    out.write("         if (.not.var(i)) then  ! Find values for variables that are not simulated\n");
 	    out.write("           if (useip(i)) then \n");
 	    out.write("              y(i)=gysmth(iexp,i,t,0,\n");
 	    out.write("     *              .false.,    ! do not recalculate parameters\n");
 	    out.write("     *              0.d0)\n");
	    out.write("            else\n");
 	    out.write("              y(i)=gy(nexp,nvare,npts2,\n");
 	    out.write("     *              edata2,\n");
 	    out.write("     *              iexp,i,t)\n");
 	    out.write("           endif\n");
 	    out.write("           ydot(i)=0\n");
	    out.write("         endif\n");
	    out.write(" 5    enddo\n");
	    out.write("      do 100 i=1,neq\n");
	    out.write("         if (dependent(i).and.var(i)) then\n");
	    out.write("            ydot(i)=0\n");
	    out.write("            do 10 ireact=1,modn(i)\n");
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		out.write("C     ---------------------------- "+p.getReactionType(i).getName()+"\n");
		if (i==0) GenerateFortran.prettyPrint(out,"if (modtype(i,ireact).eq."+(i+1)+") then",3);
		else GenerateFortran.prettyPrint(out,"else if (modtype(i,ireact).eq."+(i+1)+") then",3);
		String e=p.getReactionType(i).getEquation();
		e=GenerateFortran.adaptEquationToSim(e,p.getReactionType(i));
		GenerateFortran.prettyPrint(out,"ydot(i)=ydot(i)+"+e,4);
	    }
	    out.write("               else\n");
   	    out.write("                  write(*,*) 'code_rk_myreactions.modelgeneral, ',\n");
	    out.write("     *                 'error modtype(',i,',',ireact,')',\n");
       	    out.write("     *                 modtype(i,ireact)\n");
	    out.write("                  stop\n");
       	    out.write("               endif\n");
       	    out.write(" 10         continue\n");
       	    out.write("         endif\n");
       	    out.write(" 100  continue \n"); 
	    out.write("      end\n"); 


	    // tune
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, tune");
	    out.write("\n");
	    out.write("C     Tune model with current global error\n");
	    out.write("C     New estimation for each tried structure\n");
	    out.write("C     Output new error\n");
	    out.write("C     Model updated\n");
	    out.write("      subroutine tunewithest(error,debug)\n");
	    out.write("      implicit none\n");
	    GenerateFortran.printMaxDefinition(out,p);
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    include_edata2=false;
	    include_spline=true;
	    include_useip=false;
	    GenerateFortran.printExperimentDefinition(out,include_edata2,include_spline,include_useip);
	    out.write("      integer nexp,nvare,npts(maxexp)\n");
  	    out.write("      double precision error\n");
  	    out.write("      logical add\n");
	    GenerateFortran.printTempModelDefinition(out,include_modkinit);
  	    out.write("      integer nvar,ivar\n");
  	    out.write("      integer debug\n");
  	    out.write("      logical initconc          ! T if init conc consider as parameter, ow F\n");
  	    out.write("      integer isucc,nnn,i,ir,j\n");
  	    out.write("      double precision errtem\n");
  	    out.write("      logical didtune\n");

  	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n");
  	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
  	    out.write("      common/myinitconc/initconc\n");
       
 	    out.write("      call icopy(maxvar*maxr,modtype,1,tmodtype,1) ! make temp copy\n");
  	    out.write("      call dcopy(maxvar*maxr*maxk*3,modk,1,tmodk,1)\n");
 	    out.write("      call dcopy(maxvar*maxexp*3,modkinit,1,tmodkinit,1)\n");
 	    out.write("      call icopy(maxvar,modn,1,tmodn,1)\n");
 	    out.write("      call icopy(maxvar*maxr*maxv,modv,1,tmodv,1)\n");
      
 	    out.write("100   continue                  ! OUTER LOOP STARTS\n");
	    out.write("      didtune=.false.\n");
	    out.write("      do 1000 ivar=1,nvar\n");
	    out.write("         nnn=modn(ivar)         ! reactions to remove\n");
	    out.write("         do 500 ir=nnn,1,-1     ! remove one of the existing reactions\n");
	    if (p.isSSystem())
		out.write("            do 400 j=2,2        ! S-system, no terms are removed\n");
	    else
		out.write("            do 400 j=1,2        ! 1: normal case, 2: for GMA\n");

	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (p.getReactionType(i).getName().equals("GMAterm7Variables")){
		    Debug.staterror("GenerateFortran_myReactions","generate","GMAterm7Variables (or higher) not implemented for subroutine tunewithest.");
		    System.exit(0);
		}



		// GMA 6 vars
		if (p.getReactionType(i).getName().equals("GMAterm6Variables")){
		    ReactionType rgoto=p.getReactionType("GMAterm5Variables");
		    if (rgoto!=null){
			GenerateFortran.prettyPrint(out,"if (j.eq.2.and.modtype(ivar,ir).eq."+
						    p.getReactionType(i).getId()+
						    ") then",3);
			GenerateFortran.prettyPrint(out,"! if a GMAterm6Variables",3);
			GenerateFortran.prettyPrint(out,"if (abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,6,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,7,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove first var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,1,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,6,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,7,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 2nd var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,2,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,6,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,7,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 3rd var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,3,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
                  GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,6,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,7,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 4th var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,4,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
 			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,6,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,6,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,6,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,6,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,6,1)).lt.abs(modk(ivar,ir,7,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 5th var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,5,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"else",4);
			GenerateFortran.prettyPrint(out,"! remove 6th var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,6,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"endif",4);
			GenerateFortran.prettyPrint(out,"endif",3);
		    }
		}




		// GMA 5 vars
		if (p.getReactionType(i).getName().equals("GMAterm5Variables")){
		    ReactionType rgoto=p.getReactionType("GMAterm4Variables");
		    if (rgoto!=null){
			GenerateFortran.prettyPrint(out,"if (j.eq.2.and.modtype(ivar,ir).eq."+
						    p.getReactionType(i).getId()+
						    ") then",3);
			GenerateFortran.prettyPrint(out,"! if a GMAterm5Variables",3);
			GenerateFortran.prettyPrint(out,"if (abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,6,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove first var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,1,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,6,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 2nd var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,2,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,5,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,6,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 3rd var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,3,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
                  GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,5,1)).lt.abs(modk(ivar,ir,6,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 4th var",5);
      		GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,4,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"else",4);
			GenerateFortran.prettyPrint(out,"! remove 5th var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,5,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			GenerateFortran.prettyPrint(out,"endif",4);
			GenerateFortran.prettyPrint(out,"endif",3);
		    }
		}

            // GMA 4 vars
		if (p.getReactionType(i).getName().equals("GMAterm4Variables")){
		    ReactionType rgoto=p.getReactionType("GMAterm3Variables");
		    if (rgoto!=null){
			GenerateFortran.prettyPrint(out,"if (j.eq.2.and.modtype(ivar,ir).eq."+
						    p.getReactionType(i).getId()+
						    ") then",3);
			GenerateFortran.prettyPrint(out,"! if a GMAterm4Variables",3);
			GenerateFortran.prettyPrint(out,"if (abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,5,1))) then",4);

			GenerateFortran.prettyPrint(out,"! remove first var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,1,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove first var",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,2,1)=modk(ivar,ir,3,1)",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,3,1)=modk(ivar,ir,4,1)",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,4,1)=modk(ivar,ir,5,1)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,1)=modv(ivar,ir,2)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,2)=modv(ivar,ir,3)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,3)=modv(ivar,ir,4)",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,4,1))."+
						    "and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,5,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 2nd var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,2,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove 2nd var",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,3,1)=modk(ivar,ir,4,1)",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,4,1)=modk(ivar,ir,5,1)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,2)=modv(ivar,ir,3)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,3)=modv(ivar,ir,4)",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,2,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,4,1)).lt.abs(modk(ivar,ir,5,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 3rd var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,3,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove 3nd var",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,4,1)=modk(ivar,ir,5,1)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,3)=modv(ivar,ir,4)",5);
			GenerateFortran.prettyPrint(out,"else",4);
			GenerateFortran.prettyPrint(out,"! remove 4th var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,4,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove 4th var",5);
			GenerateFortran.prettyPrint(out,"endif",4);
			GenerateFortran.prettyPrint(out,"endif",3);
		    }
		}

		// GMA 3 vars
		if (p.getReactionType(i).getName().equals("GMAterm3Variables")){
		    ReactionType rgoto=p.getReactionType("GMAterm2Variables");
		    if (rgoto!=null){
			GenerateFortran.prettyPrint(out,"if (j.eq.2.and.modtype(ivar,ir).eq."+
						    p.getReactionType(i).getId()+
						    ") then",3);
			GenerateFortran.prettyPrint(out,"! if a GMAterm3Variables",3);
			GenerateFortran.prettyPrint(out,"if (abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,3,1))."+
						    "and.abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,4,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove first var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,1,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove first var",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,2,1)=modk(ivar,ir,3,1)",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,3,1)=modk(ivar,ir,4,1)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,1)=modv(ivar,ir,2)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,2)=modv(ivar,ir,3)",5);
			GenerateFortran.prettyPrint(out,"else if (abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,2,1))"+
						    ".and.abs(modk(ivar,ir,3,1)).lt.abs(modk(ivar,ir,4,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove 2nd var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,2,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove middle var",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,3,1)=modk(ivar,ir,4,1)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,2)=modv(ivar,ir,3)",5);
			GenerateFortran.prettyPrint(out,"else",4);
			GenerateFortran.prettyPrint(out,"! remove 3rd var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,3,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove 3rd var",5);
			GenerateFortran.prettyPrint(out,"endif",4);
			GenerateFortran.prettyPrint(out,"endif",3);
		    }
		}
		// GMA 2 vars
		if (p.getReactionType(i).getName().equals("GMAterm2Variables")){
		    ReactionType rgoto=p.getReactionType("GMAterm1Variable");
		    if (rgoto!=null){
			GenerateFortran.prettyPrint(out,"if (j.eq.2.and.modtype(ivar,ir).eq."+
						    p.getReactionType(i).getId()+
						    ") then",3);
			GenerateFortran.prettyPrint(out,"! if a GMAterm2Variables",3);
			GenerateFortran.prettyPrint(out,"if (abs(modk(ivar,ir,2,1)).lt.abs(modk(ivar,ir,3,1))) then",4);
			GenerateFortran.prettyPrint(out,"! remove first var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,1,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove first var",5);
			//GenerateFortran.prettyPrint(out,"modk(ivar,ir,2,1)=modk(ivar,ir,3,1)",5);
			//GenerateFortran.prettyPrint(out,"modv(ivar,ir,1)=modv(ivar,ir,2)",5);
			GenerateFortran.prettyPrint(out,"else",4);
			GenerateFortran.prettyPrint(out,"! remove 2nd var",5);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,2,ir,isucc)",5); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",5);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+"! remove last var",5);
			GenerateFortran.prettyPrint(out,"endif",4);
			GenerateFortran.prettyPrint(out,"endif",3);
		    }
		}
		// GMA 1 var
		if (p.getReactionType(i).getName().equals("GMAterm1Variable")){
		    ReactionType rgoto=p.getReactionType("GMAterm0Variables");
		    if (rgoto!=null){
			GenerateFortran.prettyPrint(out,"if (j.eq.2.and.modtype(ivar,ir).eq."+
						    p.getReactionType(i).getId()+
						    ") then",3);
			GenerateFortran.prettyPrint(out,"! if a GMAterm1Variable",3);
			GenerateFortran.prettyPrint(out,"! remove first var",4);
      			GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,"+
						    "modk,modn,modv,dependent,ivar,1,ir,isucc)",4); 
			GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 390 ! continue",4);
			//GenerateFortran.prettyPrint(out,"modtype(ivar,ir)="+rgoto.getId()+" ! remove first var",4);
			GenerateFortran.prettyPrint(out,"endif",3);
		    }
		}

	    
	    
	    }
	    out.write("              if (j.eq.1) then ! This is the normal case\n"); 
   	    out.write("                 call rm(maxvar,maxr,maxv,maxk,nvar,\n"); 
   	    out.write("     *                 modtype,modk,modn,modv,dependent,\n"); 
   	    out.write("     *                 ivar,\n"); 
   	    out.write("     *                 ir,\n"); 
   	    out.write("     *                 isucc)\n"); 
   	    out.write("                 if (isucc.eq.0) then\n"); 
   	    out.write("                    write(*,*) 'tunewithest, error. rm-isucc=',isucc\n"); 
   	    out.write("                    stop\n"); 
   	    out.write("                 endif\n"); 
   	    out.write("              endif\n"); 
	    if (p.isSSystem()) {
		out.write("              call est(errtem,.false.)\n"); 
	    }
	    else {
		out.write("              call estgeneral(errtem,.false.)\n"); 
	    }
   	    out.write("              if (debug.gt.0) then \n"); 
   	    out.write("                 write(*,5000) 'tunewithest, ivar=',ivar,' ir=',ir\n"); 
   	    out.write("     $                 ,' error=',errtem\n"); 
   	    out.write("5000             format(A,I3,A,I3,A,D10.3)\n"); 
  	    out.write("                  if (debug.gt.1) call printvar(maxvar,maxr,maxv\n"); 
   	    out.write("     $                 ,maxk,nvar,modtype,modk,modn,modv,dependent\n"); 
   	    out.write("     $                 ,ivar)\n"); 
   	    out.write("              endif\n"); 
   	    out.write("              if (errtem.lt.error) then\n"); 
   	    out.write("                 error=errtem\n"); 
   	    out.write("                 call icopy(maxvar*maxr,modtype,1,tmodtype,1) ! make temp copy\n"); 
  	    out.write("                  call dcopy(maxvar*maxr*maxk*3,modk,1,tmodk,1)\n"); 
  	    out.write("                  call dcopy(maxvar*maxexp*3,modkinit,1,tmodkinit,1)\n"); 
  	    out.write("                  call icopy(maxvar,modn,1,tmodn,1)\n"); 
  	    out.write("                  call icopy(maxvar*maxr*maxv,modv,1,tmodv,1)\n"); 
  	    out.write("                  didtune=.true.\n"); 
  	    out.write("               else\n"); 
   	    out.write("                  call icopy(maxvar*maxr,tmodtype,1,modtype,1) ! Reset\n"); 
  	    out.write("                  call dcopy(maxvar*maxr*maxk*3,tmodk,1,modk,1)\n"); 
  	    out.write("                  call dcopy(maxvar*maxexp*3,tmodkinit,1,modkinit,1)\n"); 
  	    out.write("                  call icopy(maxvar,tmodn,1,modn,1)\n"); 
  	    out.write("                  call icopy(maxvar*maxr*maxv,tmodv,1,modv,1)\n"); 
   	    out.write("              endif\n"); 
   	    out.write("390           continue ! go here if rmv fails\n"); 
   	    out.write("400        continue  \n"); 
   	    out.write("500     continue               ! end remove one of existing reactions\n"); 
   	    out.write("1000  enddo\n"); 
   	    out.write("      if (didtune) goto 100\n"); 
   	    out.write("      end\n"); 
      


	    // estvarrm
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, estvarrm");
	    out.write("\n");
  	    out.write("C     Evalute subsystems of the current ODE for variable ivar\n"); 
	    out.write("C\n");
  	    out.write("C     double precision errcur(maxvar), IN\n");
	    out.write("C             Best error of the variables at start of main loop in main algorithm\n"); 
	    out.write("C\n");
  	    out.write("C     double precision errloc  IN\n"); 
  	    out.write("C             best error so far when testing all testreactions\n"); 
  	    out.write("C             of this iteration of the main loop, can be smaller than errcur(ivar)\n"); 
  	    out.write("C              add=F, make sure that errloc is the best error so far, i.e., errcur(ivar)\n"); 
	    out.write("C\n");
  	    out.write("C     double precision errbes  OUT\n"); 
  	    out.write("C             best error obtained in this subroutine\n"); 
	    out.write("C\n");
  	    out.write("C     logical add, IN\n");
  	    out.write("C             add=T, a reaction was term was added, should not be removed.\n");
  	    out.write("C             add=F, no addition, consider all cases.\n"); 
	    out.write("C\n");
  	    out.write("C     integer addterm, IN\n");
  	    out.write("C             addterm>0, a variable was added at the end of GMA term addterm, should not be removed.\n");
  	    out.write("C             addterm=0, no variable addition, consider all cases.\n"); 
	    out.write("C\n");
  	    out.write("C     integer DEPTH, IN\n");
  	    out.write("C             1 or 2 reactions that can be removed \n"); 
	    out.write("C\n");
  	    out.write("C     integer debug, IN\n"); 
  	    out.write("C             0=none, 1=little, 2=much\n"); 
  	    out.write("C\n");
  	    out.write("C     terms refer to ODE term number\n");
  	    out.write("C     varindex =0 if traditional reaction\n");
  	    out.write("C     varindex > 0 if gma reaction\n");
  	    out.write("C     e.g a GMA reaction with 3 variables will contribute 3 rows to the (terms,varindex) table\n");
  	    out.write("C     terms  varindex\n");
  	    out.write("C      1        0       e.g  MM reaction\n");
  	    out.write("C      2        1       e.g. GMA 3 vars reaction\n");
  	    out.write("C      2        2       e.g. GMA 3 vars reaction\n");
  	    out.write("C      2        3       e.g. GMA 3 vars reaction\n");
  	    out.write("C      3        0       e.g  MM reaction\n");
  	    out.write("C      4        1       e.g. GMA 2 vars reaction\n");
  	    out.write("C      4        2       e.g. GMA 2 vars reaction\n");
  	    out.write("C\n");
  	    out.write("C      Table size is stored in tablesize\n");
  	    out.write("C\n");
	    out.write("C\n");
  	    out.write("      subroutine estvarrm(ivar,errloc,errcur,errbes,add,\n");
  	    out.write("     *                    addterm,DEPTH,debug)\n");
	    out.write("      implicit none\n");
	    GenerateFortran.printMaxDefinition(out,p);
	    include_edata2=false;
	    include_spline=true;
	    include_useip=false;
	    GenerateFortran.printExperimentDefinition(out,include_edata2,include_spline,include_useip);
     	    out.write("      integer nexp,nvare,npts(maxexp)\n");
	    out.write("      double precision errcur(maxvar)\n");
	    out.write("      double precision errloc\n");  
	    out.write("      double precision errbes\n");
	    out.write("      integer ivar,DEPTH,debug,addterm\n");
	    out.write("      logical add\n");
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    GenerateFortran.printTempModelDefinition(out,include_modkinit);
	    out.write("C     Temp model 2 corresponds to the best model\n");
	    GenerateFortran.printTempModelDefinition2(out,include_modkinit);
	    out.write("      integer nvar\n");
	    out.write("      integer getKsOfType,getVsOfType\n");
	    out.write("      logical promising\n");
	    out.write("      logical initconc  ! T if init conc considered as parameter, ow F\n");
	    out.write("      double precision stdev(maxexp,maxvar,maxpts), lambda\n");
	    out.write("      integer isucc,nnn,i,ir1,ir2,jj,iv1,iv2,addedvar\n");
	    out.write("      double precision errtem\n");
	    out.write("      logical sign1,sign2\n");
	    out.write("      integer ipara,ireact,j\n");
	    out.write("      integer terms(100)  \n");  
	    out.write("      integer varindex(100) \n");
	    out.write("      integer tablesize\n");
	    out.write("\n");
	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n");
	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
 	    out.write("      common/errsettings/stdev,lambda\n");
 	    out.write("      common/myinitconc/initconc\n");
	    out.write("\n");
	    out.write("      if (DEPTH.lt.1.or.DEPTH.gt.2) then\n");
	    out.write("         write(*,*)'code_rk_myreactions, estvarrm, illegal DEPTH=',DEPTH\n");
	    out.write("         stop\n");
	    out.write("      endif\n");
	    out.write("      errbes=errloc\n");
	    out.write("      call icopy(maxvar*maxr,modtype,1,tmodtype,1) ! make temp copy\n");
	    out.write("      call dcopy(maxvar*maxr*maxk*3,modk,1,tmodk,1)\n");
     	    out.write("      call icopy(maxvar,modn,1,tmodn,1)\n");
	    out.write("      call icopy(maxvar*maxr*maxv,modv,1,tmodv,1)\n");
 	    out.write("      promising=.true.\n");
      	    out.write("\n");
      	    out.write("      tablesize=0\n");
       	    out.write("      do 1500 ir1=1,modn(ivar)\n"); 
	    int count=0;
      	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (p.getReactionType(i).getName().equals("GMAterm8Variables")){
		    Debug.staterror("GenerateFortran_myReactions","generate","GMAterm8Variables (or higher) not implemented for subroutine rmv.");
		    System.exit(0);
		}
		for (int gma=1;gma<=7;gma++){
		    String r="GMAterm"+gma+"Variables";
		    if (gma==1) r="GMAterm"+gma+"Variable";
		    if (p.getReactionType(i).getName().equals(r)){
			if (count==0) 
			    GenerateFortran.prettyPrint(out,"if (modtype(ivar,ir1).eq."+
						    p.getReactionType(i).getId()+
							") then",1);
			else 
			    GenerateFortran.prettyPrint(out,"else if (modtype(ivar,ir1).eq."+
							p.getReactionType(i).getId()+
							") then",1);
			count++;
			GenerateFortran.prettyPrint(out,"! if a "+r,2);
			GenerateFortran.prettyPrint(out,"do "+(100+10*gma)+" j=1,"+gma,2);
			GenerateFortran.prettyPrint(out,"if (ir1.eq.addterm.and.j.eq."+gma+") then",3);
			GenerateFortran.prettyPrint(out,"! Skip this reaction",4);
			GenerateFortran.prettyPrint(out,"else",3);
			GenerateFortran.prettyPrint(out,"tablesize=tablesize+1",4);
			GenerateFortran.prettyPrint(out,"terms(tablesize)=ir1",4);
			GenerateFortran.prettyPrint(out,"varindex(tablesize)=j",4);
			GenerateFortran.prettyPrint(out,"endif",3);
			GenerateFortran.prettyPrint(out,"enddo",2,(100+10*gma));
		    }
		}
	    }
	    if (count>0) GenerateFortran.prettyPrint(out,"else",1);
	    else GenerateFortran.prettyPrint(out,"if (.true.) then",1);
	    GenerateFortran.prettyPrint(out,"if (ir1.eq.modn(ivar).and.add) then",2);
	    GenerateFortran.prettyPrint(out,"! Skip this reaction",3);
	    GenerateFortran.prettyPrint(out,"else",2);
	    GenerateFortran.prettyPrint(out,"tablesize=tablesize+1",3);
	    GenerateFortran.prettyPrint(out,"terms(tablesize)=ir1 ! Traditional reaction type",3);
	    GenerateFortran.prettyPrint(out,"varindex(tablesize)=0",3);
	    GenerateFortran.prettyPrint(out,"endif",2);
	    GenerateFortran.prettyPrint(out,"endif",1);
	    GenerateFortran.prettyPrint(out,"enddo",0,1500);
       	    //out.write("      do 155 j=1,tablesize\n"); 
	    //GenerateFortran.prettyPrint(out,"write(*,*) terms(j),' ',varindex(j)",1);
	    //GenerateFortran.prettyPrint(out,"enddo",0,155);

            GenerateFortran.prettyPrint(out,"! Initial estimation",0);// initial estimation
            GenerateFortran.prettyPrint(out,"if (add.or.addterm.gt.0) then ",0);
	    GenerateFortran.prettyPrint(out,"call estvar12(ivar,errtem)",1);
            GenerateFortran.prettyPrint(out,"else",0);
	    GenerateFortran.prettyPrint(out,"call estvar(ivar,errtem,2,.true.)",1);
	    GenerateFortran.prettyPrint(out,"endif",0);

	    String modv="";
	    String modvformat="";
	    if (p.get_maxv()>=3) {
		modv="modv(ivar,modn(ivar),1),modv(ivar,modn(ivar),2),modv(ivar,modn(ivar),3),";
		modvformat="I3,I3,I3,";
	    }
	    else if (p.get_maxv()==2) {
		modv="modv(ivar,modn(ivar),1),modv(ivar,modn(ivar),2),";
		modvformat="I3,I3,";
	    }
	    else if (p.get_maxv()==1) {
		modv="modv(ivar,modn(ivar),1),";
		modvformat="I3,";
	    }
	    else if (p.get_maxv()==0) {
		modv="";
		modvformat="";
	    }
            GenerateFortran.prettyPrint(out,"if (add.and.debug.gt.0) then",0);
            GenerateFortran.prettyPrint(out,"if (errtem.lt.errcur(ivar).and.errtem.lt.errbes) then",1);
            GenerateFortran.prettyPrint(out,"write(*,700) 'add ', modtype(ivar,modn(ivar)),' ',"+modv+"' (ir1/2=',0,0,') err=',errtem,' < ',errcur(ivar),' *'",2);
            GenerateFortran.prettyPrint(out,"else if (errtem.lt.errcur(ivar)) then",1);
            GenerateFortran.prettyPrint(out,"write(*,701) 'add ', modtype(ivar,modn(ivar)),' ',"+modv+"' (ir1/2=',0,0,')  err=',errtem,' < ',errcur(ivar)",2);
            GenerateFortran.prettyPrint(out,"else",1);
            GenerateFortran.prettyPrint(out,"write(*,702) 'add ', modtype(ivar,modn(ivar)),' ',"+modv+"' (ir1/2=',0,0,')  err=',errtem",2);
            GenerateFortran.prettyPrint(out,"endif",1);
            GenerateFortran.prettyPrint(out,"endif",0);

            GenerateFortran.prettyPrint(out,"if (addterm.gt.0.and.debug.gt.0) then",0);
            GenerateFortran.prettyPrint(out,"addedvar=modv(ivar,addterm,getVsOfType(modtype(ivar,addterm)))",1);
            GenerateFortran.prettyPrint(out,"if (errtem.lt.errcur(ivar).and.errtem.lt.errbes) then",1);
            GenerateFortran.prettyPrint(out,"write(*,750) 'add var ',addedvar,' to term ',addterm,' err=',errtem,' < ',errcur(ivar),' *'",2);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,D10.3,A,D10.3,A)",2,750);
            GenerateFortran.prettyPrint(out,"else if (errtem.lt.errcur(ivar)) then",1);
            GenerateFortran.prettyPrint(out,"write(*,751) 'add var ',addedvar,' to term ',addterm,' err=',errtem,' < ',errcur(ivar)",2);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,D10.3,A,D10.3)",2,751);
            GenerateFortran.prettyPrint(out,"else",1);
            GenerateFortran.prettyPrint(out,"write(*,752) 'add var ',addedvar,' to term ',addterm,' err=',errtem",2);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,D10.3)",2,752);
            GenerateFortran.prettyPrint(out,"endif",1);
            GenerateFortran.prettyPrint(out,"else if (.not.add.and.debug.gt.0) then",0);
            GenerateFortran.prettyPrint(out,"endif",0);
            GenerateFortran.prettyPrint(out,"if (.not.add.and.addterm.eq.0.and.debug.gt.0) then",0);
            GenerateFortran.prettyPrint(out,"write(*,703) 'estvarrm, ir1/2=',0,',',0,' e=',errtem",1);
            GenerateFortran.prettyPrint(out,"endif",0);



	    GenerateFortran.prettyPrint(out,"if (debug.gt.2) then ",0);
	    GenerateFortran.prettyPrint(out,"write(*,*) 'estvarrm, init error=',errtem",1);
	    GenerateFortran.prettyPrint(out,"call printvar(maxvar,maxr,maxv,maxk,nvar,modtype,"+
					"modk,modn,modv,dependent,ivar)",1);
            GenerateFortran.prettyPrint(out,"endif",0);
	    GenerateFortran.prettyPrint(out,"if (errtem.lt.errcur(ivar).and.errtem.lt.errbes) then",0);
	    GenerateFortran.prettyPrint(out,"errbes=errtem ",1);
	    GenerateFortran.prettyPrint(out,"! Make a temporary copy",1);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr,modtype,1,t2modtype,1)",1);
	    GenerateFortran.prettyPrint(out,"call dcopy(maxvar*maxr*maxk*3,modk,1,t2modk,1)",1);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar,modn,1,t2modn,1)",1);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr*maxv,modv,1,t2modv,1)",1);
            GenerateFortran.prettyPrint(out,"endif",0);
	    GenerateFortran.prettyPrint(out,"if (add) then",0);
	    GenerateFortran.prettyPrint(out,"if (errtem.gt.(errloc+lambda*getKsOfType(modtype(ivar,modn(ivar))))) then",1);
	    GenerateFortran.prettyPrint(out,"promising=.false.",2);
	    GenerateFortran.prettyPrint(out,"endif",1);
	    GenerateFortran.prettyPrint(out,"endif",0);
	    GenerateFortran.prettyPrint(out,"if (addterm.gt.0) then",0);
	    GenerateFortran.prettyPrint(out,"if (errtem.gt.(errloc+lambda*1)) then ! one more parameter",1);
	    GenerateFortran.prettyPrint(out,"promising=.false.",2);
	    GenerateFortran.prettyPrint(out,"endif",1);
	    GenerateFortran.prettyPrint(out,"endif",0);

            GenerateFortran.prettyPrint(out,"! Remove existing terms",0);// remove terms
	    GenerateFortran.prettyPrint(out,"! Note that ir2>=ir1",0);
	    GenerateFortran.prettyPrint(out,"do 600 ir1=tablesize,1,-1",0);
	    GenerateFortran.prettyPrint(out,"do 500 ir2=tablesize,ir1,-1",1);
	    GenerateFortran.prettyPrint(out,"if (.not.promising) goto 300",2);
	    GenerateFortran.prettyPrint(out,"sign1=(modk(ivar,modn(ivar),1,2).ge.0.and.modk(ivar,terms(ir1),1,2).ge.0)."+
					"or.(modk(ivar,modn(ivar),1,3).le.0.and.modk(ivar,terms(ir1),1,3).le.0)",2);
	    GenerateFortran.prettyPrint(out,"!added react same sign as react to remove?",2);
	    GenerateFortran.prettyPrint(out,"sign2=(modk(ivar,modn(ivar),1,2).ge.0.and.modk(ivar,terms(ir2),1,2).ge.0)."+
					"or.(modk(ivar,modn(ivar),1,3).le.0.and.modk(ivar,terms(ir2),1,3).le.0)",2);
	    GenerateFortran.prettyPrint(out,"if (add.and.DEPTH.eq.1.and..not.sign1) goto 300",2);
	    GenerateFortran.prettyPrint(out,"if (add.and.DEPTH.eq.2.and.(.not.sign1.and..not.sign2)) goto 300",2);
	    GenerateFortran.prettyPrint(out,"if (ir1.eq.ir2) then ! Remove one term",2);
	    GenerateFortran.prettyPrint(out,"if (varindex(ir1).gt.0) then",3);
	    GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,modk,"+
					"modn,modv,dependent,ivar,varindex(ir1),terms(ir1),isucc)",4);
	    GenerateFortran.prettyPrint(out,"else",3);
	    GenerateFortran.prettyPrint(out,"call rm(maxvar,maxr,maxv,maxk,nvar,modtype,modk,modn,"+
					"modv,dependent,ivar,terms(ir1),isucc)",4);
	    GenerateFortran.prettyPrint(out,"endif",3);
	    GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 300",3);
	    GenerateFortran.prettyPrint(out,"else if (ir1.ne.ir2.and.DEPTH.eq.2) then ! Remove 2 terms",2);
	    GenerateFortran.prettyPrint(out,"if (varindex(ir2).gt.0) then",3);
	    GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,modk,"+
					"modn,modv,dependent,ivar,varindex(ir2),terms(ir2),isucc)",4);
	    GenerateFortran.prettyPrint(out,"else",3);
	    GenerateFortran.prettyPrint(out,"call rm(maxvar,maxr,maxv,maxk,nvar,modtype,modk,modn,"+
					"modv,dependent,ivar,terms(ir2),isucc)",4);
	    GenerateFortran.prettyPrint(out,"endif",3);
	    GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 300",3);
	    GenerateFortran.prettyPrint(out,"if (varindex(ir1).gt.0) then",3);
	    GenerateFortran.prettyPrint(out,"call rmv(maxvar,maxr,maxv,maxk,nvar,modtype,modk,"+
					"modn,modv,dependent,ivar,varindex(ir1),terms(ir1),isucc)",4);
	    GenerateFortran.prettyPrint(out,"else",3);
	    GenerateFortran.prettyPrint(out,"call rm(maxvar,maxr,maxv,maxk,nvar,modtype,modk,modn,"+
					"modv,dependent,ivar,terms(ir1),isucc)",4);
	    GenerateFortran.prettyPrint(out,"endif",3);
	    GenerateFortran.prettyPrint(out,"if (isucc.eq.0) goto 300",3);
	    GenerateFortran.prettyPrint(out,"else if (ir1.ne.ir2.and.DEPTH.eq.1) then ! Continue",2);
	    GenerateFortran.prettyPrint(out,"goto 300",3);
	    GenerateFortran.prettyPrint(out,"endif",2);

	    GenerateFortran.prettyPrint(out,"! Estimation",2);
            GenerateFortran.prettyPrint(out,"if (add.or.addterm.gt.0) then ",2);
	    GenerateFortran.prettyPrint(out,"call estvar12(ivar,errtem)",3);
            GenerateFortran.prettyPrint(out,"else",3);
	    GenerateFortran.prettyPrint(out,"call estvar(ivar,errtem,2,.true.)",3);
	    GenerateFortran.prettyPrint(out,"endif",2);
	    GenerateFortran.prettyPrint(out,"if (debug.gt.2) then ",2);
	    GenerateFortran.prettyPrint(out,"write(*,*) 'estvarrm, init error=',errtem",3);
	    GenerateFortran.prettyPrint(out,"call printvar(maxvar,maxr,maxv,maxk,nvar,modtype,"+
					"modk,modn,modv,dependent,ivar)",3);
            GenerateFortran.prettyPrint(out,"endif",2);


	    GenerateFortran.prettyPrint(out,"if (add.and.debug.gt.0) then",2);
	    GenerateFortran.prettyPrint(out,"if (errtem.lt.errcur(ivar).and.errtem.lt.errbes) then",3);
	    GenerateFortran.prettyPrint(out,"write(*,700) 'add ', modtype(ivar,modn(ivar)),' ',"+modv+"' (ir1/2=',ir1,ir2,') err=',errtem,' < ',errcur(ivar),' *'",4);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,"+modvformat+"A,I3,I3,A,D10.3,A,D10.3,A)",4,700);
	    GenerateFortran.prettyPrint(out,"else if (errtem.lt.errcur(ivar)) then",3);
	    GenerateFortran.prettyPrint(out,"write(*,701) 'add ', modtype(ivar,modn(ivar)),' ',"+modv+"' (ir1/2=',ir1,ir2,')  err=',errtem,' < ',errcur(ivar)",4);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,"+modvformat+"A,I3,I3,A,D10.3,A,D10.3)",4,701);
	    GenerateFortran.prettyPrint(out,"else",3);
	    GenerateFortran.prettyPrint(out,"write(*,702) 'add ', modtype(ivar,modn(ivar)),' ',"+modv+"' (ir1/2=',ir1,ir2,')  err=',errtem",4);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,"+modvformat+"A,I3,I3,A,D10.3)",4,702);
	    GenerateFortran.prettyPrint(out,"endif",3);
            GenerateFortran.prettyPrint(out,"endif",2);
	    
	    GenerateFortran.prettyPrint(out,"if (addterm.gt.0.and.debug.gt.0) then",2);
	    GenerateFortran.prettyPrint(out,"if (errtem.lt.errcur(ivar).and.errtem.lt.errbes) then",3);
	    GenerateFortran.prettyPrint(out,"write(*,800) 'add var ',addedvar,' to term ',addterm,' (ir1/2=',ir1,ir2,'"+
					") err=',errtem,' < ',errcur(ivar),' *'",4);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,I3,I3,A,D10.3,A,D10.3,A)",4,800);
	    GenerateFortran.prettyPrint(out,"else if (errtem.lt.errcur(ivar)) then",3);
	    GenerateFortran.prettyPrint(out,"write(*,801) 'add var ',addedvar,' to term ',addterm,' (ir1/2=',ir1,ir2,'"+
					") err=',errtem,' < ',errcur(ivar)",4);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,I3,I3,A,D10.3,A,D10.3)",4,801);
	    GenerateFortran.prettyPrint(out,"else",3);
	    GenerateFortran.prettyPrint(out,"write(*,802) 'add var ',addedvar,' to term ',addterm,' (ir1/2=',ir1,ir2,'"+
					") err=',errtem",4);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,I3,I3,A,D10.3)",4,802);
	    GenerateFortran.prettyPrint(out,"endif",3);
            GenerateFortran.prettyPrint(out,"else if (.not.add.and.debug.gt.0) then",2);
            GenerateFortran.prettyPrint(out,"endif",2);


            GenerateFortran.prettyPrint(out,"if (.not.add.and.addterm.eq.0.and.debug.gt.0) then",2);
	    GenerateFortran.prettyPrint(out,"write(*,703) 'estvarrm, ir1/2=',ir1,',',ir2,' e=',errtem",3);
	    GenerateFortran.prettyPrint(out,"format(A,I3,A,I3,A,D10.3)",3,703);
            GenerateFortran.prettyPrint(out,"endif",2);


	    GenerateFortran.prettyPrint(out,"if (errtem.lt.errcur(ivar).and.errtem.lt.errbes) then",2);
	    GenerateFortran.prettyPrint(out,"errbes=errtem ",3);
	    GenerateFortran.prettyPrint(out,"! Make a temporary copy",3);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr,modtype,1,t2modtype,1)",3);
	    GenerateFortran.prettyPrint(out,"call dcopy(maxvar*maxr*maxk*3,modk,1,t2modk,1)",3);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar,modn,1,t2modn,1)",3);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr*maxv,modv,1,t2modv,1)",3);
            GenerateFortran.prettyPrint(out,"endif",2);
	    GenerateFortran.prettyPrint(out,"continue ",2,300);           
            GenerateFortran.prettyPrint(out,"! go to when continueing the reaction loop,",2);
            GenerateFortran.prettyPrint(out,"! go to next pair ir1,ir2",2);
            GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr,tmodtype,1,modtype,1) ! Reset",2);
            GenerateFortran.prettyPrint(out,"call dcopy(maxvar*maxr*maxk*3,tmodk,1,modk,1)",2);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar,tmodn,1,modn,1)",2);
            GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr*maxv,tmodv,1,modv,1)",2);
	    GenerateFortran.prettyPrint(out,"enddo",1,500);
	    GenerateFortran.prettyPrint(out,"enddo",0,600);


	    GenerateFortran.prettyPrint(out,"if (errbes.lt.errloc) then",0);
	    GenerateFortran.prettyPrint(out,"! Set best model",1);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr,t2modtype,1,modtype,1)",1);
	    GenerateFortran.prettyPrint(out,"call dcopy(maxvar*maxr*maxk*3,t2modk,1,modk,1)",1);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar,t2modn,1,modn,1)",1);
	    GenerateFortran.prettyPrint(out,"call icopy(maxvar*maxr*maxv,t2modv,1,modv,1)",1);
	    GenerateFortran.prettyPrint(out,"endif",0);
	    GenerateFortran.prettyPrint(out,"end",0);


      










	    // rmv
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, rmv");
	    out.write("\n");
	    out.write("C     Remove variable i from reaction number j of variable ivar\n");
	    out.write("C     Requires a GMA term, o.w. nothing happens, isucc=0\n");
	    out.write("C     ivar, variable\n");
	    out.write("C     i, variable index\n");
	    out.write("C     j, reaction number\n");
	    out.write("C     isucc, OUTPUT\n");
	    out.write("C            1=success, 0=no success (e.g. not a GMA term)\n");
	    out.write("      subroutine rmv(maxvar,maxr,maxv,maxk,nvar,\n");
	    out.write("     *     modtype,modk,modn,modv,dependent,\n");
 	    out.write("     *     ivar,i,j,isucc)\n");
	    out.write("      implicit none\n");
	    out.write("      integer maxvar,maxr,maxv,maxk,nvar\n");
	    include_modkinit=false;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    out.write("      integer ivar,i,j,isucc,ik,iv,ireact\n");
      	    out.write("      isucc=0\n");




            if (p.isSSystem()){
                boolean[][] g=p.getGnonzero();
		for (int i=0;i<p.getNumberOfVariables();i++){
		    for (int j=0;j<p.getNumberOfVariables();j++){
			if (g[i][j]) {
			    GenerateFortran.prettyPrint(out,"if (j.eq.1.and.ivar.eq."+(i+1)+".and.modv(ivar,j,i).eq."+(j+1)+") then",0);
			    GenerateFortran.prettyPrint(out,"return ! gnonzero("+(i+1)+","+(j+1)+")=TRUE",1);                        
			    GenerateFortran.prettyPrint(out,"endif",0);
			}
		    }
		}
                boolean[][] h=p.getHnonzero();
                for (int i=0;i<p.getNumberOfVariables();i++){
                    for (int j=0;j<p.getNumberOfVariables();j++){
                        if (h[i][j]) {
                            GenerateFortran.prettyPrint(out,"if (j.eq.2.and.ivar.eq."+(i+1)+".and.modv(ivar,j,i).eq."+(j+1)+") then",0);
                            GenerateFortran.prettyPrint(out,"return ! hnonzero("+(i+1)+","+(j+1)+")=TRUE",1);
                            GenerateFortran.prettyPrint(out,"endif",0);
                        }
                    }
                }

	    }
	    count=0;
      	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (p.getReactionType(i).getName().equals("GMAterm8Variables")){
		    Debug.staterror("GenerateFortran_myReactions","generate","GMAterm8Variables (or higher) not implemented for subroutine rmv.");
		    System.exit(0);
		}
		for (int gma=1;gma<=7;gma++){
		    String r="GMAterm"+gma+"Variables";
		    if (gma==1) r="GMAterm"+gma+"Variable";
		    if (p.getReactionType(i).getName().equals(r)){
			if (count==0) 
			    GenerateFortran.prettyPrint(out,"if (modtype(ivar,j).eq."+
							p.getReactionType(i).getId()+
							") then",0);
			else 
			    GenerateFortran.prettyPrint(out,"else if (modtype(ivar,j).eq."+
							p.getReactionType(i).getId()+
							") then",0);
			count++;
			ReactionType rgoto=p.getReactionType("GMAterm"+(gma-1)+"Variables");
			if (gma==2) rgoto=p.getReactionType("GMAterm"+(gma-1)+"Variable");
			if (rgoto!=null){
			    GenerateFortran.prettyPrint(out,"! if a "+r,0);
			    GenerateFortran.prettyPrint(out,"if(i.le."+gma+".and.i.gt.0) then",1);
			    GenerateFortran.prettyPrint(out,"! index must be in [1,"+gma+"]",1);
			    GenerateFortran.prettyPrint(out,"! if lower bound>0 return",2);
			    GenerateFortran.prettyPrint(out,"if (modk(ivar,j,i+1,2).gt.0.0) return ",2);
			    GenerateFortran.prettyPrint(out,"! if upper bound<0 return",2);
			    GenerateFortran.prettyPrint(out,"if (modk(ivar,j,i+1,3).lt.0.0) return ",2);
			    GenerateFortran.prettyPrint(out,"do "+(gma*10)+" ik=i+2,"+(gma+1),2);
			    GenerateFortran.prettyPrint(out,"! shift param and var indices towards removed variable",2);
			    GenerateFortran.prettyPrint(out,"iv=ik-1",3);
			    GenerateFortran.prettyPrint(out,"modk(ivar,j,ik-1,1)=modk(ivar,j,ik,1)",3);
			    GenerateFortran.prettyPrint(out,"modv(ivar,j,iv-1)=modv(ivar,j,iv)",3);
			    GenerateFortran.prettyPrint(out,"continue",2,(gma*10));
			    GenerateFortran.prettyPrint(out,"modk(ivar,j,"+(gma+1)+",1)=0.d0  !remove last var",2);
			    GenerateFortran.prettyPrint(out,"modv(ivar,j,"+gma+")=0",2);
			    GenerateFortran.prettyPrint(out,"modtype(ivar,j)="+rgoto.getId(),2);
			    GenerateFortran.prettyPrint(out,"! New type: "+rgoto.getName(),2);
			    GenerateFortran.prettyPrint(out,"else ! incorrect variable index",1);
			    GenerateFortran.prettyPrint(out,"return",2);
			    GenerateFortran.prettyPrint(out,"endif",1);
			}
			else{
			    GenerateFortran.prettyPrint(out,"! GMAterm"+(gma-1)+"Variable(s) is not defined",0);
			    GenerateFortran.prettyPrint(out,"! Cannot change state to it.",0);
			}
		    }
		}
	    }
	    if (count>0) {
		GenerateFortran.prettyPrint(out,"else",0);
		GenerateFortran.prettyPrint(out,"return",1);
		GenerateFortran.prettyPrint(out,"endif",0);
	    }
	    else GenerateFortran.prettyPrint(out,"return",0);
	    out.write("      isucc=1\n");
	    out.write("      end\n");
      	

	    // rm
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, rm");
	    out.write("\n");
	    out.write("C     Remove reaction number i from variable ivar\n");
	    out.write("C     isucc 1=success, 0=no success OUTPUT\n");
	    out.write("      subroutine rm(maxvar,maxr,maxv,maxk,nvar,\n");
	    out.write("     *     modtype,modk,modn,modv,dependent,\n");
 	    out.write("     *     ivar,i,isucc)\n");
	    out.write("      implicit none\n");
	    out.write("      integer maxvar,maxr,maxv,maxk,nvar\n");
	    include_modkinit=false;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
	    out.write("      integer ivar,i,isucc,ik,iv,ireact\n");
	    if (p.isSSystem()){
		GenerateFortran.prettyPrint(out,"! S-system, no removal of terms.",0);
		count=0;
		for (int i=0;i<p.getNumberOfReactionTypes();i++){
		    if (p.getReactionType(i).getName().equals("GMAterm8Variables")){
			Debug.staterror("GenerateFortran_myReactions","generate",
					"GMAterm8Variables (or higher) not implemented for subroutine rm.");
			System.exit(0);
		    }
		    for (int gma=0;gma<=7;gma++){
			String r="GMAterm"+gma+"Variables";
			if (gma==1) r="GMAterm"+gma+"Variable";
			if (p.getReactionType(i).getName().equals(r)){
			    if (count==0) 
			    GenerateFortran.prettyPrint(out,"if (modtype(ivar,i).eq."+
							p.getReactionType(i).getId()+
							") then",0);
			    else
				GenerateFortran.prettyPrint(out,"else if (modtype(ivar,i).eq."+
							    p.getReactionType(i).getId()+
							    ") then",0);
			    count++;
			    GenerateFortran.prettyPrint(out,"! if a "+r,1);
			    GenerateFortran.prettyPrint(out,"isucc=0",1);
			    GenerateFortran.prettyPrint(out,"return",1);
			}
		    }
		}
		if (count>0) GenerateFortran.prettyPrint(out,"endif",0);
	    }
	    out.write("      if (modn(ivar).gt.0) then\n");
	    out.write("         do 100 ireact=i+1,modn(ivar)\n");
	    out.write("            modtype(ivar,ireact-1)=modtype(ivar,ireact)\n");
 	    out.write("           do 30 iv=1,maxv\n");
	    out.write("               modv(ivar,ireact-1,iv)=modv(ivar,ireact,iv)\n");
 	    out.write(" 30        enddo\n");
 	    out.write("           do 60 ik=1,maxk\n");
 	    out.write("              modk(ivar,ireact-1,ik,1)=modk(ivar,ireact,ik,1)\n");
 	    out.write("              modk(ivar,ireact-1,ik,2)=modk(ivar,ireact,ik,2)\n");
 	    out.write("              modk(ivar,ireact-1,ik,3)=modk(ivar,ireact,ik,3)\n");
 	    out.write(" 60         enddo\n");
 	    out.write(" 100      continue\n");
	    out.write("         modn(ivar)=modn(ivar)-1\n");
 	    out.write("         isucc=1\n");
 	    out.write("      else\n");
 	    out.write("         isucc=0\n");
	    out.write("      endif\n");
	    out.write("      end\n");
	    




	    // addv
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, addv");
	    out.write("\n");
	    out.write("C     Add variable i to reaction number j of variable ivar\n");
	    out.write("C     Requires a GMA term, o.w. nothing happens, isucc=0\n");
	    out.write("C     ivar, variable\n");
      	    out.write("C     rlisttype(maxvar,maxirl), table of possible reaction types to add\n");
      	    out.write("C     rlistv(maxvar,maxirl,maxv), table of vars to each reaction \n");
      	    out.write("C     rlistk(maxvar,maxirl,maxk,2), table of param bounds to each reaction\n");
      	    out.write("C     maxirl, max rows in rlist\n");
	    out.write("C     i, variable index\n");
	    out.write("C     j, reaction number\n");
	    out.write("C     isucc, OUTPUT\n");
	    out.write("C            0=success, -1=no success (e.g. not a GMA term)\n");
	    out.write("      subroutine addv(nvar,ivar,rlisttype,rlistv,rlistk,maxirl,\n");
	    out.write("     *     i,j,debug,isucc)\n");
	    out.write("      implicit none\n");
	    GenerateFortran.printMaxDefinition(out,p);
	    out.write("      integer nvar\n");
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
      	    out.write("      integer ivar,debug,isucc\n");
      	    out.write("      integer maxirl\n");
      	    out.write("      integer rlisttype(maxvar,maxirl)\n");
      	    out.write("      integer rlistv(maxvar,maxirl,maxv)\n");
      	    out.write("      double precision rlistk(maxvar,maxirl,maxk,2)\n");
      	    out.write("      integer i,j,ireact\n");
      	    out.write("      integer itype,nvs,nks,getVsOfType,getKsOfType,iv,ir,jj,ik\n");
      	    out.write("      logical inmodel\n");
      	    out.write("      double precision value,lb,ub\n");
      	    out.write("      integer vs(maxv),irl\n");
      	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
      	    out.write("      isucc=-1\n");
	    GenerateFortran.prettyPrint(out,"if(i.gt.nvar.or.i.le.0) then",0);
	    GenerateFortran.prettyPrint(out,"! index must be in [1,nvar]",0);
	    GenerateFortran.prettyPrint(out,"return",1);
	    GenerateFortran.prettyPrint(out,"endif",0);
	    count=0;
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (p.getReactionType(i).getName().equals("GMAterm8Variables")){
		    Debug.staterror("GenerateFortran_myReactions","generate","GMAterm8Variables (or higher) not implemented for subroutine rmv.");
		    System.exit(0);
		}
		for (int gma=0;gma<=7;gma++){
		    String r="GMAterm"+gma+"Variables";
		    if (gma==1) r="GMAterm"+gma+"Variable";
		    if (p.getReactionType(i).getName().equals(r)){
			if (count==0) 
			    GenerateFortran.prettyPrint(out,"if (modtype(ivar,j).eq."+
							p.getReactionType(i).getId()+
							") then",0);
			else
			    GenerateFortran.prettyPrint(out,"else if (modtype(ivar,j).eq."+
							p.getReactionType(i).getId()+
							") then",0);
			count++;
			ReactionType rgoto=p.getReactionType("GMAterm"+(gma+1)+"Variables");
			if (gma==0) rgoto=p.getReactionType("GMAterm"+(gma+1)+"Variable");
			if (rgoto!=null){
			    GenerateFortran.prettyPrint(out,"! if a "+r,0);
			    GenerateFortran.prettyPrint(out,"do "+(10+gma*10)+" iv=1,"+gma,1);
			    GenerateFortran.prettyPrint(out,"! check that var i is not already in use",1);
			    GenerateFortran.prettyPrint(out,"if (modv(ivar,j,iv).eq.i) then",2);
			    GenerateFortran.prettyPrint(out,"return",3);
			    GenerateFortran.prettyPrint(out,"endif",2);
			    GenerateFortran.prettyPrint(out,"continue",1,(10+gma*10));
			    GenerateFortran.prettyPrint(out,"modv(ivar,j,"+(gma+1)+")=i",1);
			    GenerateFortran.prettyPrint(out,"ik="+(gma+2),1);
			    GenerateFortran.prettyPrint(out,"modtype(ivar,j)="+rgoto.getId(),1);
			    GenerateFortran.prettyPrint(out,"! New type: "+rgoto.getName(),1);
			}
			else{
			    GenerateFortran.prettyPrint(out,"! GMAterm"+(gma+1)+"Variable(s) is not defined",1);
			    GenerateFortran.prettyPrint(out,"! Cannot change state to it.",1);
			    GenerateFortran.prettyPrint(out,"write(*,*) 'addv, GMAterm"+(gma+1)+"Variable(s) is not defined'",1);
			    GenerateFortran.prettyPrint(out,"write(*,*) 'addv, Cannot change state to it.'",1);
			    //GenerateFortran.prettyPrint(out,"stop",1);
			    GenerateFortran.prettyPrint(out,"return",1);
			}
		    }
		}
	    }
	    if (count>0) {
		GenerateFortran.prettyPrint(out,"else",0);
		GenerateFortran.prettyPrint(out,"return",1);
		GenerateFortran.prettyPrint(out,"endif",0);
	    }
	    else GenerateFortran.prettyPrint(out,"return",0);

	    if (p.isSSystem()){
		double[][][] g=p.getG();
		GenerateFortran.prettyPrint(out,"if (j.eq.1) then ! g",0);
		for (int i=0;i<p.getNumberOfVariables();i++){
		    if (i==0) GenerateFortran.prettyPrint(out,"if (ivar.eq."+(i+1)+") then",1);
		    else GenerateFortran.prettyPrint(out,"else if (ivar.eq."+(i+1)+") then",1);
		    for (int j=0;j<p.getNumberOfVariables();j++){
			if (j==0) GenerateFortran.prettyPrint(out,"if (i.eq."+(j+1)+") then",2);
			else GenerateFortran.prettyPrint(out,"else if (i.eq."+(j+1)+") then",2);
			GenerateFortran.prettyPrint(out,"value=0.01",3);
			GenerateFortran.prettyPrint(out,"lb="+g[i][j][1],3);
			GenerateFortran.prettyPrint(out,"ub="+g[i][j][2],3);
		    }
		    GenerateFortran.prettyPrint(out,"endif",2);
		}
		GenerateFortran.prettyPrint(out,"endif",1);
		double[][][] h=p.getH();
		GenerateFortran.prettyPrint(out,"else if (j.eq.2) then ! h",0);
		for (int i=0;i<p.getNumberOfVariables();i++){
		    if (i==0) GenerateFortran.prettyPrint(out,"if (ivar.eq."+(i+1)+") then",1);
		    else GenerateFortran.prettyPrint(out,"else if (ivar.eq."+(i+1)+") then",1);
		    for (int j=0;j<p.getNumberOfVariables();j++){
			if (j==0) GenerateFortran.prettyPrint(out,"if (i.eq."+(j+1)+") then",2);
			else GenerateFortran.prettyPrint(out,"else if (i.eq."+(j+1)+") then",2);
			GenerateFortran.prettyPrint(out,"value=0.01",3);
			GenerateFortran.prettyPrint(out,"lb="+h[i][j][1],3);
			GenerateFortran.prettyPrint(out,"ub="+h[i][j][2],3);
		    }
		    GenerateFortran.prettyPrint(out,"endif",2);
		}
		GenerateFortran.prettyPrint(out,"endif",1);
		GenerateFortran.prettyPrint(out,"endif",0);
		GenerateFortran.prettyPrint(out,"if (lb.eq.0.d0.and.ub.eq.0.d0) then",0);
		GenerateFortran.prettyPrint(out,"return",1);
		GenerateFortran.prettyPrint(out,"endif",0);
		GenerateFortran.prettyPrint(out,"if (value.lt.lb.or.value.gt.ub) then",0);
		GenerateFortran.prettyPrint(out,"value=(lb+ub)/2",1);
		GenerateFortran.prettyPrint(out,"endif",0);
	    }
	    else{
		out.write("      itype=modtype(ivar,j)\n");								
		out.write("      do 100, irl=1,maxirl   ! max rows in rlist\n");
		out.write("         if (rlisttype(ivar,irl).eq.itype) then\n");
		out.write("            lb=rlistk(ivar,irl,ik,1)\n");
		out.write("            ub=rlistk(ivar,irl,ik,2)\n");
		out.write("         endif\n");
		out.write(" 100  continue\n");
		out.write("      call getInitialValue(itype,ik, !set new parameter value\n");
		out.write("     *        lb,ub,value)\n");
	    }
	    out.write("      modk(ivar,j,ik,1)=value\n");
 	    out.write("      modk(ivar,j,ik,2)=lb\n");
 	    out.write("      modk(ivar,j,ik,3)=ub\n");
	    out.write("      isucc=0\n");
	    out.write("      end\n");
         
         


	    // addr
	    Debug.stat("GenerateFortran_myReactions","generate","code_rk_myreactions.f, addr");
	    out.write("\n");
	    out.write("C     Add reaction i from table rlist to variable ivar\n");
      	    out.write("C     rlisttype(maxvar,maxirl), table of possible reaction types to add\n");
      	    out.write("C     rlistv(maxvar,maxirl,maxv), table of vars to each reaction \n");
      	    out.write("C     rlistk(maxvar,maxirl,maxk,2), table of param bounds to each reaction\n");
      	    out.write("C     maxirl, max rows in rlist\n");
	    out.write("C     isucc, OUTPUT\n");
	    out.write("C            0=success, -1=no success\n");
	    out.write("      subroutine addr(nvar,ivar,\n");
	    out.write("     *     rlisttype,rlistv,rlistk,maxirl,i,\n"); 
	    out.write("     *     debug,isucc)\n");
	    out.write("      implicit none\n");
	    GenerateFortran.printMaxDefinition(out,p);
	    out.write("      integer nvar\n");
	    include_modkinit=true;
	    GenerateFortran.printModelDefinition(out,include_modkinit);
 	    out.write("      integer ivar,debug,isucc\n");
 	    out.write("      integer maxirl \n");     
	    out.write("      integer rlisttype(maxvar,maxirl) \n");
	    out.write("      integer rlistv(maxvar,maxirl,maxv) \n");  
	    out.write("      double precision rlistk(maxvar,maxirl,maxk,2)\n"); 
	    out.write("      integer i,ireact\n");
	    out.write("      integer itype,nvs,nks,iv,jj,ik,getVsOfType,getKsOfType\n");
	    out.write("      logical inmodel\n");
	    out.write("      double precision value\n");
	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
      	    out.write("      itype=rlisttype(ivar,i)\n");
	    out.write("      nvs=getVsOfType(itype)\n");
	    out.write("      nks=getKsOfType(itype)\n");
	    count=0;
	    int GMAINIT=2;// add gma term with o or 1 variable
	    if (p.isSSystem()) GMAINIT=0; // add no GMA terms
	    for (int i=0;i<p.getNumberOfReactionTypes();i++){
		if (p.getReactionType(i).getName().equals("GMAterm8Variables")){
		    Debug.staterror("GenerateFortran_myReactions","generate",
				    "GMAterm8Variables (or higher) not implemented for subroutine addr.");
		    System.exit(0);
		}
		for (int gma=GMAINIT;gma<=7;gma++){
		    String r="GMAterm"+gma+"Variables";
		    if (gma==1) r="GMAterm"+gma+"Variable";
		    if (p.getReactionType(i).getName().equals(r)){
			if (count==0) 
			    GenerateFortran.prettyPrint(out,"if (itype.eq."+
							p.getReactionType(i).getId()+
							") then",0);
			else
			    GenerateFortran.prettyPrint(out,"else if (itype.eq."+
							p.getReactionType(i).getId()+
							") then",0);
			count++;
			GenerateFortran.prettyPrint(out,"! if a "+r,1);
			GenerateFortran.prettyPrint(out,"isucc=-1",1);
			GenerateFortran.prettyPrint(out,"! Restricted addition of GMA terms",1);
			GenerateFortran.prettyPrint(out,"return",1);
		    }
		}
	    }
	    if (count>0) GenerateFortran.prettyPrint(out,"endif",0);
      
	    out.write("      if (modn(ivar).ge.maxr) then\n");
	    out.write("         isucc=-1\n");
	    out.write("         write(*,*) 'add ',itype,' ',\n");
	    out.write("     *        (rlistv(ivar,i,jj),jj=1,nvs),\n");
	    out.write("     *        ' maxr = ',maxr,' reached'\n");
	    out.write("         stop\n");
	    out.write("         return\n");
	    out.write("      endif\n");
      
	    out.write("      do 100 ireact=1,modn(ivar)\n");
	    out.write("         if (rlisttype(ivar,i).eq.modtype(ivar,ireact)) then\n");
	    out.write("            inmodel=.true.\n");
	    out.write("            do 90 iv=1,getVsOfType(rlisttype(ivar,i))\n");
 	    out.write("              if (rlistv(ivar,i,iv).ne.modv(ivar,ireact,iv)) then \n");
	    out.write("                  inmodel=.false.\n");
 	    out.write("              endif\n");
 	    out.write(" 90         enddo\n");
 	    out.write("            if (inmodel) then\n");
 	    out.write("	             if (debug.gt.0) write(*,*) 'add ',itype,' ',\n");
	    out.write("     *              (rlistv(ivar,i,jj),jj=1,nvs),\n");
 	    out.write("     *              ' already in model'\n");
 	    out.write("              isucc=-1\n");
 	    out.write("              return\n");
	    out.write("            endif\n");
	    out.write("         endif\n");
	    out.write(" 100  enddo\n");

 	    out.write("      do 200 ik=1,nks\n");
 	    out.write("         call getInitialValue(itype,ik,\n");
 	    out.write("     *         rlistk(ivar,i,ik,1),rlistk(ivar,i,ik,2),value)\n");
	    out.write("         modk(ivar,modn(ivar)+1,ik,1)=value\n");
	    out.write("         modk(ivar,modn(ivar)+1,ik,2)=rlistk(ivar,i,ik,1)\n");
 	    out.write("         modk(ivar,modn(ivar)+1,ik,3)=rlistk(ivar,i,ik,2)\n");
 	    out.write(" 200  enddo\n");
 	    out.write("      do 300 iv=1,nvs\n");
	    out.write("         modv(ivar,modn(ivar)+1,iv)=rlistv(ivar,i,iv)\n");
	    out.write(" 300  enddo \n");
	    out.write("      do 400 iv=nvs+1,maxv       ! Set potential remaining positions to zero\n");
	    out.write("         modv(ivar,modn(ivar)+1,iv)=0\n");
	    out.write(" 400  enddo \n");           
	    out.write("      modtype(ivar,modn(ivar)+1)=itype\n");
	    out.write("      modn(ivar)=modn(ivar)+1\n");
	    out.write("      isucc=0\n");
	    out.write("      return\n");
 	    out.write("      end\n");

      
	   //Close the output stream
	    out.close();
	}catch (Exception e){
	    System.err.println("Error: " + e.getMessage());
	}
	
    }
	

}