/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/


package Java.generateFortran;

import java.io.*;
import Java.debug.*;
import Java.problem.*;

/**
 * Generate readProblem.f
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class GenerateFortran_readProblem {




    // Combinatorial tree of possible reactions
    private static int countComb=1;
    private static void comb(int[] history, int level,Problem p,Variable v,PossibleReaction pr,BufferedWriter out,boolean dowrite) throws IOException{
	//System.out.println("comb  level="+level+" "+history[0]+" "+history[1]);
	//System.out.println("comb  level="+level+" "+history[0]);
	if (level==pr.getNumberOfLocalVariableSets()){
	    // Are there any duplicates?
	    boolean[] taken=new boolean[p.getNumberOfVariables()];
	    boolean success=true;
	    for(int i=0;i<pr.getNumberOfLocalVariableSets();i++){
		if (taken[history[i]]){
		    success=false;
		}
		else {
		    taken[history[i]]=true;
		}
	    }
	    if (success && dowrite) { // If no duplicates
		out.write("C     -------- \n");
		out.write("      rlisttype("+(v.getId()+1)+","+(countComb)+")="+pr.getReactionType().getId()+"\n");
		for(int i=0;i<pr.getNumberOfLocalVariableSets();i++){
		    out.write("      rlistv("+(v.getId()+1)+","+(countComb)+","+(i+1)+")="+(history[i]+1)+"\n");
		}    
		for(int i=0;i<pr.getNumberOfLocalRanges();i++){
		    out.write("      rlistk("+(v.getId()+1)+","+(countComb)+","+(i+1)+",1)="+
					pr.getLocalRange(i).getLowerBound()+"\n");
		    out.write("      rlistk("+(v.getId()+1)+","+(countComb)+","+(i+1)+",2)="+
			      pr.getLocalRange(i).getUpperBound()+"\n");
		}
		countComb++;
		//System.out.print(" ");
	    }
	    else if (success) countComb++;
	    
	}
	else{
	    VariableSet vs=pr.getLocalVariableSet(level);
	    for (int i=0;i<vs.getNumberOfMembers();i++){
		int[] history2=new int[history.length];
		for (int j=0;j<level;j++) history2[j]=history[j];
		history2[level]=vs.getMember(i).getId();
		comb(history2,level+1,p,v,pr,out,dowrite);
	    }
	}
    }
    
    
    /**
     * Make readProblem.f in directory for a certain problem
     * @param directory the directory
     * @param p the problem
     */
    public static void generate(String directory,Problem p) {
	
	Debug.stat("GenerateFortran_readProblem","generate","code_rk_readProblem.f");
	File file = new File(directory + 
			     "readProblem.f");
	try {
	    FileWriter fstream = new FileWriter(file);
	    BufferedWriter out = new BufferedWriter(fstream);
	    
	    out.write("C     This file was automatically generated by GenerateFortran_readProblem.generate()\n");
	    out.write("C     on "+(new java.util.Date())+"\n");
	    out.write("C\n");
	    out.write("C     Read a problem from st input\n");
     	    out.write("C     Return name of problem\n");
     	    out.write("      subroutine readProblem(pname,debug)\n");
     	    out.write("      implicit none\n");
      	    GenerateFortran.printMaxDefinition(out,p);
      	    boolean include_modkinit=true;
      	    GenerateFortran.printModelDefinition(out,include_modkinit);

	    boolean include_edata2=true;
	    boolean include_spline=true;
	    boolean include_useip=true;
	    GenerateFortran.printExperimentDefinition(out,include_edata2,include_spline,include_useip);

	    out.write("       integer nvar,nexp,nvare,npts(maxexp),npts2\n");


	    // How many possible reactions are there?
	    boolean dowrite=false;
	    int maxirl=0;
	    for (int i=0;i<p.getNumberOfVariables();i++){
		countComb=1;
		for (int jpr=0;jpr<p.getVariable(i).getNumberOfPossibleReactions();jpr++){
		    PossibleReaction pre=p.getVariable(i).getPossibleReaction(jpr);
		    int[] history=new int[pre.getNumberOfLocalVariableSets()];
		    comb(history,0,p,p.getVariable(i),pre,out,dowrite);
		}
		if (countComb-1>maxirl) maxirl=countComb-1;
	    }



	    out.write("\n");
	    out.write("      integer debug,iexp,j\n");
	    out.write("      logical perfectData(maxexp)\n");
	    out.write("\n");
	    out.write("      integer maxirl            ! max rows in rlist\n");
	    out.write("      integer irl(maxvar)       ! no of active rows in rlist\n");
	    // out.write("      parameter (maxirl="+maxirl+");\n"); TOO BE ACTIVATED!!
	    out.write("      parameter (maxirl=1000);\n");
	    out.write("      integer rlisttype(maxvar,maxirl) ! Table of possible reaction types to add\n");
	    out.write("      integer rlistv(maxvar,maxirl,maxv) ! Table of variables associated to each reaction\n");
	    out.write("      double precision rlistk(maxvar,maxirl,maxk,2) ! Table of parameter bounds associated to each reaction\n");
	    out.write("\n");
	    out.write("      character*30 pname\n");
	    out.write("\n");
	    out.write("      double precision stdev(maxexp,maxvar,maxpts), lambda\n");
	    out.write("\n");
	    out.write("      integer SEED\n");
	    out.write("      double precision s ! smoothing parameter\n");
	    out.write("      integer RAND(2) ! Random starts in estvar\n");
	    out.write("\n");
	    out.write("      common/mydata/edata,spline,npts,nexp,nvar,nvare\n");
	    out.write("      common/mydata2/edata2,npts2,useip\n");
	    out.write("      common/mydata3/modk,modkinit,modn,modv,modtype,dependent\n");
	    out.write("      common/errsettings/stdev,lambda\n");
	    out.write("      common/mymodelspace/rlistk,rlistv,rlisttype,irl\n");
	    out.write("      common/mysmoothingpara/s\n");
	    out.write("      common/myrandomstarts/RAND\n");
	    out.write("      common/myrandomseed/SEED\n");

	    out.write("      pname='"+p.getName()+"'\n");
	    out.write("      lambda="+p.getErrorFunction().getLambda()+"\n");
	    out.write("\n");
	    out.write("C     SETTINGS\n");
	    out.write("      SEED="+p.getSEED()+"\n");
	    out.write("      s="+GenerateFortran.fixExponent(p.getSmoothingParameter())+"\n");
	    out.write("      RAND(1)="+p.getRAND1()+"\n");
	    out.write("      RAND(2)="+p.getRAND2()+"\n");

	    out.write("C     Variables\n");
	    out.write("      nvar="+p.getNumberOfVariables()+"\n");
	    out.write("      nvare="+p.getNumberOfVariables()+"\n");
	    for (int i=0;i<p.getNumberOfVariables();i++){
		if (p.getVariable(i).isDependent()) {
		    out.write("      dependent("+(i+1)+")=.true.\n");
		}
		else {
		    out.write("      dependent("+(i+1)+")=.false.\n");
		}
	    }
	    dowrite=true;
	    for (int i=0;i<p.getNumberOfVariables();i++){
		countComb=1;
		out.write("\n");
		out.write("C     Possible reactions for variable "+(i+1)+"\n");
		for (int jpr=0;jpr<p.getVariable(i).getNumberOfPossibleReactions();jpr++){
		    PossibleReaction pre=p.getVariable(i).getPossibleReaction(jpr);
		    //Debug.stat("GenerateFortran_readProblem","generate","Consider: "+pre);
		    int[] history=new int[pre.getNumberOfLocalVariableSets()];
		    comb(history,0,p,p.getVariable(i),pre,out,dowrite);
		}
		out.write("\n");
		out.write("C     Number of reactions for variable "+(i+1)+"\n");
		out.write("      irl("+(i+1)+")="+(countComb-1)+"\n");
		out.write("\n");
	    }
	    
	    
	    out.write("\n");
	    out.write("      nexp="+p.getNumberOfExperiments()+"\n");
	    for (int i=0;i<p.getNumberOfExperiments();i++){
		out.write("\n");
		out.write("C     Experiment "+(i+1)+"\n");
		out.write("      npts("+(i+1)+")="+p.getExperiment(i).getNumberOfSamples()+"\n"); 
		if (!p.getExperiment(i).hasPerfectData()) {
		    out.write("      perfectData("+(i+1)+")=.false.\n");
		}
		else {
		    out.write("      perfectData("+(i+1)+")=.true.\n");
		}
		out.write("C     Time-points\n");
		for (int j=0;j<p.getExperiment(i).getNumberOfSamples();j++){
		    out.write("      edata("+(i+1)+",1,"+(j+1)+")="+GenerateFortran.fixExponent(p.getExperiment(i).getSample(j).getTime())+"\n");
		}
		for (int ivar=0;ivar<p.getNumberOfVariables();ivar++){
		    out.write("C     Variable "+(ivar+1)+"\n");
		    if (!p.getExperiment(i).hasPerfectData()) {
			out.write("      modkinit("+(i+1)+","+(ivar+1)+",2)="+
				  GenerateFortran.fixExponent(p.getExperiment(i).getLowerInitialBound(ivar))+"\n");
			out.write("      modkinit("+(i+1)+","+(ivar+1)+",3)="+
				  GenerateFortran.fixExponent(p.getExperiment(i).getUpperInitialBound(ivar))+"\n");
		    }
		    for (int j=0;j<p.getExperiment(i).getNumberOfSamples();j++){
			out.write("      edata("+(i+1)+","+(ivar+2)+","+(j+1)+")="+
				  GenerateFortran.fixExponent(p.getExperiment(i).getSample(j).getValue(ivar))+"\n");
			out.write("      stdev("+(i+1)+","+(ivar+1)+","+(j+1)+")="+
				  GenerateFortran.fixExponent(p.getExperiment(i).getSample(j).getSdev(ivar))+"\n");
		    }
		}
	    }
	    	 


	    // Initial model
	    out.write("C     Initial model\n");
	    for (int i=0;i<p.getNumberOfVariables();i++){
		out.write("C     Variable "+(i+1)+"\n");
		out.write("      modn("+(i+1)+")=0\n");
		for (int jr=0;jr<p.getVariable(i).getNumberOfReactions();jr++){
		    Reaction re=p.getVariable(i).getReaction(jr);
		    out.write("      modtype("+(i+1)+","+(jr+1)+")="+re.getReactionType().getId()+"\n");
		    out.write("      modn("+(i+1)+")=modn("+(i+1)+")+1\n");
		    for (int jloc=0;jloc<re.getNumberOfLocalVariables();jloc++){
			out.write("      modv("+(i+1)+","+(jr+1)+","+
				  (jloc+1)+")="+(re.getLocalVariable(jloc).getId()+1)+"\n");
		    }
		    for (int jra=0;jra<re.getNumberOfLocalRanges();jra++){
			if (re.getLocalRange(jra).getLowerBound()>=0.0) {
			    out.write("      modk("+(i+1)+","+(jr+1)+","+
				      (jra+1)+",1)="+
				      GenerateFortran.fixExponent(re.getLocalRange(jra).getInitialGuess())+"\n");
			}
			else {
			    out.write("      modk("+(i+1)+","+(jr+1)+","+
				      (jra+1)+",1)="+
				      GenerateFortran.fixExponent(re.getLocalRange(jra).getInitialGuess())+"\n");
			}
			out.write("      modk("+(i+1)+","+(jr+1)+","+
				  (jra+1)+",2)="+
				  GenerateFortran.fixExponent(re.getLocalRange(jra).getLowerBound())+"\n");
			out.write("      modk("+(i+1)+","+(jr+1)+","+
				  (jra+1)+",3)="+
				  GenerateFortran.fixExponent(re.getLocalRange(jra).getUpperBound())+"\n");
		    }
		    out.write("\n");
		    
		}
	    }
		    
	    out.write("\n");
	    out.write("      do 200 iexp=1,nexp\n");
	    out.write("         do 100 j=1,nvar\n");
	    out.write("            modkinit(iexp,j,1)=edata(iexp,j+1,1)\n");
	    out.write("            if (perfectData(iexp)) then\n");
	    out.write("               modkinit(iexp,j,2)=edata(iexp,j+1,1)\n");
	    out.write("               modkinit(iexp,j,3)=edata(iexp,j+1,1)\n");
	    out.write("            endif\n");
	    out.write(" 100     enddo\n");
	    out.write(" 200  enddo   \n");
	    out.write("      end \n");
	    out.close();
	}catch (Exception e){
	    Debug.error("GenerateFortran_readProblem","generate",e.getMessage());
	    System.exit(0);
	}
    }
    


    

}
