/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/


package Java.generateFortran;

import java.io.*;
import Java.debug.*;
import Java.problem.*;

/**
 * Generate spline.f
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class GenerateFortran_spline{


/**
 * Make spline.f in directory for a certain problem
 * @param directory the directory
 * @param p the problem
 */
    public static void generate(String directory,Problem p) {
	File file = new File(directory + 
			     "spline.f");


	try{
	   // Create file 
	    FileWriter fstream = new FileWriter(file);
	    BufferedWriter out = new BufferedWriter(fstream);

	    
	    out.write("C     This file was automatically generated by GenerateFortran_spline.generate()\n");
	    out.write("C     on "+(new java.util.Date())+"\n");
	    out.write("C\n");

	    // getNumberOfReactionTypes
	    Debug.stat("GenerateFortran_myReactions","generate","spline.f, getNumberOfReactionTypes");
	    out.write("\n"); 
	    out.write("C     Get spline data\n");
	    GenerateFortran.prettyPrint(out,"double precision function ppvalumain(ivar,iexp,time,jderiv,npts)",0);
	    GenerateFortran.prettyPrint(out,"implicit none",0);
	    GenerateFortran.prettyPrint(out,"integer ivar,iexp,jderiv,npts",0);
	    GenerateFortran.prettyPrint(out,"double precision time,ppvalu",0);
	    GenerateFortran.prettyPrint(out,"double precision f(4,"+(p.get_maxpts()-1)+")",0);
	    GenerateFortran.prettyPrint(out,"double precision x("+p.get_maxpts()+")",0);
		  
	    boolean first=true;
	    for (int i=0;i<p.getNumberOfExperiments();i++) {
		Experiment e=p.getExperiment(i);
		double[] t=e.getTimeArray();
		if (first) {
		    GenerateFortran.prettyPrint(out,"if (iexp.eq."+(i+1)+") then",0);
		    first=false;
		}
		else GenerateFortran.prettyPrint(out,"else if (iexp.eq."+(i+1)+") then",0);
		for (int val=0; val<t.length; val++) {
		    GenerateFortran.prettyPrint(out,"x("+(val+1)+")="+t[val],1);
		}
	    }
	    if (!first) GenerateFortran.prettyPrint(out,"endif",0);
	

	    first=true;
	    for (int i=0;i<p.getNumberOfExperiments();i++) {
		Experiment e=p.getExperiment(i);
		double[] t=e.getTimeArray();
		for (int j=0;j<p.getNumberOfVariables();j++){
		    
		    if (p.getVariable(j).isDependent()) {
			double[] values = e.getValueArray(j);

			double[][] coef=Pppack.getCoef(t,values,e.getSdevArray(j),t.length,p.getSmoothingParameter()*t.length);
			GenerateFortran.prettyPrint(out,"! Smoothing parameter=s*npts="+p.getSmoothingParameter()*t.length,0);
			if (first) {
			    GenerateFortran.prettyPrint(out,"if (iexp.eq."+(i+1)+".and.ivar.eq."+(j+1)+") then",0);
			    first=false;
			}
			else GenerateFortran.prettyPrint(out,"else if (iexp.eq."+(i+1)+".and.ivar.eq."+(j+1)+") then",0);
			for (int index=0; index<4; index++) {
			    for (int val=0; val<t.length-1; val++) {
				GenerateFortran.prettyPrint(out,"f("+(index+1)+","+(val+1)+")="+coef[index][val],1);
			    }
			}
			
		    }
		}
	    }
	    if (!first) GenerateFortran.prettyPrint(out,"endif",0);

	    
	    GenerateFortran.prettyPrint(out,"ppvalumain=ppvalu(x,f,npts-1,4,time,jderiv)",0);
	    GenerateFortran.prettyPrint(out,"return",0);
	    GenerateFortran.prettyPrint(out,"end",0);
    
	  	    
	    
	    //Close the output stream
	    out.close();
	
	}catch (Exception e){
	    System.err.println("Error: " + e.getMessage());
	}
	
    }
	

}