/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    The code in this class is a Java implementation of the program Pppack published in C. de Boor, A Practical Guide to Splines, Springer-Verlag,  1978

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/


package Java.generateFortran;

import java.io.*;

/**
 * The code in this class is a Java implementation of the program Pppack published in C. de Boor, A Practical Guide to Splines, Springer-Verlag,  1978
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class Pppack{

    
    public static void chol1d(double p, double[][] v, double[] qty, int npoint, 
		       int ncol, double[]  u, double[]  qu) {
	// subroutine chol1d ( p, v, qty, npoint, ncol, u, qu )
	
	//  from  * a practical guide to splines *  by c. de boor    
	//  from  * a practical guide to splines *  by c. de boor    
	//  to be called in  s m o o t h
	//  constructs the upper three diags. in v(i,j), i=2,,npoint-1, j=1,3, of
	// the matrix  6*(1-p)*q-transp.*(d**2)*q + p*r, then computes its
	// l*l-transp. decomposition and stores it also in v, then applies
	// forward and backsubstitution to the right side q-transp.*y in  qty
	// to obtain the solution in  u .
	
	int npm1,npm2;   
	double prev,ratio,six1mp,twop;
	npm1 = npoint - 1;
	six1mp = 6.0*(1.0-p);
	twop = 2.0*p;
	for (int i=1;i<npm1;i++) {
	    v[i][0] = six1mp*v[i][4] + twop*(v[i-1][3]+v[i][3]);
	    v[i][1] = six1mp*v[i][5] + p*v[i][3];
	    v[i][2] = six1mp*v[i][6];
	}


	npm2 = npoint - 2;
	if (npm2>=2){
	    for (int i=1;i<npm2;i++) {
		ratio = v[i][1]/v[i][0];
		v[i+1][0] = v[i+1][0] - ratio*v[i][1];
		v[i+1][1] = v[i+1][1] - ratio*v[i][2];
		v[i][1] = ratio;
		ratio = v[i][2]/v[i][0];
		v[i+2][0] = v[i+2][0] - ratio*v[i][2];
		v[i][2] = ratio;
	    }
	    u[0] = 0.0;
	    v[0][2] = 0.0;
	    u[1] = qty[1];
     	    for (int i=1;i<npm2;i++) {
		u[i+1] = qty[i+1] - v[i][1]*u[i] - v[i-1][2]*u[i-1];
	    }
	    u[npoint-1] = 0.0;
	    u[npm1-1] = u[npm1-1]/v[npm1-1][0];
	    int i = npm2-1;
	    for (;;) { 
		u[i] = u[i]/v[i][0]-u[i+1]*v[i][1]-u[i+2]*v[i][2];
		i = i - 1;
		if (i>0) { 
		}
		else { 
		    break;
		}
	    }

	}
	else {      
	    u[0] = 0.0;
	    u[1] = qty[1]/v[1][0];
	    u[2] = 0.0;
	}

	prev = 0.0;
	for (int i=1;i<npoint;i++) {
	    qu[i] = (u[i] - u[i-1])/v[i-1][3];
	    qu[i-1] = qu[i] - prev;
	    prev = qu[i];
	}
	qu[npoint-1] = -qu[npoint-1]; 
    }



    public static void setupq(double[] x,double[] dx, double[] y,
		       int npoint,double[][] v, double[] qty) {
				 
	//subroutine setupq ( x, dx, y, npoint, v, qty )
	// from  * a practical guide to splines *  by c. de boor    
	// to be called in  s m o o t h
	// put  delx = x(.+1) - x(.)  into  v(.,4),
	// put  the three bands of  q-transp*d  into  v(.,1-3), and
	// put the three bands of  (d*q)-transp*(d*q)  at and above the diagonal
	//    into  v(.,5-7) .
	//    here,  q is  the tridiagonal matrix of order (npoint-2,npoint)
	// with general row  1/delx(i) , -1/delx(i) - 1/delx(i+1) , 1/delx(i+1)
	// and   d  is the diagonal matrix  with general row  dx(i) .
	  
	int npm1;
	double diff,prev;
	npm1 = npoint - 1;
	v[0][3]=x[1]-x[0];

	for (int i=1;i<npm1;i++) {
	    v[i][3] = x[i+1] - x[i];
	    v[i][0] = dx[i-1]/v[i-1][3];
	    v[i][1] = - dx[i]/v[i][3] - dx[i]/v[i-1][3];
	    v[i][2] = dx[i+1]/v[i][3];
	}
    
	v[npoint-1][0] = 0.0;
    

	for (int i=1;i<npm1;i++) 
	    v[i][4] = Math.pow(v[i][0],2) + Math.pow(v[i][1],2) + Math.pow(v[i][2],2);


	if (npm1<3) {
	}
	else {
	    for (int i=2;i<npm1;i++) 
		v[i-1][5] = v[i-1][1]*v[i][0] + v[i-1][2]*v[i][1];
	}
	v[npm1-1][5] = 0.0; 
	if (npm1<4) {
	}
	else {
	    for (int i=3;i<npm1;i++) 
		v[i-2][6] = v[i-2][2]*v[i][0];
	}
	v[npm1-2][6] = 0.0;
	v[npm1-1][6] = 0.0;
	prev = (y[1] - y[0])/v[0][3];
	for (int i=1;i<npm1;i++) {
	    diff = (y[i+1]-y[i])/v[i][3];
	    qty[i] = diff - prev;
	    prev = diff;
	}
    }




    public static double smooth (double[] x, double[] y, double[] dy, 
				 int npoint, double s, double[][] v, 
				 double[][] a ) {
	// from  * a practical guide to splines *  by c. de boor    
	// calls  setupq, chol1d
	
	// constructs the cubic smoothing spline  f  to given data  (x(i),y(i)),
	// i=1,...,npoint, which has as small a second derivative as possible
	// while
	// s(f) = sum( ((y(i)-f(x(i)))/dy(i))**2 , i=1,...,npoint ) .le. s .
	//
	//******  i n p u t  ******
	// x(1),...,x(npoint)   data abscissae,  a s s u m e d  to be strictly
	//       increasing .
	// y(1),...,y(npoint)     corresponding data ordinates .
	// dy(1),...,dy(npoint)     estimate of uncertainty in data,  a s s u m-
	//       e d  to be positive .
	// npoint.....number of data points,  a s s u m e d  .gt. 1
	// s.....upper bound on the discrete weighted mean square distance of
	//       the approximation  f  from the data .
	//
	//******  w o r k  a r r a y s  *****
	// v.....of size (npoint,7)
	// a.....of size (npoint,4)
	//
	//*****  o u t p u t  *****
	// a(.,1).....contains the sequence of smoothed ordinates .
	// a(i,j) = f^(j-1)(x(i)), j=2,3,4, i=1,...,npoint-1 ,  i.e., the
	//       first three derivatives of the smoothing spline  f  at the
	//       left end of each of the data intervals .
	//    w a r n i n g . . .   a  would have to be transposed before it
	//       could be used in  ppvalu .
	//
	//******  m e t h o d  ******
	//    The matrices  Q-transp*d  and  Q-transp*D**2*Q  are constructed in
	//  s e t u p q  from  x  and  dy , as is the vector  qty = Q-transp*y .
	// Then, for given  p , the vector  u  is determined in  c h o l 1 d  as
	// the solution of the linear system
	//              (6(1-p)Q-transp*D**2*Q + p*Q)u  = qty  .
	// From  u , the smoothing spline  f  (for this choice of smoothing par-
	// ameter  p ) is obtained in the sense that
	//                       f(x(.))  =  y - 6(1-p)D**2*Q*u        and
	//                     f''(x(.))  =  6*p*u                      .
	//    The smoothing parameter  p  is found (if possible) so that
	//               sf(p)  =  s ,
	// with  sf(p) = s(f) , where  f  is the smoothing spline as it depends
	// on  p .  if  s = 0, then p = 1 . if  sf(0) .le. s , then p = 0 .
	// Otherwise, the secant method is used to locate an appropriate  p  in
	// the open interval  (0,1) . However, straightforward application of
	// the secant method, as done in the original version of this program,
	// can be very slow and is influenced by the units in which  x  and  y 
	// are measured, as C. Reinsch has pointed out. Instead, on recommend-
	// ation from C. Reinsch, the secant method is applied to the function
	//          g:q |--> 1/sqrt{sfq(q)} - 1/sqrt{s} ,
	// with  sfq(q) := sf(q/(1+q)), since  1/sqrt{sfq}  is monotone increasing
	// and close to linear for larger  q . One starts at  q = 0  with a
	// Newton step, i.e., 
	//               q_0 = 0,  q_1 = -g(0)/g'(0)
	// with  g'(0) = -(1/2) sfq(0)^{-3/2} dsfq, where dsfq = -12*u-transp*r*u ,
	// and  u  as obtained for  p = 0 . Iteration terminates as soon as 
	//  abs(sf - s) .le. .01*s .
	//
	//    logical test 
	//    parameter (test = .true.)
	//    int itercnt
	
	int npm1;
	double change,ooss,oosf,p,prevsf,prevq,q,sfq,sixp,six1mp,utru,smoothreturn;
	
	q=0.0; 


	double[] tem0=new double[npoint];
	double[] tem2=new double[npoint];
	double[] tem3=new double[npoint];
	
	
	double[] tem=new double[npoint];
	for (int j=0;j<npoint;j++) tem[j]=a[j][3];
	setupq(x,dy,y,npoint,v,tem);
	for (int j=0;j<npoint;j++) a[j][3]=tem[j];

	
	if (s>0.0){ 
	    p = 0.0;
	    for (int j=0;j<npoint;j++) tem0[j]=a[j][0];
	    for (int j=0;j<npoint;j++) tem2[j]=a[j][2];
	    for (int j=0;j<npoint;j++) tem3[j]=a[j][3];
	    chol1d(p,v,tem3,npoint,1,tem2,tem0);
	    for (int j=0;j<npoint;j++) a[j][0]=tem0[j];
	    for (int j=0;j<npoint;j++) a[j][2]=tem2[j];
	    for (int j=0;j<npoint;j++) a[j][3]=tem3[j];
	    
	    sfq = 0.0;
	    for (int i=0;i<npoint;i++)
		sfq = sfq + Math.pow((a[i][0]*dy[i]),2);
	    
	    sfq = sfq*36.0;
	    if (sfq<=s) {
	
	    }
	    else {
		utru = 0.0;
		for (int i=1;i<npoint;i++)
		    utru = utru + v[i-1][3]*(a[i-1][2]*(a[i-1][2]+a[i][2])+
					     Math.pow(a[i][2],2));
		
		ooss = 1.0/Math.sqrt(s);
		oosf = 1.0/Math.sqrt(sfq);
		q = -(oosf-ooss)*sfq/(6.0*utru*oosf);
		// secant iteration for the determination of p starts here.
		prevq = 0.0;
		prevsf = oosf;
		for (;;){
		    for (int j=0;j<npoint;j++) tem0[j]=a[j][0];
		    for (int j=0;j<npoint;j++) tem2[j]=a[j][2];
		    for (int j=0;j<npoint;j++) tem3[j]=a[j][3];
		    chol1d(q/(1.0+q),v,tem3,npoint,1,tem2,tem0);
		    for (int j=0;j<npoint;j++) a[j][0]=tem0[j];
		    for (int j=0;j<npoint;j++) a[j][2]=tem2[j];
		    for (int j=0;j<npoint;j++) a[j][3]=tem3[j];
		    
		    sfq = 0.0;	
		    for (int i=0;i<npoint;i++)
			sfq = sfq + Math.pow((a[i][0]*dy[i]),2);
		    sfq = sfq*36.0/Math.pow((1.0+q),2);
				
		    if (Math.abs(sfq-s)<=0.01*s) {
			break;
		    }
		    oosf = 1.0/Math.sqrt(sfq);
		    change = (q-prevq)/(oosf-prevsf)*(oosf-ooss);
		    prevq = q;
		    q = q - change;
		    prevsf = oosf;
		}
		p = q/(1.0+q);
		// correct value of p has been found.
		// compute pol.coefficients from  Q*u (in a(.,1)).
	    }
	}
	else {
	    p = 1.0;
	    for (int j=0;j<npoint;j++) tem0[j]=a[j][0];
	    for (int j=0;j<npoint;j++) tem2[j]=a[j][2];
	    for (int j=0;j<npoint;j++) tem3[j]=a[j][3];
	    chol1d(p,v,tem3,npoint,1,tem2,tem0);
	    for (int j=0;j<npoint;j++) a[j][0]=tem0[j];
	    for (int j=0;j<npoint;j++) a[j][2]=tem2[j];
	    for (int j=0;j<npoint;j++) a[j][3]=tem3[j];
	    
	    sfq = 0.0;
	}
	
	smoothreturn = sfq;
	six1mp = 6.0/(1.0+q);

	for (int i=0;i<npoint;i++) 
	    a[i][0] = y[i] - six1mp*Math.pow(dy[i],2)*a[i][0];

	sixp = 6.0*p;

	for (int i=0;i<npoint;i++) 
	    a[i][2] = a[i][2]*sixp;
	
	npm1 = npoint - 1;
	for (int i=0;i<npm1;i++) {
	    a[i][3] = (a[i+1][2]-a[i][2])/v[i][3];
	    a[i][1] = (a[i+1][0]-a[i][0])/v[i][3]- (a[i][2]+a[i][3]/3.0*v[i][3])/2.0*v[i][3];
	}
	return smoothreturn;
    }

    // public static void printA(double[][] a){
    // 	for (int i=0;i<a.length;i++){
    // 	    System.out.println("a("+i+",:)=");
    // 	    for (int j=0;j<4;j++) System.out.println(a[i][j] +" ");
    // 	    System.out.println();
    // 	}
    // }
    // public static void printV(double[][] a){
    // 	for (int i=0;i<a.length;i++){
    // 	    System.out.println("v("+i+",:)=");
    // 	    for (int j=0;j<7;j++) System.out.println(a[i][j] +" ");
    // 	    System.out.println();
    // 	}
    // }

  
    public static double[][] getCoef(double[] x, double[] y, double[] dy, int npoint, double s) {
	double[][] v=new double[npoint][7];
	double[][] a=new double[npoint][4];
	double[][] coef=new double[4][npoint];
	double ss=smooth (x,y,dy,npoint,s,v,a);
	int lsm = npoint - 1;
	for (int i=0;i<lsm;i++){
	    for (int j=0;j<4;j++){
		coef[j][i] = a[i][j];
	    }
	}
	return coef;
    }


    // /** Test program */
    // public static void main(String[] argv) {

    // 	double[] t=new double[16];
    // 	double[] y=new double[16];
    // 	double[] dy=new double[16];
    // 	double[][] a=new double[16][4];
    // 	double[][] coef=new double[4][16];
    // 	double[][] v=new double[16][7];
    // 	int i=0;
    // 	//samle_1 of experiment_1
    // 	t[i]=  0.00000;
    // 	y[i]= 0.7000000000000;
    // 	dy[i]= 0.01;
 
    // 	//samle_2 of experiment_1
    // 	i++;
    // 	t[i]=  0.010000000;
    // 	y[i]= 0.7740854288738029;
    // 	dy[i]= 0.01;
 
    // 	//samle_3 of experiment_1
    // 	i++;
    // 	t[i]=  0.020000000;
    // 	y[i]= 0.9176227459530631;
    // 	dy[i]= 0.01;
 
    // 	//samle_4 of experiment_1
    // 	i++;
    // 	t[i]=  0.030000000;
    // 	y[i]= 1.038420604594351;
    // 	dy[i]= 0.01;
 
    // 	//samle_5 of experiment_1
    // 	i++;
    // 	t[i]=  0.040000000;
    // 	y[i]=1.102231122300274;
    // 	dy[i]= 0.01;
 
    // 	//samle_6 of experiment_1
    // 	i++;
    // 	t[i]=  0.050000000;
    // 	y[i]= 1.116768952070325;
    // 	dy[i]= 0.01;
 
    // 	//samle_7 of experiment_1
    // 	i++;
    // 	t[i]=  0.060000000;
    // 	y[i]= 1.101765107371942;
    // 	dy[i]= 0.01;
 
    // 	//samle_8 of experiment_1
    // 	i++;
    // 	t[i]=  0.070000000;
    // 	y[i]= 1.072251252873835;
    // 	dy[i]= 0.01;
 
    // 	//samle_9 of experiment_1
    // 	i++;
    // 	t[i]=  0.080000000;
    // 	y[i]= 1.037151709789823;
    // 	dy[i]= 0.01;
 
    // 	//samle_10 of experiment_1
    // 	i++;
    // 	t[i]=  0.090000000;
    // 	y[i]= 1.001187549478120;
    // 	dy[i]= 0.01;
 
    // 	//samle_11 of experiment_1
    // 	i++;
    // 	t[i]=  0.10000;
    // 	y[i]= 0.9667741417113107;
    // 	dy[i]= 0.01;
 
    // 	//samle_12 of experiment_1
    // 	i++;
    // 	t[i]=  0.15000;
    // 	y[i]= 0.8393664994200825;
    // 	dy[i]= 0.01;
 
    // 	//samle_13 of experiment_1
    // 	i++;
    // 	t[i]=  0.20000;
    // 	y[i]= 0.7765171155409907;
    // 	dy[i]= 0.01;
 
    // 	//samle_14 of experiment_1
    // 	i++;
    // 	t[i]=  0.25000;
    // 	y[i]= 0.7499212257351757;
    // 	dy[i]= 0.01;
 
    // 	//samle_15 of experiment_1
    // 	i++;
    // 	t[i]=  0.30000;
    // 	y[i]= 0.7399013998170061;
    // 	dy[i]= 0.01;
 
    // 	//samle_16 of experiment_1
    // 	i++;
    // 	t[i]=  0.50000;             
    // 	y[i]=   0.7342866583002997;       
    // 	dy[i]= 0.01;        
 
    // 	int npts=16;
    // 	double s=0.001;
    // 	double sfinal=s*npts;
	
    // 	for (i=0;i<npts;i++){
    // 	    System.out.println("t="+t[i]+" y="+y[i]+" dy="+dy[i]);
    // 	}

    // 	coef=getCoef(t,y,dy,npts,sfinal);
    // 	for (i=0;i<npts-1;i++){
    // 	    System.out.print("coef(:,"+i+")=");
    // 	    for (int j=0;j<4;j++) System.out.print(coef[j][i] +" ");
    // 	    System.out.println();
    // 	}

	

    // }

}
