/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/

package Java.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.*;
import Java.debug.*;
import Java.misc.*;
import Java.problem.*;


/** The panel for the graph
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class GraphPanel extends JPanel{


    Problem p;
    Experiment experiment;
    double[][] yp;
    double[] xp;
    int steps;

    int w,h;

    double max_y=0; //max y value to plot
    double min_y=0; //min y value to plot
    double max_x=0; //max x value to plot
    double min_x=0; //min x value to plot
    boolean default_axes=true;//if true we calculate axes ourselves

    private boolean plotNames=true;
    private boolean connectPoints=false;
    double localSmoothingParameter=-1.0;
    JCheckBox[] checkedVariables;
    Color[] colors;


    /** The plot
     * @param p the problem
     * @param experiment the experiment
     * @param checkedVariables which variables to plot
     * @param colors color of each variable
     */
    public GraphPanel(Problem p,
		      Experiment experiment,
                      JCheckBox[] checkedVariables,
                      Color[] colors) {
	this.p=p;
	this.experiment=experiment;
        this.checkedVariables=checkedVariables;
	this.colors=colors;
	yp=null;
	xp=null;
	steps=0;
	addMouseListener(new MouseL());
	addMouseMotionListener(new MouseML());
	//setMinimumSize(new Dimension(500,400));
	//setPreferredSize(new Dimension(700,600));
	setBackground(Color.white);
	setOpaque(true);
	plotNames=false;
	connectPoints=false;
	setToolTipText("Double-click the plot for setting axes.");
    }



    class MouseL extends MouseAdapter {

	public void mousePressed(MouseEvent e) {
	    
	}
	
	public void mouseClicked(MouseEvent e) {
	    if (e.getClickCount()==2 && e.isControlDown()){ 

	    }
	    else if (e.getClickCount()==2){
		SetAxesDialog sad=new SetAxesDialog();
		sad.xmin.setText((new Double(Misc.Round(min_x,2))).toString());
		sad.xmax.setText((new Double(Misc.Round(max_x,2))).toString());
		sad.ymin.setText((new Double(Misc.Round(min_y,2))).toString());
		sad.ymax.setText((new Double(Misc.Round(max_y,2))).toString());
		if (default_axes){//uses default axes
		    sad.default_axes.setSelected(true);
		    sad.xmin.setEnabled(false);
		    sad.xmax.setEnabled(false);
		    sad.ymin.setEnabled(false);
		    sad.ymax.setEnabled(false);
		}	
		else{
		    sad.default_axes.setSelected(false);	  
		    sad.xmin.setEnabled(true);
		    sad.xmax.setEnabled(true);
		    sad.ymin.setEnabled(true);
		    sad.ymax.setEnabled(true);
		}
	    
	    
		String errorMessage="";
		int res;
		for (;;) {
		    res = JOptionPane.showOptionDialog(null, sad.panel,
						       "Set axes",
						       JOptionPane.OK_CANCEL_OPTION,
						       JOptionPane.QUESTION_MESSAGE,
						       null, null, null);
		    /* Om anvaendaren avbroet, sluta dialogen haer. */
		    if (res == JOptionPane.OK_OPTION) {
			errorMessage="";
			//control of values
			Double Xmin=new Double(0);
			Double Xmax=new Double(0);
			Double Ymin=new Double(0);
			Double Ymax=new Double(0);
			try{
			    Xmin=new Double(sad.xmin.getText());
			} catch (NumberFormatException nfe){
			    errorMessage+="X min is not a real number.\n";
			}
			try{
			    Xmax=new Double(sad.xmax.getText());
			} catch (NumberFormatException nfe){
			    errorMessage+="X max is not a real number.\n";
			}
			try{
			    Ymin=new Double(sad.ymin.getText());
			} catch (NumberFormatException nfe){
			    errorMessage+="Y min is not a real number.\n";
			}
			try{
			    Ymax=new Double(sad.ymax.getText());
			} catch (NumberFormatException nfe){
			    errorMessage+="Y max is not a real number.\n";
			}
			if (errorMessage.equals("") && 
			    (Ymax.doubleValue()<Ymin.doubleValue() || Xmax.doubleValue()<Xmin.doubleValue()) ){
			    errorMessage+="X max is larger than X min or Y max is larger than Y min.\n";
			}
			if (errorMessage.equals("")) break;	    
			else  JOptionPane.showMessageDialog(null,
							    errorMessage,
							    "Error",
							    JOptionPane.ERROR_MESSAGE); 
		    }
		    else break;//cancel
		}	  
		if (res == JOptionPane.OK_OPTION) { //not cancel or error
		    //Get and set parameters
		    min_x=(new Double(sad.xmin.getText())).doubleValue();
		    max_x=(new Double(sad.xmax.getText())).doubleValue();
		    min_y=(new Double(sad.ymin.getText())).doubleValue();  
		    max_y=(new Double(sad.ymax.getText())).doubleValue();
		
	  
		    //set default axes
		    default_axes=sad.default_axes.isSelected();
		
		    repaint();
		}
		else {
		    //System.out.println("Cancel"); //cancel
		}
	    
	    }
	}
    }
    

    class MouseML extends MouseMotionAdapter {
	public void mouseDragged(MouseEvent e) {

	}
    }

    public void clear() {
	repaint();
    }
    
    public void setLocalSmoothingParameter(double s){
        localSmoothingParameter=s;
    }  
    
    public void setCheckedVariables(JCheckBox[] jcb){
        checkedVariables=jcb;
    }
    

    
    public void setMaxMin(){
	//Debug.print(this,"setMaxMin");
	max_y=0;
	min_y=9999999;
	max_x=0;
	min_x=9999999;
	double e_max_y=experiment.getMaxValue()*1.1;
	//double e_min_y=experiment.getMinValue()*0.9;
	double e_min_y=0.0;
	double e_max_x=experiment.getMaxTime();
	double e_min_x=experiment.getMinTime();
	if (e_max_y>max_y) max_y=e_max_y;
	if (e_min_y<min_y) min_y=e_min_y;
	if (e_max_x>max_x) max_x=e_max_x;//change x if necessary
	if (e_min_x<min_x) min_x=e_min_x;
	
	max_y=prettyPlotValue(max_y);
    }

    private double prettyPlotValue(double d){	
	int nDown=0;
	int nUp=0;
	if (d>=30) {
	    while (d>=30) {
		d=d/10.0;
		nDown++;
	    }
	}
	else if (d<3) {
	    while (d<3) {
		d=d*10.0;
		nUp++;
	    }
	    
	}
	int i=(int)d+1;
	double f=i*Math.pow(10.0,nDown)*Math.pow(10.0,-nUp);
	return f;
    }

    public void paintComponent(Graphics g) {
	super.paintComponent(g);
	Dimension d=getSize();
	
	if (default_axes) setMaxMin();
	
	w = d.width;
	h = d.height;
	g.setColor(Color.black);
	g.setFont(new Font("Dialog",Font.PLAIN,11));

	int DIST=70; // distance from border to graph
	int DIST2=DIST*2;

	/*Rita koordinatsystem*/
	g.drawLine(DIST,h-DIST,w-DIST,h-DIST);
	g.drawLine(DIST,DIST,DIST,h-DIST);
	//small lines
	for (int i=0; i<11; i++){
	    g.drawLine(DIST-3,DIST+i*(h-DIST2)/10,DIST+3,DIST+i*(h-DIST2)/10);
	}
	//small lines
	for (int i=0; i<11; i++){
	    g.drawLine(DIST+i*(w-DIST2)/10,h-DIST-3,DIST+i*(w-DIST2)/10,h-DIST+3);
	}
	g.setFont(new Font("Dialog",Font.BOLD,11));
	g.drawString("Time",w/2,h-DIST/2);
	g.drawString("Value",DIST/3,DIST/2);
	g.setFont(new Font("Dialog",Font.PLAIN,11));
	g.drawString((new Double(Misc.Round(max_y,3))).toString(),DIST/3,DIST+5);//draw max_y conc.
	g.drawString((new Double(Misc.Round(min_y,3))).toString(),DIST/3,h-DIST+5);//draw min_y conc.
	g.drawString((new Double(Misc.Round(max_x,3))).toString(),w-DIST-5,h-DIST+18);//draw max time
	g.drawString((new Double(Misc.Round(min_x,3))).toString(),DIST-5,h-DIST+18);//draw min time


	g.setFont(new Font("Dialog",Font.PLAIN,11));

	int x_old=0;//for drawing connecting lines
	int y_old=0;//for drawing connecting lines



    
	// PLOT EXPERIMENTAL DATA
	double[] tArray=experiment.getTimeArray();
	for (int i=0;i<p.getNumberOfVariables();i++){

	    if (p.getVariable(i).isDependent() && checkedVariables[i].isSelected()) {

		// Draw legend
		int SPACE=60;
		int LINELENGTH=10;
		int row=DIST/3+(i/10)*DIST/3;
		int col=2*DIST+(i%10)*SPACE;
		g.setColor(Color.black);
		g.setFont(new Font("Dialog",Font.PLAIN,10));
		g.drawString("Var "+(i+1),col+LINELENGTH+5,row);
		g.setColor(colors[i%colors.length]);
		g.drawLine(col,row-5,col+LINELENGTH,row-5);

		double[] vArray=experiment.getValueArray(i);
		for (int j=0;j<tArray.length;j++){
		    //Debug.print(this,"paintComponent","v("+j+")="+vArray[j]);
		    int x=DIST+(int)((w-DIST2)*(tArray[j]-min_x)/(max_x-min_x));
		    int y=h-DIST-(int)((h-DIST2)*(vArray[j]-min_y)/(max_y-min_y));
		    if (tArray[j]>=min_x && tArray[j]<=max_x && vArray[j]>=min_y && vArray[j]<=max_y) {
			g.drawString("o",x-2,y+3);//compensate
			// if we want to connect the points or if a linear interpolation is wanted 
			// these two options are in this case equal
			if (j!=0 && connectPoints) 
			    g.drawLine(x_old,y_old,x,y);
		    }
		    x_old=x;
		    y_old=y;
		    if (j==tArray.length-1 && plotNames) 
			g.drawString(experiment.toString(),x-200,y-5);//name of experiment
		}
    
		// PLOT INTERPOLATION
		double[] values = experiment.getValueArray(i);
		Pppack_smooth pp=new Pppack_smooth(experiment.getTimeArray(),values,experiment.getSdevArray(i));
		if (localSmoothingParameter<0.0) localSmoothingParameter=p.getSmoothingParameter();
		pp.calcSpline(localSmoothingParameter);
		double minTime=experiment.getMinTime();
		double maxTime=experiment.getMaxTime();
		for (int j=0;j<101;j++){ //always plot 101 data points
		    double t=minTime+j*(maxTime-minTime)/100;
		    double v=pp.get(t,0);
		    int x=DIST+(int)((w-DIST2)*(t-min_x)/(max_x-min_x));
		    int y=h-DIST-(int)((h-DIST2)*(v-min_y)/(max_y-min_y));
		    if (t>=min_x && t<=max_x && v>=min_y && v<=max_y) {
			//g.drawString("x",x-2,y+3);//compensate
			if (j!=0) g.drawLine(x_old,y_old,x,y);
		    }
		    x_old=x;
		    y_old=y;
		}
	    }
	}
	
	
	
	
    }
    



  
  

}





