/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/

package Java.gui;

import Java.misc.*;
import javax.swing.*;
import java.util.Hashtable;
import Java.debug.*;



/** Slider to set smoothing parameter in smoothing spline interpolation
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class MyJSlider extends JSlider{
    


    public MyJSlider(double min, double max, double val){
	super(JSlider.HORIZONTAL,0,100,50);
	set(min, max, val);
	setToolTipText("Smoothing parameter in smoothing spline interpolation");
    }

    
    
    
    /** Returns the slider's value. 
     * @param min current min value
     * @param max current max value
     * @return the slider value
     */
  public double getDoubleValue(double min, double max){
    int i=getValue();
    //Debug.print(this,"getDoubleValue","i="+i);
    double d=(new Double(i)).doubleValue()/100.0;
    //Debug.print(this,"getDoubleValue","d="+d);
    double value=min+d*(max-min);
    //Debug.print(this,"getDoubleValue","return="+value);
    if (value<0.0){
	value=0.0;
    }
    return value;
  }
   
    /** Set the slider's min, max and value. 
     * @param min min value
     * @param max max value
     * @param val value
     */
    public void set(double min, double max, double val){
    
	String minS=(new Double(Misc.Round(min,2))).toString();
	String maxS=(new Double(Misc.Round(max,2))).toString();
	int i=(int)(100*(val-min)/(max-min));
	setValue(i);
	
	Hashtable<Integer,JLabel> labelTable = new Hashtable<Integer,JLabel>();
	labelTable.put(0, new JLabel(minS) );
	labelTable.put(100, new JLabel(maxS) );
	setLabelTable( labelTable );
	
	
  }
    



}

