/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org
*/


package Java.misc;

import Java.debug.*;
import java.util.*;


/**
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class Misc{

/**
 * Round to a double to Ndec decimals
 * @param  d the input real value
 * @param  Ndec number of decimals
 * @return the rounded value having Ncec decimals
 */
    public static double Round(double d, int Ndec) {
	double p=(double)Math.pow(10,Ndec);
	d=d*p;
	double t=Math.round(d);
	return (double)t/p;
    }

/**
 * Split an input String at spaces, return array of Strings. If empty line, return null
 * @param  s the input String
 * @param  line_no row number in input file


 * @param  debug debug level, 0=no output, 1=average, 2=much
 * @return an array of Strings
 */
    public static String[] split(String s, int line_no, int debug) { 
	String[] commands =s.split(" ",1000);
	Vector<String> trcommands=new Vector<String>();
	for (int i=0;i<commands.length;i++){
	    //if (debug>1) Debug.print(this,"read","split "+i+": "+commands[i]);
	    commands[i]=commands[i].trim();
	    if (commands[i].length()>0) trcommands.add(commands[i]);
	    //if (debug>1) Debug.print(this,"read","split "+i+": "+commands[i]);
	}
	if (trcommands.size()==0) {
	    //if (debug>0) Debug.print(this,"read","empty line");
	    return null;
	}
	commands =new String[trcommands.size()];
	for (int i=0;i<trcommands.size();i++){
	    commands[i]=trcommands.get(i);
	    if (debug>1) Debug.stat("Problem","split","commands "+i+": "+commands[i]);
	}
	return commands;
    }



    /**
       returns index after underscore in string s
       if s ends with underscore, return -1
    */  
    public static int getInteger(String s, int line_no) { 
	String w[] =s.split("_",2);   
	//Debug.print(this,"read","w="+w[0]);
	//Debug.print(this,"read","w="+w[1]);
	if (w.length==2){
	    try{
		int i = Integer.valueOf(w[1]).intValue(); 
		return i;

	    }
	    catch(NumberFormatException e){ 
		String errorMsg=s + " not of type integer. Line: " + line_no; 
		Debug.print(null,"Problem:getInteger",errorMsg); 
		System.exit(0);
		return -1;
	    }
	}
	else if (w.length==1) {
	    try { 
		int d = Integer.valueOf(s).intValue(); 
		return d; 
	    } 
	    catch(NumberFormatException e){ 
		String errorMsg=s + " not of type integer. Line: " + line_no; 
		Debug.print("Problem","getInteger",errorMsg); 
		System.exit(0);	
		return -1;
	    }
	}
	else{
	    String errorMsg=s + " lacks underscore (followed by a potential integer index). Line: " + line_no; 
	    Debug.error("Problem","getInteger",errorMsg); 
	    System.exit(0);
	    return -1;
	}

    }


    /**
       returns 2 integer indeces after underscore in string s
    */  
    public static int[] getIntegers(String s, int line_no) { 
	String w[] =s.split("_",3);   
	//System.out.println("w="+w[0]+" "+w[1]+" "+w[2]);

	if (w.length==3){
	    try{
		int[] i=new int[2];
		i[0] = Integer.valueOf(w[1]).intValue(); 
		i[1] = Integer.valueOf(w[2]).intValue(); 
		return i;

	    }
	    catch(NumberFormatException e){ 
		String errorMsg=s + " not of type integer. Line: " + line_no; 
		Debug.print(null,"Problem:getInteger",errorMsg); 
		System.exit(0);
		return null;
	    }
	}
	else{
	    String errorMsg=s + " lacks underscore (followed by 2 integer indices). Line: " + line_no; 
	    Debug.error("Problem","getIntegers",errorMsg); 
	    System.exit(0);
	    return null;
	}

    }





    /** Reads a real number from a String in an input file on line line_no
     * @param s input String representing a double value
     * @param line_no current row in input file
     * @return a real number.
     */
    public static double getDouble(String s, int line_no) { 
	s=s.replace("D","E");
	try { 
	    double d = Double.valueOf(s).doubleValue(); 
	    return d; 
	} 
	catch(NumberFormatException e){ 
	    String errorMsg=s + " not of type double. Line: " + line_no; 
	    Debug.print("Misc","getDouble",errorMsg); 
	    System.exit(0);	
	    return 0; 
	} 
    }
















}