/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  This class is partly based on the "funcpars" package by Hendrik Lipka 2001 which is free software under the terms of the GNU General Public
  License.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org

*/

package Java.parser;



import java.util.*;
import java.math.*;
import Java.debug.*;

/** Class supporting parsing of equations
 * @author      Peter Gennemark, www.odeidentification.org
 * @version     1.0                   
 * @since       1.0
 */
public class FunctionParser {


    public static final int ERROR_FORMEVAL=1;

    protected String expr=null;
    protected Vector<PFNField> PFNExpr=new Vector<PFNField>();// Postfix notation 
    protected Vector<PFNField> tokenExpr=new Vector<PFNField>();
    protected Stack<PFNField> tokenStack=new Stack<PFNField>();
    protected int _error=0;

    /**
     *
     */
    public FunctionParser(String function) {
        expr=function;
        tokenize();
        convertToPFN();
    }

 
    protected PFNField getNextMorphem() {
        int pos=0;
        String chars="+-*/^";

        PFNField rf=new PFNField();

        char c=expr.charAt(pos);
        if (Character.isDigit(c)) { // numbers
	    while (true) {
		pos++;
		if (pos>=expr.length()) break;
		if (!(Character.isDigit(expr.charAt(pos)) ||
		      ('.'==expr.charAt(pos))))
		    break;
	    }
	    rf._strValue=expr.substring(0,pos);
	    rf._type=rf.TOK_NUMBER;
	    rf._doubleValue=Double.parseDouble(rf._strValue);
	    pos--;
	}
        else if (Character.isLetter(c)) {
	    while (true) {
		pos++;
		if (pos>=expr.length()) break;
		// 		if (!Character.isLetter(expr.charAt(pos)))
		// 		    break;
		if (Character.isLetter(expr.charAt(pos)) || 
		    Character.isDigit(expr.charAt(pos))) {
		    // first char must be a letter, but 2nd, 3rd etc. can be a number
		}
		else break;
	    }
	    String s=expr.substring(0,pos);
	    rf._strValue=s;
	    //if (s.equalsIgnoreCase("X")||s.equalsIgnoreCase("Y")||s.equalsIgnoreCase("Z")) {
	    if (s.startsWith("X") || s.startsWith("k")) {
		rf._type=rf.TOK_PARAMETER;
	    }
	    else {
		rf._type=rf.TOK_OPERATOR;
	    }
	    pos--;
	}
        else if ('('==c) {
	    rf._strValue="(";
	    rf._type=rf.TOK_OPEN_BRACKET;
	}
        else if (')'==c) {
	    rf._strValue=")";
	    rf._type=rf.TOK_CLOSE_BRACKET;
	}
        else if (-1!=chars.indexOf(c)) {
	    rf._strValue=String.valueOf(c);
	    rf._type=rf.TOK_OPERATOR;
	}
        else {
	    Debug.error(this,"getNextMorphen","unknown token "+c);
	}
        // determine funtion object, if funtion
        if (rf._type==rf.TOK_OPERATOR) {
	    rf._func=determineFunctionForToken(rf._strValue);
	}
	//        System.out.println("token="+rf._strValue);
        expr=expr.substring(pos+1);
        return rf;
    }




    /**
     */
    protected Function determineFunctionForToken(String token)
    {
        if (null==token)
            return null;
        for (int i=0;i<_functions.length;i++)
	    {
		if (token.equalsIgnoreCase(_functions[i].getName()))
		    return _functions[i];
	    }
        return null;
    }

    /**
     *
     */
    protected void tokenize() {
        boolean type=true ;
        PFNField rf=null ;
        do{
	    rf=getNextMorphem() ;
	    if (null==rf) {
		_error=ERROR_FORMEVAL ;
		System.err.println("error: tokenize");
		break;
	    };
	    if (PFNField.TOK_NUMBER==rf._type) {
		tokenExpr.addElement(rf) ;
		type=false ;
	    }
	    else if (PFNField.TOK_PARAMETER==rf._type) {
		tokenExpr.addElement(rf) ;
		type=false ;
	    }
	    else if (PFNField.TOK_OPEN_BRACKET==rf._type) {
		tokenExpr.addElement(rf) ;
		type=true ;
	    }
	    else if (PFNField.TOK_CLOSE_BRACKET==rf._type) {
		tokenExpr.addElement(rf) ;
		type=false ;
	    }
	    else if (PFNField.TOK_OPERATOR==rf._type) {
		if (type&&(rf._strValue.equals("-"))) {
		    //System.out.println("negation!");
		    rf._strValue="~";
		    rf._func=determineFunctionForToken("~");
		}
		tokenExpr.addElement(rf) ;
		type=true;
	    }
	    else if (PFNField.TOK_UNUSED==rf._type) {
		_error=ERROR_FORMEVAL ;
		//System.err.println("error: tokenize");
		Debug.error(this,"tokenize","type unused "+rf._type);
		break ;
	    } ;
	} while (expr.length()>0) ;
    }
    


    /**
     */
    protected void convertToPFN() {
        int aktPos;
        int ii=tokenExpr.size();
        for (aktPos=0;aktPos<ii;aktPos++)
	    {
		PFNField h =(PFNField)tokenExpr.elementAt(aktPos) ;
		PFNField hh=null;
		if (!tokenStack.empty()) {
		    hh=(PFNField)tokenStack.peek() ;
		} ;
		if (null!=h){
		    //System.out.println("h="+h._strValue);
		}
		switch (h._type){
		case PFNField.TOK_OPERATOR :
		    if (null!=hh) {
			//System.out.println("hh="+hh._strValue);
		    }
		    if (tokenStack.empty()) {
			//System.out.println("op1 - push h to stack");
			tokenStack.push(h) ;
		    }
		    else if ( (PFNField.TOK_OPEN_BRACKET==hh._type)|| 
			      (getFuncNum(hh._func)>getFuncNum(h._func))) {
			//System.out.println("op2 - push h to stack");
			tokenStack.push(h) ;
		    }
		    else {
			//System.out.println("op3");
			while (!tokenStack.empty() &&
			       (PFNField.TOK_OPEN_BRACKET!=hh._type) && 
			       (getFuncNum(hh._func)<getFuncNum(h._func))) {
			    //System.out.println("adding to PFN: "+((PFNField)tokenStack.peek())._strValue);
			    PFNExpr.addElement(tokenStack.pop()) ;
			    if (tokenStack.empty()) break;
			    hh=(PFNField)tokenStack.peek();
			}
			//System.out.println("pushing h to stack");
			tokenStack.push(h) ;
		    };
		    break;
		case PFNField.TOK_NUMBER:
		    //System.out.println("num - adding to PFN");
		    PFNExpr.addElement(h) ;
		    break ;
		case PFNField.TOK_PARAMETER:
		    //System.out.println("param - adding to PFN");
		    PFNExpr.addElement(h) ;
		    break ;
		case PFNField.TOK_OPEN_BRACKET:
		    //System.out.println("open - push to stack");
		    tokenStack.push(h) ;
		    break ;
		case PFNField.TOK_CLOSE_BRACKET:
		    //System.out.println("found closing bracket");
		    while (((PFNField)tokenStack.peek())._type!=PFNField.TOK_OPEN_BRACKET)	{
			//System.out.println("adding to PFN"+((PFNField)tokenStack.peek())._strValue);
			PFNExpr.addElement(tokenStack.pop()) ;
			//System.out.println("next="+((PFNField)tokenStack.peek())._strValue);
		    }
		    //System.out.println("popping (");
		    tokenStack.pop() ; // '(' rauswerfen
		    break ;
		}
	    }
        while (!tokenStack.empty()){
	    PFNExpr.addElement(tokenStack.pop()) ;
	}
    }

    /**
     */
    public void DumpPFN() {
        for (int i=0;i<PFNExpr.size();i++) {
	    PFNField rf=(PFNField)PFNExpr.elementAt(i);
	    //System.out.println("#"+i+"="+rf._strValue);
	    Debug.print(this,"DumpPFN","#"+i+"="+rf._strValue);
	}
    }
    
    /**
     */
    protected int getFuncNum(Function func) {
        if (null==func)
            return -1;
        for (int i=0;i<_functions.length;i++) {
	    if (_functions[i]==func)
		return i;
	}
        return -1;
    }

    /**
     *
     */
    protected static abstract class Function {
        public abstract double calculate(Stack stack);
        public abstract String getName();
        public abstract String getSBML();
        public abstract int getArgs();
    }

    /**
     * Supported functions
     * Here, order is significant.
     */
    protected Function[] _functions={
        new Function(){
	    public String getName(){return "sin";} 
	    public String getSBML(){return "<sinus>";} 
	    public int getArgs(){return 1;} 
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		return Math.sin(d1);
	    }
	},
        new Function(){
	    public String getName(){return "~";}
 	    public String getSBML(){return "<initial negation???>";} 
	    public int getArgs(){return 1;}
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		return -d1;
	    }
	},
        new Function(){
	    public String getName(){return "^";} 
	    public String getSBML(){return "<power/>";} 
	    public int getArgs(){return 2;}
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		double d2=((Double)stack.pop()).doubleValue();
		return Math.pow(d1,d2);
	    }
	},
        new Function(){
	    public String getName(){return "*";} 
	    public String getSBML(){return "<times/>";} 
	    public int getArgs(){return 2;}
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		double d2=((Double)stack.pop()).doubleValue();
		return d1*d2;}},
        new Function(){
	    public String getName(){return "/";} 
	    public String getSBML(){return "<divide/>";} 
	    public int getArgs(){return 2;}
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		double d2=((Double)stack.pop()).doubleValue();
		return d2/d1;
	    }
	},
        new Function(){
	    public String getName(){return "+";} 
	    public String getSBML(){return "<plus/>";} 
	    public int getArgs(){return 2;}
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		double d2=((Double)stack.pop()).doubleValue();
		return d1+d2;
	    }
	},
        new Function(){
	    public String getName(){return "-";}
	    public String getSBML(){return "<minus/>";} 
	    public int getArgs(){return 2;}
	    public double calculate(Stack stack) {
		double d1=((Double)stack.pop()).doubleValue();
		double d2=((Double)stack.pop()).doubleValue();
		return d2-d1;
	    }
	}
    };

    /**
     *
     */
    protected class PFNField {
        public static final int TOK_UNUSED=0;
        public static final int TOK_NUMBER=1;
        public static final int TOK_OPERATOR=2;
        public static final int TOK_OPEN_BRACKET=3;
        public static final int TOK_CLOSE_BRACKET=4;
        public static final int TOK_PARAMETER=5;
	
        public int _type=TOK_UNUSED ; // Type of this element
        public Function _func=null; // function for this PFN element
        public String _strValue=null;
        public double _doubleValue ; // numeric value
    }




    public String getSBML() {
	
	index=PFNExpr.size()-1;
	SBMLNode root=new SBMLNode();//make tree
	return root.toString();

    }


    public static int index;

    /**
     *
     */
    protected class SBMLNode {
	public PFNField rf=null;
        public SBMLNode left=null;
        public SBMLNode right=null;
	public SBMLNode(){
	    if (index>=0 && index<PFNExpr.size()){
		rf=(PFNField)PFNExpr.elementAt(index);
		//System.out.println("New node #"+index+"="+rf._strValue);
		Debug.constructor(this,"New node #"+index+"="+rf._strValue);
		index--;
		if (rf._type== PFNField.TOK_OPERATOR){
		    //System.out.println("   #args="+rf._func.getArgs());
		    
		    if (rf._func.getArgs()==1){
			left=new SBMLNode();
		    }
		    else if (rf._func.getArgs()==2){
			right=new SBMLNode();
			//System.exit(0);
			left=new SBMLNode();
		    }
		    else {
			Debug.error(this,"constructor",rf._strValue+" calling _func.getArgs gives "+
				    rf._func.getArgs()+" (illegal)");
		    }
		}
		else {
		    // value or parameter, do nothing
		}
	    
	    }
	    else {

	    }
	}
	public String toString(){
	    String s="";
	    if (rf._type== PFNField.TOK_OPERATOR) {
		s+="<apply>\n";
		s+=rf._func.getSBML()+"\n";
	    }
	    else if (rf._type== PFNField.TOK_PARAMETER) {
		s+="<ci>"+rf._strValue+"</ci>\n";
	    }
	    else if (rf._type== PFNField.TOK_NUMBER) {
		s+="<cn type=\"real\">"+rf._strValue+"</cn>\n";
	    }
	    if (left!=null) s+=left.toString();
	    if (right!=null)s+=right.toString();
	    if (rf._type== PFNField.TOK_OPERATOR) s+="</apply>\n";
	    return s;
	}
	
    
    }






    public static void main(String[] args){

	String s="k1*X1/((k2+X1)*(1+X2/k3))";
	System.out.println("formula="+s);
	FunctionParser fp=new FunctionParser(s);
	fp.DumpPFN();
	System.out.println("Call make tree");
	System.out.println(fp.getSBML());
	
        s="k1*X1^k2*X2^k3";
	System.out.println("formula="+s);
	fp=new FunctionParser(s);
	fp.DumpPFN();
	System.out.println("Call make tree");
	System.out.println(fp.getSBML());
    }

}
