/** 
    ODEidon, a program for structural identification of ODEs
    Copyright (C) 2011 Peter Gennemark, Dag Wedelin

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Contact: Peter Gennemark, Dag Wedelin
             www.odeidentification.org

*/


package Java.problem;

import java.util.*;
import Java.debug.*;
import java.io.*;
import Java.misc.*;

/**
  * Description of an initial bound
  * @author Peter Gennemark
  * @version     1.0                   
  * @since       1.0
  */
public class InitialBound  {
  
    private double lowerBound=-99999999999.0;
    private double upperBound=-99999999999.0;
    
    public InitialBound(){
    }


    /** Read range from input file in form of a buffered reader
     * @param commands vector of strings on first line to read
     * @param in buffered reader
     * @param line_no current line in the buffered reader (i.e., problem file)
     * @param debug debug level
     * @param variables vector of variables
     * @param experiments vector of experiments
     */
    public int read(String[] commands,
		    BufferedReader in,
		    int line_no,
		    int debug,
		    Vector<Variable> variables,
		    Vector<Experiment> experiments){
	
	int ivar = Misc.getInteger(commands[0],line_no);
	if (ivar<=0) {
	    Debug.error(this,"read","variable_"+ivar+" should be positive on line="+line_no);
	    System.exit(0);
	}
	if (ivar>variables.size()) {
	    Debug.error(this,"read","Illegal id of variable_"+ivar+" (too large), on line="+line_no);
	    System.exit(0);
	}
	if (!commands[2].startsWith("experiment_")){
	    Debug.error(this,"read","Unexpected command: "+commands[2]+", on line="+line_no);
	    System.exit(0); 
	}
	int iexp = Misc.getInteger(commands[2],line_no);
	if (iexp<=0) {
	    Debug.error(this,"read","experiment_"+iexp+" should be positive on line="+line_no);
	    System.exit(0);
	}
	if (iexp>experiments.size()) {
	    Debug.error(this,"read","Illegal id of experiment_"+iexp+" (too large), on line="+line_no);
	    System.exit(0);
	}
	try {
	    for (;;) {
		String s= in.readLine();
		line_no++;
		commands = Misc.split(s,line_no,debug);
		if (commands==null){
		    if (debug>0) Debug.print(this,"read","empty line");
		    continue;
		}
		else if (commands[0].equals("has") && commands[1].equals("lowerInitialBound") 
			 && commands[2].equals("=")){
		    lowerBound=Misc.getDouble(commands[3],line_no);
		}
		else if (commands[0].equals("has") && commands[1].equals("upperInitialBound") 
			 && commands[2].equals("=")){
		    upperBound=Misc.getDouble(commands[3],line_no);
		}
		else {
		    Debug.error(this,"read","Unexpected command: "+commands[0]+", on line="+line_no);
		    System.exit(0); 
		}

		if (lowerBound>-99999999999.0 && upperBound>-99999999999.0) break;

	    }
	}
	catch (IOException ioe) {
	    Debug.error(this,"read","IOExpection on line="+line_no);
	    System.exit(0); 
	}

	// add to experiment
	experiments.get(iexp-1).addInitialBound(this,variables.get(ivar-1));


	return line_no;

    }
   
    /** Get lower bound
     * @return the bound
     */ 
    public double getLowerBound(){
	return lowerBound;
    }
   
    /** Get upper bound
     * @return the bound
     */ 
    public double getUpperBound(){
	return upperBound;
    }

    /** Get string representation
     * @return the string
     */ 
    public String toString(){
	String s="InitialBounds = ["+lowerBound+", "+upperBound+"]\n";
	return s;
    }

  
    // public static void main(String[] argv) {

    // }
  
}
