# Run a set of problems defined in the vector @problems
# Each problem is run $ITER times, where $ITER is a constant set below.
# For each run i=1...ITER of a problem, the seed to the random number generator is set to i.

$DEBUG=1; # 0=none, 1=average, 2=much
$ITER=2; # Number of iterations with different random seed for each problem

@problems  = ("simpleLin1","pe_3genes1");

$size=@problems;
if ($DEBUG>0) {print "Consider ".$size." problem(s)\n";}

$mydir="PROBLEMS/";

# For all problems
for ($ip=0;$ip<$size;$ip++){

    if ($DEBUG>0) {print $problems[$ip]."\n";}

    # Create a directory to save output files in
    $outfile="OUTPUT/RES_".$problems[$ip]."/";
    $s="mkdir ".$outfile;
    if (-d $outfile){print $outfile." exists\n"}
    else {
	system($s);
	print("Creates the directory ".$outfile."\n");
    }

    # For all iterations with different random seed.
    for ($i=0;$i<$ITER;$i++){

	if ($i > 0){
	    # Create input file with correct randomSeed
	    $old="randomSeed = ".$i;
	    $new="randomSeed = ".($i+1);
	    $s="sed 's/".$old."/".$new."/g' ".$mydir."temp_in > TMPFILE && mv TMPFILE ".$mydir."temp_in";
	    system($s);
	    if ($DEBUG>1) {print "Replace ".$old." by ".$new."\n";}
	}	   
	else {
	    # Create temp_in (problem file merge with randomSeed)
	    $s="cat ".$mydir.$problems[$ip]." ".$mydir."SETTING"." > ".$mydir."temp_in";
	    system($s);
	}
	
	# Generate Fortran program
	$s="java Java/GenerateProgram ".$mydir."temp_in";
	if ($DEBUG>0) {print("Generate.");}
	system($s);
	if ($DEBUG>0) {print(" OK.");}

	# Compile program
	$s="gfortran -o F_program -w F_generatedCode/*.f";
	if ($DEBUG>0) {print(" Compile.");}
	system($s);
	if ($DEBUG>0) {print(" OK.");}

	# Run problem
	$outfile="OUTPUT/RES_".$problems[$ip]."/RES_".$problems[$ip]."_".($i+1);
	$s="./F_program < ".$mydir."temp_in > ".$outfile;
	if ($DEBUG>0) {print(" Run.");}
	system($s);
	if ($DEBUG>0) {print(" OK\n");}
	system("tail -2 ".$outfile);
	#print "\n";
    }

    # reset input file
    $old="randomSeed = ".$ITER;
    $new="randomSeed = 1";
    
    $s="sed 's/".$old."/".$new."/g' ".$mydir."SETTING > TMPFILE && mv TMPFILE ".$mydir."SETTING";
    system($s);
    if ($DEBUG>1) {print $problems[$ip].", replace ".$old." by ".$new."\n";}

    # Move output files to subdirectories OUTPUT/RES_X_X
    $outfile="OUTPUT/RES_".$problems[$ip]."/";
    $s="mv OUT_".$problems[$ip]."* ".$outfile;
    system($s);
}
